/*****************************************************************************
*                                                                            *
*       DISPLAY.C                                                            *
*       Affichage des resultats                                              *
*                                                                            *
*       Laurent de Soras                                                     *
*       Eric Lalitte                                                         *
*       A1B7E                                                                *
*                                                                            *
* Tab = 3                                                                    *
*****************************************************************************/



/*\\\ FICHIERS INCLUDE \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*/

#include <stdio.h>
#include <stdlib.h>
#include <stdarg.h>
#include	<math.h>
#include <string.h>
#include	<ctype.h>
#include	<limits.h>

#include "param.h"
#include	"solver.h"
#include "display.h"
#include "main.h"



/*\\\ CONSTANTES ET MACROS PRIVEES \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*/

/* Nombre maximum de colonnes sur une ligne */
#define	DISP_MAX_LINE_LEN		255

enum
{
	DISP_TYPE_SECTION,
	DISP_TYPE_TEACHER,
	DISP_TYPE_ROOM
};



/*\\\ TYPES & STRUCTURES PRIVEES \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*/

typedef struct
{
	int		planning;
	int		first_section;
	int		last_section;
	int		first_period;
	int		last_period;
	int		first_room;
	int		last_room;
	int		first_teacher;
	int		last_teacher;
} DISP_INFO;



/*\\\ PROTOTYPES DES FONCTIONS PRIVEES \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*/

void	DISP_display_period (int period);
void	DISP_display_teacher (int teacher);
void	DISP_display_room (int room);
void	DISP_scan_used_teachers (void);
void	DISP_scan_used_rooms (void);
void	DISP_display_week_planning (PARM_PLANNING *planning_ptr [7] [2], const char *header_line_0, int disp_type);
void	DISP_string_copy (char *dest_0, const char *source_0, size_t length);
void	DISP_output_html (void);
FILE	*DISP_open_html_file (const char *filename_0, const char *title_0, long bgcolor, long txcolor);
void	DISP_close_html_file (FILE *file_ptr);
void	DISP_display_week_planning_html (FILE *file_ptr, PARM_PLANNING *planning_ptr [7] [2], const char *header_line_0, int disp_type);



/*\\\ DEFINITION DES VARIABLES EXTERNES \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*/

/* Taille - 1 d'une ligne d'affichage */
int	DISP_line_len = 79;



/*\\\ VARIABLES PRIVEES \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*/

long	**DISP_teacher_time_ptr_ptr;
long	**DISP_room_time_ptr_ptr;



/*\\\ FONCTIONS \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*/

int	DISP_display (int line_width)
{
	int		section;
	int		teacher;
	int		room;
	int		period;
	int		choice;
	int		choice_2;
	int		ret_value;
	char		choice_0 [99+1];

	DISP_line_len = line_width;

	/* Regarde quelles sont les salles et profs qui ont ete utilises,
		et pendant combien de temps */
	DISP_scan_used_teachers ();
	DISP_scan_used_rooms ();

	choice = 0;
	while (choice != 6 && choice != 7)
	{
		printf ("\n\n\nPLANNING %04X%04X.\n\n\n"
			"1. Planning par section\n"
			"2. Planning par professeur\n"
			"3. Planning par salle\n"
			"4. Tous les plannings\n"
			"5. Sortie des resultats en HTML\n"
			"6. Chercher un autre planning\n"
			"7. Quitter le programme\n\n\? ",
			(int) (SOLV_planning_key >> 16), (int) SOLV_planning_key);
		scanf ("%99s", choice_0);
		choice = atoi (choice_0);
		
		switch (choice)
		{
		case	1:
			choice_2 = -1;
			while (choice_2 != MAIN_nbr_periods + 2)
			{
				printf ("\n\n\n");
				for (period = 0; period < MAIN_nbr_periods; period ++)
				{
					printf ("%d. %s - Periode %s\n", period + 1,
					        MAIN_section_ptr [MAIN_period_ptr [period].section].name_0,
					        MAIN_period_ptr [period].name_0);
				}
				printf ("%d. Toutes les sections\n", MAIN_nbr_periods + 1);
				printf ("%d. Revenir au menu precedent\n\n\? ", MAIN_nbr_periods + 2);
				scanf ("%99s", choice_0);
				choice_2 = atoi (choice_0);

				if (choice_2 == MAIN_nbr_periods + 1)
				{
					for (period = 0; period < MAIN_nbr_periods; period ++)
					{
						DISP_display_period (period);
					}
				}
				else if (choice_2 >= 1 && choice_2 <= MAIN_nbr_periods)
				{
					DISP_display_period (choice_2 - 1);
				}
			}
			break;

		case	2:
			choice_2 = -1;
			while (choice_2 != MAIN_nbr_teachers + 2)
			{
				printf ("\n\n\n");
				for (teacher = 0; teacher < MAIN_nbr_teachers; teacher ++)
				{
					printf ("%d. %s\n",
					        teacher + 1, MAIN_teacher_ptr [teacher].name_0);
				}
				printf ("%d. Tous les professeurs\n", MAIN_nbr_teachers + 1);
				printf ("%d. Revenir au menu precedent\n\n\? ", MAIN_nbr_teachers + 2);
				scanf ("%99s", choice_0);
				choice_2 = atoi (choice_0);

				if (choice_2 == MAIN_nbr_teachers + 1)
				{
					for (teacher = 0; teacher < MAIN_nbr_teachers; teacher ++)
					{
						DISP_display_teacher (teacher);
					}
				}
				else if (choice_2 >= 1 && choice_2 <= MAIN_nbr_teachers)
				{
					DISP_display_teacher (choice_2 - 1);
				}
			}
			break;

		case	3:
			choice_2 = -1;
			while (choice_2 != MAIN_nbr_rooms + 2)
			{
				printf ("\n\n\n");
				for (room = 0; room < MAIN_nbr_rooms; room ++)
				{
					printf ("%d. %s (%s)\n", room + 1,
					        MAIN_room_ptr [room].name_0,
					        MAIN_building_ptr [MAIN_room_ptr [room].building].name_0);
				}
				printf ("%d. Toutes les salles\n", MAIN_nbr_rooms + 1);
				printf ("%d. Revenir au menu precedent\n\n\? ", MAIN_nbr_rooms + 2);
				scanf ("%99s", choice_0);
				choice_2 = atoi (choice_0);

				if (choice_2 == MAIN_nbr_rooms + 1)
				{
					for (room = 0; room < MAIN_nbr_rooms; room ++)
					{
						DISP_display_room (room);
					}
				}
				else if (choice_2 >= 1 && choice_2 <= MAIN_nbr_rooms)
				{
					DISP_display_room (choice_2 - 1);
				}
			}
			break;

		case	4:
			for (section = 0; section < MAIN_nbr_sections; section ++)
			{
				for (period = 0; period < MAIN_nbr_periods; period ++)
				{
					if (MAIN_period_ptr [period].section != section)
					{
						continue;
					}
					DISP_display_period (period);
				}
			}
			for (teacher = 0; teacher < MAIN_nbr_teachers; teacher ++)
			{
				DISP_display_teacher (teacher);
			}
			for (room = 0; room < MAIN_nbr_rooms; room ++)
			{
				DISP_display_room (room);
			}
			break;

		case	5:
			DISP_output_html ();
			break;

		case	6:
			ret_value = FALSE;
			break;

		case	7:
			ret_value = TRUE;
			break;

		default:
			printf ("Essaie encore une fois...\n");
		}
	}

	/* Libere les tableaux reserves */
	for (teacher = 0; teacher < MAIN_nbr_teachers; teacher ++)
	{
		free (DISP_teacher_time_ptr_ptr [teacher]);
	}
	free (DISP_teacher_time_ptr_ptr);

	for (room = 0; room < MAIN_nbr_rooms; room ++)
	{
		free (DISP_room_time_ptr_ptr [room]);
	}
	free (DISP_room_time_ptr_ptr);
	
	return (ret_value);
}



void	DISP_display_period (int period)
{
	int		section;
	int		period_len;
	int		start_week;
	int		week;
	int		day_1;
	int		day_2;
	int		month_1;
	int		month_2;
	int		year_1;
	int		year_2;
	char		header_line_0 [DISP_MAX_LINE_LEN+1];

	section = MAIN_period_ptr [period].section;
	start_week = (int) MAIN_period_ptr [period].start_week;
	period_len = MAIN_period_ptr [period].duration;

	for (week = 0; week < period_len; week ++)
	{
		MAIN_get_date (&day_1, &month_1, &year_1,
		               MAIN_first_day, MAIN_first_week + start_week + week);
		MAIN_get_date (&day_2, &month_2, &year_2,
		               MAIN_last_day, MAIN_first_week + start_week + week);

		sprintf (header_line_0,
		         "Section %s, periode %s, semaine du %d/%d/%d au %d/%d/%d.",
		         MAIN_section_ptr [section].name_0,
		         MAIN_period_ptr [period].name_0,
		         day_1 + 1, month_1 + 1, year_1,
		         day_2 + 1, month_2 + 1, year_2);

		DISP_display_week_planning (MAIN_section_ptr [section].planning [start_week + week],
					    header_line_0, DISP_TYPE_SECTION);
	}
}



void	DISP_display_teacher (int teacher)
{
	int		week;
	int		day_1;
	int		day_2;
	int		month_1;
	int		month_2;
	int		year_1;
	int		year_2;
	char		header_line_0 [DISP_MAX_LINE_LEN+1];

	for (week = 0; week <= (int) (MAIN_last_week - MAIN_first_week); week ++)
	{
		/* Si le prof ne travaille pas cette semaine, on passe a la suite */
		if (DISP_teacher_time_ptr_ptr [teacher] [week] <= 0)
		{
			continue;
		}

		MAIN_get_date (&day_1, &month_1, &year_1,
		               MAIN_first_day, MAIN_first_week + week);
		MAIN_get_date (&day_2, &month_2, &year_2,
		               MAIN_last_day, MAIN_first_week + week);

		sprintf (header_line_0,
		         "Professeur %s, semaine du %d/%d/%d au %d/%d/%d.",
		         MAIN_teacher_ptr [teacher].name_0,
		         day_1 + 1, month_1 + 1, year_1,
		         day_2 + 1, month_2 + 1, year_2);

		DISP_display_week_planning (MAIN_teacher_ptr [teacher].planning [week],
		                            header_line_0, DISP_TYPE_TEACHER);
	}
}



void	DISP_display_room (int room)
{
	int		week;
	int		day_1;
	int		day_2;
	int		month_1;
	int		month_2;
	int		year_1;
	int		year_2;
	char		header_line_0 [DISP_MAX_LINE_LEN+1];

	for (week = 0; week <= MAIN_last_week - MAIN_first_week; week ++)
	{
		/* Si la salle n'est pas utilisee cette semaine, on passe a la suite */
		if (DISP_room_time_ptr_ptr [room] [week] <= 0)
		{
			continue;
		}

		MAIN_get_date (&day_1, &month_1, &year_1,
		               MAIN_first_day, MAIN_first_week + week);
		MAIN_get_date (&day_2, &month_2, &year_2,
		               MAIN_last_day, MAIN_first_week + week);

		sprintf (header_line_0,
		         "Salle %s, semaine du %d/%d/%d au %d/%d/%d.",
		         MAIN_room_ptr [room].name_0,
		         day_1 + 1, month_1 + 1, year_1,
		         day_2 + 1, month_2 + 1, year_2);

		DISP_display_week_planning (MAIN_room_ptr [room].planning [week],
		                            header_line_0, DISP_TYPE_ROOM);
	}
}



void	DISP_scan_used_teachers (void)
{
	int		nbr_weeks;
	int		count;
	int		week;
	int		day;
	int		half;
	long		*teacher_time_ptr;
	PARM_PLANNING	*planning_ptr;

	DISP_teacher_time_ptr_ptr = (long **) malloc (  MAIN_nbr_teachers
																 * sizeof (*DISP_teacher_time_ptr_ptr));
	if (DISP_teacher_time_ptr_ptr == NULL)
	{
		fprintf (stderr, "Error: malloc failed.\n");
		exit (-1);
	}

	nbr_weeks = (int) (MAIN_last_week - MAIN_first_week + 1);
	for (count = 0; count < MAIN_nbr_teachers; count ++)
	{
		teacher_time_ptr = (long *) calloc (nbr_weeks,
		                                    sizeof (*teacher_time_ptr));
		if (teacher_time_ptr == NULL)
		{
			fprintf (stderr, "Error: malloc failed.\n");
			exit (-1);
		}
		DISP_teacher_time_ptr_ptr [count] = teacher_time_ptr;
	}

	for (count = 0; count < MAIN_nbr_teachers; count ++)
	{
		teacher_time_ptr = DISP_teacher_time_ptr_ptr [count];
		for (week = 0; week < nbr_weeks; week ++)
		{
			for (day = 0; day < 7; day ++)
			{
				for (half = 0; half < 2; half ++)
				{
					planning_ptr = MAIN_teacher_ptr [count].planning [week] [day] [half];
					while (planning_ptr != NULL)
					{
						/* Saute les jours feries mais pas les demi-journees
							reservees */
						if (planning_ptr->start < 0)
						{
							break;
						}
						teacher_time_ptr [week] += (long)planning_ptr->duration;
						planning_ptr = planning_ptr->next_ptr;
					}
				}
			}
		}
	}
}



void	DISP_scan_used_rooms (void)
{
	int		nbr_weeks;
	int		count;
	int		week;
	int		day;
	int		half;
	long		*room_time_ptr;
	PARM_PLANNING	*planning_ptr;

	DISP_room_time_ptr_ptr = (long **) malloc (  MAIN_nbr_rooms
															 * sizeof (*DISP_room_time_ptr_ptr));
	if (DISP_room_time_ptr_ptr == NULL)
	{
		fprintf (stderr, "Error: malloc failed.\n");
		exit (-1);
	}

	nbr_weeks = (int) (MAIN_last_week - MAIN_first_week + 1);
	for (count = 0; count < MAIN_nbr_rooms; count ++)
	{
		room_time_ptr = (long *) calloc (nbr_weeks, sizeof (*room_time_ptr));
		if (room_time_ptr == NULL)
		{
			fprintf (stderr, "Error: malloc failed.\n");
			exit (-1);
		}
		DISP_room_time_ptr_ptr [count] = room_time_ptr;
	}

	for (count = 0; count < MAIN_nbr_rooms; count ++)
	{
		room_time_ptr = DISP_room_time_ptr_ptr [count];
		for (week = 0; week < nbr_weeks; week ++)
		{
			for (day = 0; day < 7; day ++)
			{
				for (half = 0; half < 2; half ++)
				{
					planning_ptr = MAIN_room_ptr [count].planning [week] [day] [half];
					while (planning_ptr != NULL)
					{
						/* Saute les jours feries mais pas les demi-journees
							reservees */
						if (planning_ptr->start < 0)
						{
							break;
						}
						room_time_ptr [week] += (long)planning_ptr->duration;
						planning_ptr = planning_ptr->next_ptr;
					}
				}
			}
		}
	}
}






void	DISP_display_week_planning (PARM_PLANNING *planning_ptr [7] [2], const char *header_line_0, int disp_type)
{
	int		minute_1;
	int		minute_2;
	int		hour_1;
	int		hour_2;
	int		count;
	int		day;
	int		half;
	int		offset;
	int		type;
	int		type_2;
	int		subject;
	int		teacher;
	int		section;
	int		room;
	int		nbr_subdiv;
	int		subdiv;
	int		holiday;
	int		field_offset [6];
	int		field_len [6];
	int		field;
	int		half_day;
	PARM_PLANNING	*half_day_planning_ptr;
	const char	*type_name_0 [2] [3] =
	{
		{ "", "TD de ", "TP de " },
		{ "", "TD d\'", "TP d\'" }
	};
	const char	*day_name_0 [7] =
	{
		"Lundi",
		"Mardi",
		"Mercredi",
		"Jeudi",
		"Vendredi",
		"Samedi",
		"Dimanche"
	};
	char		line_0 [DISP_MAX_LINE_LEN+1];
	char		field_0 [DISP_MAX_LINE_LEN+1];

	/* Calcule les differentes longueurs et positions des champs */
	field_offset [0] = 0;
	field_len [0] = 13;
	field_offset [1] = field_offset [0] + field_len [0];
	field_len [5] = 6;
	field_len [1] = (DISP_line_len - field_len [5] - field_offset [1]) / 4;
	field_offset [2] = field_offset [1] + field_len [1];
	field_len [2] = (DISP_line_len - field_len [5] - field_offset [2]) / 3;
	field_offset [3] = field_offset [2] + field_len [2];
	field_len [3] = (DISP_line_len - field_len [5] - field_offset [3]) / 2;
	field_offset [4] = field_offset [3] + field_len [3];
	field_len [4] = DISP_line_len - field_len [5] - field_offset [4];
	field_offset [5] = field_offset [4] + field_len [4];

	/* Ligne de titre, centree */
	offset = (DISP_line_len - strlen (header_line_0)) / 2;
	for (count = 0; count < offset; count ++)
	{
		fprintf (stdout, " ");
	}
	fprintf (stdout, "%s\n", header_line_0);

	/* Ligne de separation + saut de ligne */
	for (count = 0; count < DISP_line_len; count ++)
	{
		fprintf (stdout, "-");
	}
	fprintf (stdout, "\n\n");

	/* Pour chaque jour de la semaine */
	for (day = MAIN_first_day; day <= MAIN_last_day; day ++)
	{
		/* S'il n'y a rien dans cette journee, on la saute */
		if (   planning_ptr [day] [0] == NULL
			 && planning_ptr [day] [1] == NULL)
		{
			continue;
		}

		/* Ecrit le jour au milieu de la ligne puis saute une ligne */
		offset = (DISP_line_len - strlen (day_name_0 [day])) / 2;
		for (count = 0; count < offset; count ++)
		{
			fprintf (stdout, " ");
		}
		fprintf (stdout, "%s\n\n", day_name_0 [day]);

		/* On met les 2 1/2 journees dans le meme panier */
		for (half = 0; half < 2; half ++)
		{
			half_day_planning_ptr = planning_ptr [day] [half];

			/* Scanne chaque horaire (ils sont deja classes dans
				l'ordre chronologique) */
			while (half_day_planning_ptr != NULL)
			{
				/* Efface la ligne */
				memset (line_0, ' ', DISP_line_len);
				line_0 [DISP_line_len] = '\0';

				/* Calcule et affiche les heures de debut et de fin du cours */
				minute_1 = half_day_planning_ptr->start;
				if (minute_1 >= 0)
				{
					minute_1 +=   (half == 0)
									? MAIN_am_start_h * 60 + MAIN_am_start_m
									: MAIN_pm_start_h * 60 + MAIN_pm_start_m;
					minute_2 = minute_1 + half_day_planning_ptr->duration;
					hour_1 = minute_1 / 60;
					hour_2 = minute_2 / 60;
					minute_1 %= 60;
					minute_2 %= 60;
					sprintf (field_0, "%02d:%02d - %02d:%02d",
								hour_1, minute_1, hour_2, minute_2);
					DISP_string_copy (line_0 + field_offset [0], field_0, field_len [0]);
				}

				/* Horaire negatif, c'est un jour ferie */
				else
				{
					holiday = half_day_planning_ptr->subdiv;
					fprintf (stdout, "Ferie: %s\n",
								MAIN_holiday_ptr [holiday].name_0);
					half = 1;
					break;
				}

				/* Type du cours */
				type = half_day_planning_ptr->type;
				if (type >= 0)
				{
					/* Type et matiere */
					subject = half_day_planning_ptr->subject;
					type_2 = 0;
					switch (tolower (*MAIN_subject_ptr [subject].name_0))
					{
					case	'a':
					case	'e':
					case	'i':
					case	'o':
					case	'u':
					case	'y':
						type_2 = 1;
						break;
					}
					sprintf (field_0, "   %s%s",
								type_name_0 [type_2] [type],
								MAIN_subject_ptr [subject].name_0);
					DISP_string_copy (line_0 + field_offset [1], field_0,
											field_len [1] + field_len [2]);

					/* Groupe */
					nbr_subdiv = half_day_planning_ptr->subdiv_tot;
					if (nbr_subdiv > 1)
					{
						subdiv = half_day_planning_ptr->subdiv;
						sprintf (field_0, " Groupe %d/%d", subdiv + 1, nbr_subdiv);
						DISP_string_copy (line_0 + field_offset [3], field_0, field_len [3]);
					}

					field = 4;

					/* Prof */
					if (disp_type != DISP_TYPE_TEACHER)
					{
						teacher = half_day_planning_ptr->teacher;
						sprintf (field_0, "%s", MAIN_teacher_ptr [teacher].name_0);
						DISP_string_copy (line_0 + field_offset [field], field_0, field_len [field]);
						field ++;
					}

					/* Section */
					if (disp_type != DISP_TYPE_SECTION)
					{
						section = half_day_planning_ptr->section;
						sprintf (field_0, "%s", MAIN_section_ptr [section].name_0);
						DISP_string_copy (line_0 + field_offset [field], field_0, field_len [field]);
						field ++;
					}

					/* Salle */
					if (disp_type != DISP_TYPE_ROOM)
					{
						room = half_day_planning_ptr->room;
						sprintf (field_0, "%s", MAIN_room_ptr [room].name_0);
						DISP_string_copy (line_0 + field_offset [field], field_0, field_len [field]);
						field ++;
					}

					fprintf (stdout, "%s\n", line_0);
				}

				/* C'est une demi-journee reservee */
				else
				{
					/* Nom de cette demi-journee */
					half_day = half_day_planning_ptr->subdiv;
					sprintf (field_0, "   Reserve %s ",
								MAIN_half_day_ptr [half_day].name_0);

					/* Nom de la section qui en profite */
					if (disp_type != DISP_TYPE_SECTION)
					{
						section = half_day_planning_ptr->section;
						sprintf (field_0 + strlen (field_0),
									"%s ", MAIN_section_ptr [section].name_0);
					}

					/* Recherche toutes les salles qui y sont associees */
					strcat (field_0, "  ");
					for ( ; ; )
					{
						room = half_day_planning_ptr->room;
						if (room >= 0)
						{
							if (disp_type != DISP_TYPE_ROOM)
							{
									sprintf (field_0 + strlen (field_0),
												"%s", MAIN_room_ptr [room].name_0);
							}
						}
						else
						{
							break;
						}

						half_day_planning_ptr = half_day_planning_ptr->next_ptr;
						if (half_day_planning_ptr == NULL)
						{
							break;
						}
						strcat (field_0, ", ");
					}

					DISP_string_copy (line_0 + field_offset [1], field_0,
											DISP_line_len - field_offset [1]);
					fprintf (stdout, "%s\n", line_0);
					break;
				}

				/* Cours suivant */
				half_day_planning_ptr = half_day_planning_ptr->next_ptr;
			}

			/* Saute une ligne entre les 2 1/2 journees */
			if (half < 1)
			{
				fprintf (stdout, "\n");
			}
		}

		/* Saute 1+2 lignes entre 2 jours */
		fprintf (stdout, "\n\n");
	}

	/* Double ligne de separation et saute 5 lignes */
	for (count = 0; count < DISP_line_len; count ++)
	{
		fprintf (stdout, "=");
	}
	fprintf (stdout, "\n\n\n\n\n\n");
}



void	DISP_string_copy (char *dest_0, const char *source_0, size_t length)
{
	char		c;

	for ( ; length > 0; length --)
	{
		c = *source_0++;
		if (c == '\0')
		{
			break;
		}
		*dest_0++ = c;
	}
}



void	DISP_output_html (void)
{
	int		count;
	int		period;
	int		week;
	int		start_week;
	int		period_len;
	int		day_1;
	int		day_2;
	int		month_1;
	int		month_2;
	int		year_1;
	int		year_2;
	int		building;
	FILE		*index_file_ptr;
	FILE		*plan_file_ptr;
	char		filename_0 [255+1];
	char		title_0 [255+1];
	char		header_line_0 [255+1];

	/* Creation du fichier d'index */
	index_file_ptr = DISP_open_html_file ("index.html", "Index des emplois du temps", 0x000000, 0xFFFFFF);

/*______________________________________________
 *
 * Les sections
 *______________________________________________
 */

	fprintf (index_file_ptr,
	         "<TABLE CELLSPACING=0 CELLPADDING=0 BORDER=1 BGCOLOR=\"#FF0000\" WIDTH=\"100%%\">\n"
	         "   <TR>\n"
	         "      <TD><DT><CENTER><B><FONT COLOR=\"#FFFF00\" SIZE=+3>PLANNINGS PAR SECTIONS</FONT></B><CENTER></DT></TD>\n"
	         "   </TR>\n"
	         "</TABLE>\n");

	for (count = 0; count < MAIN_nbr_sections; count ++)
	{
		fprintf (index_file_ptr,
		         "   <CENTER><H2>%s</H2></CENTER>\n"
		         "      <CENTER><P>\n",
			 MAIN_section_ptr [count].name_0);

		for (period = 0; period < MAIN_nbr_periods; period ++)
		{
			if (MAIN_period_ptr [period].section != count)
			{
				continue;
			}

			sprintf (filename_0, "s%d_p%d.html", count, period);

			/* Lien dans l'index */
			fprintf (index_file_ptr,
						"      <A HREF=\"%s\">Periode %s</A><BR>\n",
						filename_0, MAIN_period_ptr [period].name_0);

			/* Creation du fichier */
			sprintf (title_0, "Emploi du temps des %s, periode %s",
						MAIN_section_ptr [count].name_0,
						MAIN_period_ptr [period].name_0);
			plan_file_ptr = DISP_open_html_file (filename_0, title_0, 0xFFFFFF, 0x000000);
			printf ("Writing %s... ", filename_0);
			fflush (stdout);

			fprintf (plan_file_ptr,
			         "<TABLE CELLSPACING=0 CELLPADDING=0 BORDER=1 BGCOLOR=\"#FF0000\" WIDTH=\"100%%\">\n"
			         "   <TR>\n"
			         "      <TD><DT><CENTER><B><FONT COLOR=\"#FFFF00\" SIZE=+3>%s</FONT></B><CENTER></DT></TD>\n"
			         "   </TR>\n"
			         "</TABLE>\n", title_0);

			start_week = (int) MAIN_period_ptr [period].start_week;
			period_len = MAIN_period_ptr [period].duration;

			for (week = 0; week < period_len; week ++)
			{
				MAIN_get_date (&day_1, &month_1, &year_1,
				               MAIN_first_day, MAIN_first_week + start_week + week);
				MAIN_get_date (&day_2, &month_2, &year_2,
				               MAIN_last_day, MAIN_first_week + start_week + week);

				sprintf (header_line_0, "Semaine du %d/%d/%d au %d/%d/%d.",
							day_1 + 1, month_1 + 1, year_1,
							day_2 + 1, month_2 + 1, year_2);

				DISP_display_week_planning_html (plan_file_ptr,
															MAIN_section_ptr [count].planning [start_week + week],
															header_line_0, DISP_TYPE_SECTION);
			}

			/* Fermeture du fichier de la section/periode */
			DISP_close_html_file (plan_file_ptr);
			printf ("Done.\n");
		}

		fprintf (index_file_ptr, "      </P><CENTER>\n");
	}
	
	fprintf (index_file_ptr, "<P>\n<HR WIDTH=\"100%%\">\n");

/*______________________________________________
 *
 * Les professeurs
 *______________________________________________
 */

	fprintf (index_file_ptr,
	         "<TABLE CELLSPACING=0 CELLPADDING=0 BORDER=1 BGCOLOR=\"#FF0000\" WIDTH=\"100%%\">\n"
	         "   <TR>\n"
	         "      <TD><DT><CENTER><B><FONT COLOR=\"#FFFF00\" SIZE=+3>PLANNINGS PAR PROFESSEURS</FONT></B><CENTER></DT></TD>\n"
	         "   </TR>\n"
	         "</TABLE>\n");

	fprintf (index_file_ptr, "      <CENTER><P>\n");

	for (count = 0; count < MAIN_nbr_teachers; count ++)
	{
		sprintf (filename_0, "p%d.html", count);

		/* Lien dans l'index */
		fprintf (index_file_ptr,
			 "      <A HREF=\"%s\">%s</A><BR>\n",
			 filename_0, MAIN_teacher_ptr [count].name_0);

		/* Creation du fichier */
		sprintf (title_0, "Emploi du temps de %s",
					MAIN_teacher_ptr [count].name_0);
		plan_file_ptr = DISP_open_html_file (filename_0, title_0, 0xFFFFFF, 0x000000);
		printf ("Writing %s... ", filename_0);
		fflush (stdout);

		fprintf (plan_file_ptr,
		         "<TABLE CELLSPACING=0 CELLPADDING=0 BORDER=1 BGCOLOR=\"#FF0000\" WIDTH=\"100%%\">\n"
		         "   <TR>\n"
		         "      <TD><DT><CENTER><B><FONT COLOR=\"#FFFF00\" SIZE=+3>%s</FONT></B><CENTER></DT></TD>\n"
		         "   </TR>\n"
		         "</TABLE>\n", title_0);

		for (week = 0; week <= (int) (MAIN_last_week - MAIN_first_week); week ++)
		{
			/* Si le prof ne travaille pas cette semaine, on passe a la suite */
			if (DISP_teacher_time_ptr_ptr [count] [week] <= 0)
			{
				continue;
			}

			MAIN_get_date (&day_1, &month_1, &year_1,
								MAIN_first_day, MAIN_first_week + week);
			MAIN_get_date (&day_2, &month_2, &year_2,
								MAIN_last_day, MAIN_first_week + week);

			sprintf (header_line_0, "Semaine du %d/%d/%d au %d/%d/%d.",
						day_1 + 1, month_1 + 1, year_1,
						day_2 + 1, month_2 + 1, year_2);

			DISP_display_week_planning_html (plan_file_ptr,
														MAIN_teacher_ptr [count].planning [week],
														header_line_0, DISP_TYPE_TEACHER);
		}

		/* Fermeture du fichier du professeur */
		DISP_close_html_file (plan_file_ptr);
		printf ("Done.\n");
	}

	fprintf (index_file_ptr, "      </P><CENTER>\n");
	fprintf (index_file_ptr, "<P>\n<HR WIDTH=\"100%%\">\n");

/*______________________________________________
 *
 * Les salles
 *______________________________________________
 */

	fprintf (index_file_ptr,
	         "<TABLE CELLSPACING=0 CELLPADDING=0 BORDER=1 BGCOLOR=\"#FF0000\" WIDTH=\"100%%\">\n"
	         "   <TR>\n"
	         "      <TD><DT><CENTER><B><FONT COLOR=\"#FFFF00\" SIZE=+3>PLANNINGS PAR SALLES</FONT></B><CENTER></DT></TD>\n"
	         "   </TR>\n"
	         "</TABLE>\n");

	for (building = 0; building < MAIN_nbr_buildings; building ++)
	{
		fprintf (index_file_ptr,
		         "   <CENTER><H2>%s</H2></CENTER>\n"
		         "      <CENTER><P>\n",
			 MAIN_building_ptr [building].name_0);

		fprintf (index_file_ptr, "      <CENTER><P>\n");

		for (count = 0; count < MAIN_nbr_rooms; count ++)
		{
			if (MAIN_room_ptr [count].building != building)
			{
				continue;
			}

			sprintf (filename_0, "r%d.html", count);

			/* Lien dans l'index */
			fprintf (index_file_ptr,
				 "      <A HREF=\"%s\">%s</A><BR>\n",
				 filename_0, MAIN_room_ptr [count].name_0);

			/* Creation du fichier */
			sprintf (title_0, "Occupation de la salle %s",
						MAIN_room_ptr [count].name_0);
			plan_file_ptr = DISP_open_html_file (filename_0, title_0, 0xFFFFFF, 0x000000);
			printf ("Writing %s... ", filename_0);
			fflush (stdout);

			fprintf (plan_file_ptr,
			         "<TABLE CELLSPACING=0 CELLPADDING=0 BORDER=1 BGCOLOR=\"#FF0000\" WIDTH=\"100%%\">\n"
			         "   <TR>\n"
			         "      <TD><DT><CENTER><B><FONT COLOR=\"#FFFF00\" SIZE=+3>%s</FONT></B><CENTER></DT></TD>\n"
			         "   </TR>\n"
			         "</TABLE>\n", title_0);

			for (week = 0; week <= (int) (MAIN_last_week - MAIN_first_week); week ++)
			{
				/* Si la salle n'est pas utilisee cette semaine, on passe a la suite */
				if (DISP_room_time_ptr_ptr [count] [week] <= 0)
				{
					continue;
				}

				MAIN_get_date (&day_1, &month_1, &year_1,
									MAIN_first_day, MAIN_first_week + week);
				MAIN_get_date (&day_2, &month_2, &year_2,
									MAIN_last_day, MAIN_first_week + week);

				sprintf (header_line_0, "Semaine du %d/%d/%d au %d/%d/%d.",
							day_1 + 1, month_1 + 1, year_1,
							day_2 + 1, month_2 + 1, year_2);

				DISP_display_week_planning_html (plan_file_ptr,
					MAIN_room_ptr [count].planning [week],
					header_line_0, DISP_TYPE_ROOM);
			}

			/* Fermeture du fichier de salle */
			DISP_close_html_file (plan_file_ptr);
			printf ("Done.\n");
		}

		fprintf (index_file_ptr, "      </P><CENTER>\n");
	}

	fprintf (index_file_ptr, "<P>\n<HR WIDTH=\"100%%\">\n");

	/* Fermeture du fichier d'index */
	DISP_close_html_file (index_file_ptr);
}



FILE	*DISP_open_html_file (const char *filename_0, const char *title_0, long bgcolor, long txcolor)
{
	FILE		*file_ptr;

	file_ptr = fopen (filename_0, "w");
	if (file_ptr == NULL)
	{
		fprintf (stderr, "Error: C\'ant create file %s\n", filename_0);
		exit (-1);
	}

	fprintf (file_ptr,
				"<HTML>\n<HEAD>\n"
				"   <META HTTP-EQUIV=\"Content-Type\" CONTENT=\"text/html; charset=iso-8859-1\">\n"
				"   <META NAME=\"GENERATOR\" CONTENT=\"Bedaine 1.0 - Generateur d\'emplois du temps - (c) Laurent de Soras & Eric Lalitte 1998\">\n"
				"<TITLE>%s</TITLE>"
				"</HEAD>\n<BODY TEXT=\"#%06X\" BGCOLOR=\"#%06X\" LINK=\"#6495ED\">\n", title_0, (int)txcolor, (int)bgcolor);

	return (file_ptr);
}



void	DISP_close_html_file (FILE *file_ptr)
{
	fprintf (file_ptr, "</BODY>\n</HTML>\n");
	fclose (file_ptr);
}



void	DISP_display_week_planning_html (FILE *file_ptr, PARM_PLANNING *planning_ptr [7] [2], const char *header_line_0, int disp_type)
{
	int		minute_1;
	int		minute_2;
	int		hour_1;
	int		hour_2;
	int		day;
	int		half;
	int		type;
	int		type_2;
	int		subject;
	int		teacher;
	int		section;
	int		room;
	int		nbr_subdiv;
	int		subdiv;
	int		holiday;
	int		half_day;
	PARM_PLANNING	*half_day_planning_ptr;
	const char	*type_name_0 [2] [3] =
	{
		{ "", "TD de ", "TP de " },
		{ "", "TD d\'", "TP d\'" }
	};
	const char	*day_name_0 [7] =
	{
		"Lundi",
		"Mardi",
		"Mercredi",
		"Jeudi",
		"Vendredi",
		"Samedi",
		"Dimanche"
	};
	char		line_0 [DISP_MAX_LINE_LEN+1];

	fprintf (file_ptr, "<CENTER><H2><U>%s</U></H2></CENTER>\n", header_line_0);

	/* Pour chaque jour de la semaine */
	for (day = MAIN_first_day; day <= MAIN_last_day; day ++)
	{
		/* S'il n'y a rien dans cette journee, on la saute */
		if (   planning_ptr [day] [0] == NULL
			 && planning_ptr [day] [1] == NULL)
		{
			continue;
		}

		fprintf (file_ptr, "   <CENTER><H3>%s</H3></CENTER>\n", day_name_0 [day]);
		for (half = 0; half < 2; half ++)
		{
			fprintf (file_ptr,
						"      <CENTER><TABLE CELLSPACING=0 CELLPADDING=0 WIDTH=\"100%%\">\n");

			half_day_planning_ptr = planning_ptr [day] [half];

			/* Scanne chaque horaire (ils sont deja classes dans
				l'ordre chronologique) */
			while (half_day_planning_ptr != NULL)
			{
				fprintf (file_ptr, "         <TR>\n");

				/* Calcule et affiche les heures de debut et de fin du cours */
				minute_1 = half_day_planning_ptr->start;
				if (minute_1 >= 0)
				{
					minute_1 +=   (half == 0)
									? MAIN_am_start_h * 60 + MAIN_am_start_m
									: MAIN_pm_start_h * 60 + MAIN_pm_start_m;
					minute_2 = minute_1 + half_day_planning_ptr->duration;
					hour_1 = minute_1 / 60;
					hour_2 = minute_2 / 60;
					minute_1 %= 60;
					minute_2 %= 60;
					sprintf (line_0, "%02d:%02d - %02d:%02d",
								hour_1, minute_1, hour_2, minute_2);
					fprintf (file_ptr,
								"            <TD WIDTH=\"15%%\"><B>%s</B></TD>\n",
								line_0);
				}

				/* Horaire negatif, c'est un jour ferie */
				else
				{
					holiday = half_day_planning_ptr->subdiv;
					fprintf (file_ptr,
								"            <TD WIDTH=\"15%%\"><B>Ferie<B></TD>\n"
								"            <TD WIDTH=\"85%%\"><B>%s<B></TD>\n",
								MAIN_holiday_ptr [holiday].name_0);
					half = 1;
					fprintf (file_ptr, "         </TR>\n");
					goto fin_ligne;
				}

				/* Type du cours */
				type = half_day_planning_ptr->type;
				if (type >= 0)
				{
					/* Type et matiere */
					subject = half_day_planning_ptr->subject;
					type_2 = 0;
					switch (tolower (*MAIN_subject_ptr [subject].name_0))
					{
					case	'a':
					case	'e':
					case	'i':
					case	'o':
					case	'u':
					case	'y':
						type_2 = 1;
						break;
					}
					fprintf (file_ptr,
								"            <TD WIDTH=\"40%%\">%s%s</TD>\n",
								type_name_0 [type_2] [type],
								MAIN_subject_ptr [subject].name_0);

					/* Groupe */
					nbr_subdiv = half_day_planning_ptr->subdiv_tot;
					if (nbr_subdiv > 1)
					{
						subdiv = half_day_planning_ptr->subdiv;
						fprintf (file_ptr,
									"            <TD WIDTH=\"15%%\">Groupe %d/%d</TD>\n",
									subdiv + 1, nbr_subdiv);
					}
					else
					{
						fprintf (file_ptr, "            <TD WIDTH=\"15%%\"></TD>\n");
					}

					/* Prof */
					if (disp_type != DISP_TYPE_TEACHER)
					{
						teacher = half_day_planning_ptr->teacher;
						fprintf (file_ptr,
									"            <TD WIDTH=\"15%%\"><B>%s</B></TD>\n",
									MAIN_teacher_ptr [teacher].name_0);
					}

					/* Section */
					if (disp_type != DISP_TYPE_SECTION)
					{
						section = half_day_planning_ptr->section;
						fprintf (file_ptr,
									"            <TD WIDTH=\"15%%\"><B>%s</B></TD>",
									MAIN_section_ptr [section].name_0);
					}

					/* Salle */
					if (disp_type != DISP_TYPE_ROOM)
					{
						room = half_day_planning_ptr->room;
						fprintf (file_ptr,
									"            <TD WIDTH=\"15%%\"><B>%s</B></TD>\n",
									MAIN_room_ptr [room].name_0);
					}
				}

				/* C'est une demi-journee reservee */
				else
				{
					/* Nom de cette demi-journee */
					half_day = half_day_planning_ptr->subdiv;
					fprintf (file_ptr,
								"            <TD WIDTH=\"55%%\">Reserve %s</TD>\n",
								MAIN_half_day_ptr [half_day].name_0);

					/* Nom de la section qui en profite */
					if (disp_type != DISP_TYPE_SECTION)
					{
						section = half_day_planning_ptr->section;
						fprintf (file_ptr,
									"            <TD WIDTH=\"15%%\"><B>%s</B></TD>\n",
									MAIN_section_ptr [section].name_0);
					}
					else
					{
						fprintf (file_ptr,
									"            <TD WIDTH=\"15%%\"></TD>\n");
					}

					/* Recherche toutes les salles qui y sont associees */
					fprintf (file_ptr, "            <TD WIDTH=\"15%%\"><B>");
					if (disp_type != DISP_TYPE_ROOM)
					{
						for ( ; ; )
						{
							room = half_day_planning_ptr->room;
							if (room >= 0)
							{
								fprintf (file_ptr, "%s", MAIN_room_ptr [room].name_0);
							}
							else
							{
								break;
							}

							half_day_planning_ptr = half_day_planning_ptr->next_ptr;
							if (half_day_planning_ptr == NULL)
							{
								break;
							}
							fprintf (file_ptr, ", ");
						}
					}
					fprintf (file_ptr, "</B></TD>\n");

					fprintf (file_ptr, "         </TR>\n");
					goto fin_ligne;
				}

				/* Cours suivant */
				half_day_planning_ptr = half_day_planning_ptr->next_ptr;

				fprintf (file_ptr, "         </TR>\n");
			}

fin_ligne:
			fprintf (file_ptr, "      </TABLE></CENTER>\n");
			if (half == 0)
			{
				fprintf (file_ptr, "<P>\n");
			}
		}
	}

	/* Ligne de fin de semaine */
	fprintf (file_ptr, "<HR WIDTH=\"100%%\">\n<P>\n\n\n");
}




/****************************************************************************/
/*                                                                          */
/*                                                                          */
/*                                                                          */
/****************************************************************************/



/*==========================================================================*/
/*      Nom:                                                                */
/*      Description:                                                        */
/*      Parametres en entree:                                               */
/*      Parametres en sortie:                                               */
/*      Parametres en entree/sortie:                                        */
/*      Retour:                                                             */
/*==========================================================================*/
