/*****************************************************************************
*                                                                            *
*       MAIN.C                                                               *
*       Programme principal                                                  *
*                                                                            *
*       Laurent de Soras                                                     *
*       Eric Lalitte                                                         *
*       A1B7E                                                                *
*                                                                            *
* Tab = 3                                                                    *
*****************************************************************************/



/*\\\ FICHIERS INCLUDE \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*/

#include <stdio.h>
#include <stdlib.h>
#include <stdarg.h>
#include	<math.h>
#include <string.h>
#include	<ctype.h>
#include	<limits.h>
#include	<time.h>

#include "param.h"
#include "parser.h"
#include "solver.h"
#include	"display.h"
#include "main.h"



/*\\\ CONSTANTES ET MACROS PRIVEES \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*/

#define	MAIN_CMD_LINE_MSG_0	"Syntax:\n" \
				"bedaine [-key n] [-center m] [-width c] parameter_file_name\n"



/*\\\ TYPES & STRUCTURES PRIVEES \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*/



/*\\\ PROTOTYPES DES FONCTIONS PRIVEES \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*/



/*\\\ DEFINITION DES VARIABLES EXTERNES \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*/

int	MAIN_am_start_h = 8;
int	MAIN_am_start_m = 0;
int	MAIN_am_length = 240;	/* Duree de la matinee, en minutes */
int	MAIN_pm_start_h = 13;
int	MAIN_pm_start_m = 30;
int	MAIN_pm_length = 240;	/* Duree de l'apres-midi, en minutes */
int	MAIN_first_day = 0;
int	MAIN_last_day = 4;		/* Inclusif */
long	MAIN_first_week;
long	MAIN_last_week;			/* Inclusif */
int	MAIN_time_granularity = 15;

/* Nombre d'objets */
int	MAIN_nbr_holidays = 0;
int	MAIN_nbr_buildings = 0;
int	MAIN_nbr_periods = 0;
int	MAIN_nbr_subjects = 0;
int	MAIN_nbr_sections = 0;
int	MAIN_nbr_classes = 0;
int	MAIN_nbr_teachers = 0;
int	MAIN_nbr_rooms = 0;
int	MAIN_nbr_half_days = 0;

/* Pointeur sur les tableaux dynamiques des objets */
PARM_HOLIDAY	*MAIN_holiday_ptr;
PARM_BUILDING	*MAIN_building_ptr;
PARM_PERIOD		*MAIN_period_ptr;
PARM_SUBJECT	*MAIN_subject_ptr;
PARM_SECTION	*MAIN_section_ptr;
PARM_CLASS		*MAIN_class_ptr;
PARM_TEACHER	*MAIN_teacher_ptr;
PARM_ROOM		*MAIN_room_ptr;
PARM_HALF_DAY	*MAIN_half_day_ptr;



/*\\\ VARIABLES PRIVEES \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*/

/* Indice des jours de chaque mois dans l'annee de 365 jours */
int	MAIN_day_index [12] = { 0, 31, 59, 90, 120, 151, 181, 212, 243, 273, 304, 334 };



/*\\\ FONCTIONS \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*/



signed int  main (int argc, char *argv [])
{
	signed int	bad_result;
	int		option_cnt;
	signed long	planning_key;
	int		center;
	int		line_width;
	int		exit_flag;
	int		planning_count;
	int		nbr_plannings;
	time_t		current_time;
	const char	*option_0;
	const char	*filename_0;

	/* Teste les differentes options */
	option_cnt = 1;
	planning_key = 0;
	nbr_plannings = 1;
	center = 1;
	line_width = 79;
	filename_0 = NULL;
	while (option_cnt < argc)
	{
		option_0 = argv [option_cnt];
		if (*option_0 == '-')
		{
			switch (tolower (option_0 [1]))
			{
			case	'k':
				planning_key = strtol (argv [option_cnt + 1], NULL, 0);
				if (planning_key < 0)
				{
					nbr_plannings = (int) (-planning_key);
				}
				option_cnt += 2;
				break;

			case	'c':
				center = atoi (argv [option_cnt + 1]);
				if (center < 0 || center > 2)
				{
					fprintf (stderr, "Error: bad -key parameter. Range: [0-2]\n");
					exit (-1);
				}
				option_cnt += 2;
				break;

			case	'w':
				line_width = atoi (argv [option_cnt + 1]) - 1;
				if (line_width < 79)
				{
					fprintf (stderr, "Error: bad -width parameter. Range: >= 80\n");
					exit (-1);
				}
				option_cnt += 2;
				break;

			default:
				fprintf (stderr, MAIN_CMD_LINE_MSG_0);
				option_cnt ++;
				exit (-1);
				break;
			}
		}

		/* Dernier parametre: nom du fichier */
		else
		{
			if (option_cnt < argc - 1)
			{
				fprintf (stderr, MAIN_CMD_LINE_MSG_0);
				exit (-1);
			}
			filename_0 = option_0;
			option_cnt ++;
		}
	}

	if (filename_0 == NULL)
	{
		fprintf (stderr, MAIN_CMD_LINE_MSG_0);
		exit (-1);
	}

	/* Lit et decrypte le fichier de parametres */
	PARS_read_parameter_file (filename_0);

	/* Initialise les donnees precalculees et
		effectue quelques verifications */
	SOLV_init_and_check_planning ();

	/* Le generateur de nombres aleatoires va donner de vrais
	   nombres aleatoires... */
	current_time = time (NULL);
	srand ((unsigned int) current_time);

	bad_result = TRUE;
	do
	{
		/* Trouve un planning */
		if (planning_key < 0)
		{
			for (planning_count = 0; planning_count < nbr_plannings; planning_count ++)
			{
				bad_result = SOLV_make_planning ((long)rand () * 1000 / RAND_MAX, center);
				if (! bad_result)
				{
					break;
				}
				SOLV_restore_planning ();
			}
		}
		else
		{
			bad_result = SOLV_make_planning (planning_key, center);
		}

		if (bad_result)
		{
			fprintf (stderr, "Error: Can't find a schedule.\n");
			return (-1);
		}

		/* Affiche le planning */
		exit_flag = DISP_display (line_width);

		/* Efface le planning mais laisse quand meme
			tout ce qui a ete precalcule. */
		SOLV_restore_planning ();

		planning_key = -1;
	}
	while (! exit_flag);

	printf ("\nBye !\n");

	return (0);
}



void	MAIN_get_absolute_day (int *day_ptr, long *week_ptr, int day, int month, int year)
{
	int		bi_flag;
	long		abs_day;

	/* L'annee fait-elle 366 jours ? */
	bi_flag =    ((year % 4) == 0)
				 && (   (year % 100) != 0
					  || (year % 1000) == 0);

	/* Indice du jour a partir du 1/1/0 */
	abs_day =   (long)year * 365	/* 365 jours par annee normale */
				 + (year / 4)			/* Plus 1 jour par annee bissextile */
				 - (year / 100)		/* Les annee seculaires ne sont pas bissextiles */
				 + (year / 1000)		/* Mais les millenaires le sont. */
				 + MAIN_day_index [month]		/* Tout ca,          */
				 + day								/* c'est pour        */
				 + (month >= 2 && bi_flag);	/* l'annee courante. */

	/* Indice de la semaine */
	*week_ptr = (abs_day + 2) / 7;	/* Le 1/1/0 etait un mercredi... */

	/* Numero de jour */
	*day_ptr = (int) ((abs_day + 2) % 7);
}



void  MAIN_get_date (int *day_ptr, int *month_ptr, int *year_ptr, int day, long week)
{
	int		year;
	int		bi_year;
	int		sec_year;
	int		mil_year;
	int		month;
	int		bi_flag;
	long		abs_day;

	abs_day = week * 7 - 2 + day;

	/* Trouve l'annee */
	mil_year = (int) (abs_day / (1000L * 365 + 250 - 10 + 1));	/* Nombre d'annees millenaires */
	abs_day -= (long)mil_year * (1000L * 365 + 250 - 10 + 1);
	sec_year = (int) (abs_day / (100L * 365 + 25 - 1));		/* Nombre d'annees seculaires */
	abs_day -= (long)sec_year * (100L * 365 + 25 - 1);
	bi_year  = (int) (abs_day / (4 * 365 + 1));			/* Nombre d'annees bissextiles */
	abs_day -= (long)bi_year * (4 * 365 + 1);
	year = (int) (abs_day / 365);
	abs_day -= (long)year * 365;
	year += bi_year * 4;
	year += sec_year * 100;
	year += mil_year * 1000;	/* Ouf ! */
	*year_ptr = year;

	/* L'annee fait-elle 366 jours ? */
	bi_flag =    ((year % 4) == 0)
				 && (   (year % 100) != 0
					  || (year % 1000) == 0);

	/* Trouve le mois */
	for (month = 11; month > 0; month --)	/* A la sortie de la boucle, month = 0 */
	{
		if (MAIN_day_index [month] + (month >= 2 && bi_flag) < abs_day)
		{
			break;
		}
	}
	*month_ptr = month;

	*day_ptr = (int)abs_day - (MAIN_day_index [month] + (month >= 2 && bi_flag));
}



/****************************************************************************/
/*                                                                          */
/*                                                                          */
/*                                                                          */
/****************************************************************************/



/*==========================================================================*/
/*      Nom:                                                                */
/*      Description:                                                        */
/*      Parametres en entree:                                               */
/*      Parametres en sortie:                                               */
/*      Parametres en entree/sortie:                                        */
/*      Retour:                                                             */
/*==========================================================================*/
