/*****************************************************************************
*                                                                            *
*       PARAM.H                                                              *
*       Definition des structures des differents parametres du programme     *
*                                                                            *
*       Laurent de Soras                                                     *
*       Eric Lalitte                                                         *
*       A1B7E                                                                *
*                                                                            *
* Tab = 3                                                                    *
*****************************************************************************/



/*\\\ CONSTANTES ET MACROS PUBLIQUES \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*/

/* Nombre maximum de matieres enseignables dans un meme labo */
#define	PARM_MAX_SUBJECTS_PER_LAB		4
#define	PARM_MAX_SECTIONS_BY_TEACHER	4

/* Type de cours */
enum
{
	PARM_CLASS_TYPE_LECT,	/* Cours */
	PARM_CLASS_TYPE_TUT,		/* TD */
	PARM_CLASS_TYPE_LAB		/* TP */
};



/*\\\ TYPES & STRUCTURES PUBLIQUES \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*/

typedef struct PARM_Planning
{
	int		type;			/* Type du cours (cours, TD, TP). -1 = demi-journee reservee ou jour ferie */
	int		start;		/* Debut du cours, par rapport au debut de la demi-journee (en minutes). -1 = jour ferie */
	int		duration;	/* Duree du cours (en minutes) */
	int		section;		/* Section qui a cours en ce moment */
	int		subdiv;		/* Numero de la subdivision de la section (0 a n-1), ou numero du jour ferie. */
	int		subdiv_tot;	/* Nombre total de subdivisions pour ce cours */
	int		subject;		/* Matiere enseignee */
	int		teacher;		/* Professeur */
	int		room;			/* Numero de salle */

	struct PARM_Planning	*next_ptr;	/* Pointeur sur le cours suivant dans la demi-journee (NULL si aucun) */
} PARM_PLANNING;



typedef struct
{
	char		*name_0;			/* Pointeur sur le nom du jour ferie */
	int		day;				/* Jour dans le mois */
	int		month;			/* Mois dans l'annee */
} PARM_HOLIDAY;



typedef struct
{
	char		*name_0;			/* Pointeur sur le nom de la salle */
	int		capacity;		/* Nombre de personnes maximum pour cette salle */
	int		building;		/* Identificateur du batiment auquel appartient la salle */
	int		type;				/* Amphi, salle, labo */
	/* Optinonnel: pour les labos, liste de matieres enseignables dans celui_ci. */
	int		nbr_subjects;	/* Nombre de matieres dans le tableau ci-dessous */
	int		lab [PARM_MAX_SUBJECTS_PER_LAB];

	PARM_PLANNING	*((*planning) [7] [2]);	/* Planning associe */
	long		activity;		/* Temps potentiel total d'utilisation */
} PARM_ROOM;



typedef struct
{
	char		*name_0;			/* Nom de la section */
	int		nbr_students;	/* Nombre d'eleves dans cette section */
	/* Recoupement des donnees */
	int		nbr_classes;	/* Nombre de cours que doit recevoir cette section */
	int		*class_ptr;		/* Pointeur sur un tableau contenant le numero de chaque cours de la section */

	PARM_PLANNING	*((*planning) [7] [2]);	/* Planning associe */
	long		activity;		/* Temps potentiel total de cours */
} PARM_SECTION;



typedef struct
{
	char		*name_0;			/* Pointeur sur le nom de la matiere */
} PARM_SUBJECT;



typedef struct
{
	char		*name_0;			/* Pointeur sur le nom du batiment */
} PARM_BUILDING;



typedef struct
{
	int		section;			/* Identificateur de la section */
	int		nbr_subdiv;		/* Nombre de subdivisions de la classe pour ce cours. 0 = automatique */
	int		subject;			/* Identificateur de la matiere enseignee */
	int		type;				/* 0 = Cours, 1 = TD, 2 = TP */
	int		period;			/* Identificateur de la periode */
	int		nbr_classes;	/* Nombre de cours dans la periode */
	int		duration;		/* Duree de chaque cours en unite de temps */
	/* Recoupement des donnees */
	int		nbr_teachers;	/* Nombre de profs susceptibles d'assurer ce cours */
	int		*teacher_ptr;	/* Pointeur sur un tableau indiquant quels profs sont susceptibles d'assurer ce cours */
	int		nbr_rooms;		/* Nombre de salles susceptibles d'accueillir ce cours */
	int		*room_ptr;		/* Pointeur sur un tableau indiquant quels salles sont susceptibles d'accueillir ce cours */

	long		priority;		/* Priorite de placement dans l'emploi du temps */
} PARM_CLASS;



struct PARM_Half_day
{
	char		*name_0;			/* Pointeur sur l'intitule de la demi-journee */
	int		period;			/* Numero de la periode a qui elle appartient */
	int		day;				/* Numero du jour dans la semaine (0 a 6) */
	int		half;				/* Numero de moitie (0 ou 1) */
	int		nbr_reserved_rooms;	/* Nombre de salles prises pendant la demi-journee */
	/* Optionnel: si nbr != 0 */
	int		reserved_room_type;	/* Type des salles reservees */
	/* Optionnel: si lab */
	int		nbr_subjects;	/* Nombre de matieres dans le tableau ci-dessous */
	int		lab [PARM_MAX_SUBJECTS_PER_LAB];
	struct PARM_Half_day	*next_ptr;
};
typedef struct PARM_Half_day PARM_HALF_DAY;



typedef struct
{
	char		*name_0;			/* Nom de la periode */
	int		section;			/* Identificateur de la section */
	long		start_week;		/* Numero de la premiere semaine */
	int		duration;		/* Nombre de semaines */
} PARM_PERIOD;



typedef struct
{
	char		*name_0;			/* Pointeur sur le nom du prof */
	int		nbr_sections;	/* Nombre de sections */
	struct
	{
		int		subject;		/* Identificateur de la matiere enseignee */
		int		section;		/* Identificateur de la section */
	} 			section [PARM_MAX_SECTIONS_BY_TEACHER];

	PARM_PLANNING	*((*planning) [7] [2]);	/* Planning associe */
	long		activity;		/* Temps potentiel total d'enseignement */
} PARM_TEACHER;



/*\\\ PROTOTYPES DES FONCTIONS PUBLIQUES \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*/

/*--------------------------------------------------------------------------*/
/*                                                                          */
/*--------------------------------------------------------------------------*/



/*\\\ VARIABLES EXTERNES \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*/

