/*****************************************************************************
*                                                                            *
*       PARSER.C                                                             *
*       Lecture et interpretation du fichier de parametres                   *
*                                                                            *
*       Laurent de Soras                                                     *
*       Eric Lalitte                                                         *
*       A1B7E                                                                *
*                                                                            *
* Tab = 3                                                                    *
*****************************************************************************/



/*\\\ FICHIERS INCLUDE \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*/

#include <stdio.h>
#include <stdlib.h>
#include <stdarg.h>
#include	<math.h>
#include <string.h>
#include	<ctype.h>
#include	<limits.h>

#include "param.h"
#include	"parser.h"
#include	"main.h"



/*\\\ CONSTANTES ET MACROS PRIVEES \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*/

/* Nombre maximum de caracteres dans un lexeme */
#define	PARS_LEXEME_MAX_LEN			99



/*\\\ TYPES & STRUCTURES PRIVEES \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*/



/*\\\ PROTOTYPES DES FONCTIONS PRIVEES \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*/

/*--------------------------------------------------------------------------*/
/*      Fonctions utiliaires                                                */
/*--------------------------------------------------------------------------*/

signed int	PARS_read_lexeme_in_file (FILE *file_ptr, char *lexeme_0, int max_len, long *current_line_ptr, int fatal_eof_flag);
signed int	PARS_string_compare (const char *text_1_0, const char *text_2_0);
signed int	PARS_get_time (const char *param_0, int *hours_ptr, int *minutes_ptr);
signed int	PARS_get_date (const char *param_0, long *week_ptr);
signed int	PARS_get_day (const char *param_0);
signed int	PARS_get_type (const char *param_0);

/*--------------------------------------------------------------------------*/
/*      Traitement des tableaux                                             */
/*--------------------------------------------------------------------------*/

signed int	PARS_get_building_nbr (const char *building_name_0, int creation_flag);
signed int	PARS_get_period_nbr (const char *period_name_0, int section, int creation_flag);
signed int	PARS_get_subject_nbr (const char *period_name_0, int creation_flag);
signed int	PARS_get_section_nbr (const char *period_name_0, int creation_flag);
signed int	PARS_allocate_mem_for_new_object (void **block_ptr_ptr, int **flag_ptr_ptr, int size, size_t object_size);

/*--------------------------------------------------------------------------*/
/*      Traitement des cles                                                 */
/*--------------------------------------------------------------------------*/

void	PARS_read_key_general (void);
void	PARS_read_key_holiday (void);
void	PARS_read_key_building (void);
void	PARS_read_key_class (void);
void	PARS_read_key_period (void);
void	PARS_read_key_room (void);
void	PARS_read_key_section (void);
void	PARS_read_key_teacher (void);



/*\\\ DEFINITION DES VARIABLES EXTERNES \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*/



/*\\\ VARIABLES PRIVEES \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*/

/* Traitement du fichier de parametres */
long	PARS_current_line;
long	PARS_nbr_errors;
long	PARS_nbr_warnings;
char	PARS_lexeme_0 [PARS_LEXEME_MAX_LEN+1];
FILE	*PARS_file_ptr;

/* Tableau des numeros d'objets references mais non declares */
int	*PARS_defined_building_flag_ptr = NULL;
int	*PARS_defined_period_flag_ptr = NULL;
int	*PARS_defined_section_flag_ptr = NULL;



/*\\\ FONCTIONS \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*/



/*==========================================================================*/
/*      Nom: PARS_read_parameter_file                                       */
/*      Description: Lit et decrypte le fichier de parametres. Si des       */
/*                   erreurs sont recontrees, elles sont signalees puis le  */
/*                   programme est automatiquement abandonne.               */
/*      Parametres en entree:                                               */
/*        - filename_0: pointeur sur le nom du fichier a lire, termine par  */
/*                      0.                                                  */
/*==========================================================================*/

void	PARS_read_parameter_file (const char *filename_0)
{
	signed int	result;
	signed int	key;
	int		count;
	struct
	{
		char		*name_0;
		void		(*func_ptr) (void);
	} key_def [] =
	{
		{ ":general",	PARS_read_key_general },
		{ ":holiday",	PARS_read_key_holiday },
		{ ":building",	PARS_read_key_building },
		{ ":class",		PARS_read_key_class },
		{ ":period",	PARS_read_key_period },
		{ ":room",		PARS_read_key_room },
		{ ":section",	PARS_read_key_section },
		{ ":teacher",	PARS_read_key_teacher },
		{ NULL,			NULL }
	};

	fprintf (stderr, "Reading parameter file...\n");

	/* Ouverture du fichier */
	PARS_file_ptr = fopen (filename_0, "r");
	if (PARS_file_ptr == NULL)
	{
		fprintf (stderr, "Error: can\'t open parameter file %s\n", filename_0);
		exit (-1);
	}

/*______________________________________________
 *
 * Lecture des donnees
 *______________________________________________
 */

	PARS_current_line = 1;
	PARS_nbr_errors = 0;
	PARS_nbr_warnings = 0;
	for ( ; ; )
	{
		/* Lit un lexeme */
		result = PARS_read_lexeme_in_file (PARS_file_ptr, PARS_lexeme_0, PARS_LEXEME_MAX_LEN, &PARS_current_line, FALSE);

		/* Erreur de lecture dans le fichier */
		if (result == -1)
		{
			fprintf (stderr, "Error (line %ld): while reading parameter file.\n", PARS_current_line);
			PARS_nbr_errors ++;
			exit (-1);
		}

		/* Fin du fichier */
		if (result == 1)
		{
			break;
		}

		/* Cherche la cle ou son parametre, et la traite */
		key = 0;
		for ( ; ; key ++)
		{
			/* Cle introuvable */
			if (key_def [key].name_0 == NULL)
			{
				fprintf (stderr, "Error (line %ld): Unknown key.\n", PARS_current_line);
				PARS_nbr_errors ++;
				break;
			}

			/* Cle trouvee */
			if (PARS_string_compare (PARS_lexeme_0, key_def [key].name_0) == 0)
			{
				(key_def [key].func_ptr) ();
				break;
			}
		}
	}

	/* Fermeture du fichier */
	fclose (PARS_file_ptr);

/*______________________________________________
 *
 * Controle des donnees
 *______________________________________________
 */

	/* Verifie que toutes les entites utilisees ont ete definies */
	for (count = 0; count < MAIN_nbr_buildings; count ++)
	{
		if (! PARS_defined_building_flag_ptr [count])
		{
			fprintf (stderr, "Error: Building \"%s\" has not been defined.\n",
						MAIN_building_ptr [count].name_0);
			PARS_nbr_errors ++;
		}
	}
	if (MAIN_nbr_buildings <= 0)
	{
		fprintf (stderr, "Error: No building defined.\n");
		PARS_nbr_errors ++;
	}
	else
	{
		free (PARS_defined_building_flag_ptr);
	}

	for (count = 0; count < MAIN_nbr_sections; count ++)
	{
		if (! PARS_defined_section_flag_ptr [count])
		{
			fprintf (stderr, "Error: Section \"%s\" has not been defined.\n",
						MAIN_section_ptr [count].name_0);
			PARS_nbr_errors ++;
		}
	}
	if (MAIN_nbr_sections <= 0)
	{
		fprintf (stderr, "Error: No section defined.\n");
		PARS_nbr_errors ++;
	}
	else
	{
		free (PARS_defined_section_flag_ptr);
	}

	for (count = 0; count < MAIN_nbr_periods; count ++)
	{
		if (! PARS_defined_period_flag_ptr [count])
		{
			fprintf (stderr,
						"Error: Period \"%s\" has not been defined "
						"for section %s.\n",
						MAIN_period_ptr [count].name_0,
						MAIN_section_ptr [MAIN_period_ptr [count].section].name_0);
			PARS_nbr_errors ++;
		}
	}
	if (MAIN_nbr_periods <= 0)
	{
		fprintf (stderr, "Error: No period defined.\n");
		PARS_nbr_errors ++;
	}
	else
	{
		free (PARS_defined_period_flag_ptr);
	}

	/* Verifie que les demi-journees ne se chevauchent pas */
	if (  MAIN_pm_start_h * 60 + MAIN_pm_start_m
		 < MAIN_am_start_h * 60 + MAIN_pm_start_m + MAIN_pm_length)
	{
		fprintf (stderr, "Error: Overlapping half-days.\n");
		PARS_nbr_errors ++;
	}

	/* Verifie que le premier jour de la semaine est avant le dernier */
	if (MAIN_first_day > MAIN_last_day)
	{
		fprintf (stderr, "Error: Overlapping half-days.\n");
		PARS_nbr_errors ++;
	}

	/* Recherche le numero de la premiere semaine */
	MAIN_first_week = LONG_MAX;
	MAIN_last_week = 0;
	for (count = 0; count < MAIN_nbr_periods; count ++)
	{
		MAIN_first_week = MIN (MAIN_first_week,
									  MAIN_period_ptr [count].start_week);
		MAIN_last_week = MAX (MAIN_last_week,
										MAIN_period_ptr [count].start_week
									 + MAIN_period_ptr [count].duration - 1);
	}

	/* Base tous les debuts de periode sur la premiere semaine */
	for (count = 0; count < MAIN_nbr_periods; count ++)
	{
		MAIN_period_ptr [count].start_week -= MAIN_first_week;
	}

	fprintf (stderr, "%d errors, %d warnings.\n", PARS_nbr_errors, PARS_nbr_warnings);
	if (PARS_nbr_errors > 0)
	{
		fprintf (stderr, "Too many errors. Execution aborted.\n");
		exit (-1);
	}
	printf ("\n");
}



/****************************************************************************/
/*                                                                          */
/*      FONCTIONS UTILITAIRES                                               */
/*                                                                          */
/****************************************************************************/



/*==========================================================================*/
/*      Nom: PARS_read_lexeme_in_file                                       */
/*      Description: Lit un lexeme dans un fichier et pointe juste apres.   */
/*                   Un lexeme est ici un groupe de signes, optionnellement */
/*                   encadre par des guillemets ("). Ceux-ci ne sont pas    */
/*                   pris en compte.                                        */
/*                   Les lexeme sont separes par des separateurs qui sont   */
/*                   espaces, sauts de lignes ou de page, ou tabulations.   */
/*                   D'eventuels commentaires sont ignores. Les             */
/*                   commentaires commencent par un point-virgule (;) et    */
/*                   se terminent a la fin de la ligne.                     */
/*      Parametres en entree:                                               */
/*        - max_len: taille maximum du texte (sans compter le 0 final). Si  */
/*                   le lexeme depasse cette taille, il est automatiquement */
/*                   tronque.                                               */
/*        - fatal_eof_flag: TRUE si une fin de fichier doit etre considere  */
/*                          comme une erreur (fin inattendue).              */
/*      Parametres en sortie:                                               */
/*        - lexeme_0: Le lexeme lu arrive a cette adresse, et est termine   */
/*                    par 0.                                                */
/*      Parametres en entree/sortie:                                        */
/*        - file_ptr: pointeur sur le fichier que l'on veut lire. Le        */
/*                    fichier est positionne juste a la fin du lexeme lu.   */
/*        - current_line_ptr: pointeur sur le compteur de ligne, qui est    */
/*                            augmente a chaque saut de ligne ou de page.   */
/*      Retour: 0 si tout s'est bien passe,                                 */
/*              1 si la fin du fichier a ete atteinte (avant lecture d'un   */
/*                quelconque lexeme), et si fatal_eof_flag == FALSE,        */
/*              -1 si une erreur est survenue.                              */
/*==========================================================================*/

signed int	PARS_read_lexeme_in_file (FILE *file_ptr, char *lexeme_0, int max_len, long *current_line_ptr, int fatal_eof_flag)
{
	int		c;
	int		dq_flag;
	int		trunc_flag;
	long		old_offset = 0;

	/* Recherche le debut du lexeme */
	do
	{
		old_offset = ftell (file_ptr);
		if (old_offset == -1L)
		{
			fprintf (stderr, "Error (line %ld): while reading parameter file.\n", PARS_current_line);
			PARS_nbr_errors ++;
			return (-1);
		}
		c = fgetc (file_ptr);

		/* Commentaire */
		if (c == ';')
		{
			do
			{
				c = fgetc (file_ptr);
			}
			while (   c != EOF
					 && c != '\n'
					 && c != '\f'
					 && c != '\v');
		}

		/* Fin du fichier */
		if (c == EOF)
		{
			return (fatal_eof_flag ? -1 : 1);
		}

		/* Changement de ligne */
		if (   c == '\n'
			 || c == '\f'
			 || c == '\v')
		{
			(*current_line_ptr) ++;
		}

	}
	while (   c == ' '
			 || c == '\t'
			 || c == '\n'
			 || c == '\f'
			 || c == '\v');

	/* On vire le guillemet s'il y en a un */
	dq_flag = FALSE;
	if (c == '\"')
	{
		dq_flag = TRUE;
		old_offset = ftell (file_ptr);
		if (old_offset == -1L)
		{
			fprintf (stderr, "Error (line %ld): while reading parameter file.\n", PARS_current_line);
			PARS_nbr_errors ++;
			return (-1);
		}
		c = fgetc (file_ptr);

		/* Fin du fichier */
		if (c == EOF)
		{
			return (fatal_eof_flag ? -1 : 1);
		}
	}

	/* Lecture du lexeme */
	trunc_flag = FALSE;
	for (;;)
	{
		/* Test de fin du lexeme */
		if (dq_flag)
		{
			if (c == '\"')
			{
				break;
			}
		}
		else
		{
			if (   c == ' '
				 || c == '\t'
				 || c == '\n'
				 || c == '\f'
				 || c == '\v'
				 || c == EOF)
			{
				/* On revient en arriere d'un caractere pour etre juste a la
					fin du lexeme. */
				if (fseek (file_ptr, old_offset, SEEK_SET) != 0)
				{
					fprintf (stderr, "Error (line %ld): while reading parameter file.\n", PARS_current_line);
					PARS_nbr_errors ++;
					return (-1);
				}
				break;
			}
		}

		/* Regarde si on change de ligne (si entre guillemets) */
		if (   c == '\n'
			 || c == '\f'
			 || c == '\v')
		{
			(*current_line_ptr) ++;
		}

		/* Ajoute le caractere au lexeme */
		if (max_len > 0)
		{
			*lexeme_0++ = (char) c;
			max_len --;
		}
		else
		{
			trunc_flag = TRUE;
		}

		/* Lit le caractere suivant */
		old_offset = ftell (file_ptr);
		if (old_offset == -1L)
		{
			fprintf (stderr, "Error (line %ld): while reading parameter file.\n", PARS_current_line);
			PARS_nbr_errors ++;
			return (-1);
		}
		c = fgetc (file_ptr);
	}

	/* Termine le lexeme par 0 */
	*lexeme_0 = '\0';

	if (trunc_flag)
	{
		fprintf (stderr, "Warning (line %ld): key or parameter truncated.\n", PARS_current_line);
		PARS_nbr_warnings ++;
	}

	return (0);
}



/*==========================================================================*/
/*      Nom: PARS_string_compare                                            */
/*      Description: Compare deux chaines, sans tenir compte de la case, ou */
/*                   des espaces, underscores ou tabulations qui trainent   */
/*                   dans la chaine.                                        */
/*      Parametres en entree:                                               */
/*        - text_1_0: pointeur sur la premiere chaine.                      */
/*        - text_2_0: pointeur sur la deuxieme chaine.                      */
/*      Retour: 0 si les deux chaines sont jugees egales.                   */
/*==========================================================================*/

signed int	PARS_string_compare (const char *text_1_0, const char *text_2_0)
{
	do
	{
		while (   *text_1_0 == ' '
				 || *text_1_0 == '_'
				 || *text_1_0 == '\t'
				 || *text_1_0 == '\n'
				 || *text_1_0 == '\f'
				 || *text_1_0 == '\v')
		{
			text_1_0 ++;
		}

		while (   *text_2_0 == ' '
				 || *text_2_0 == '_'
				 || *text_2_0 == '\t'
				 || *text_2_0 == '\n'
				 || *text_2_0 == '\f'
				 || *text_2_0 == '\v')
		{
			text_2_0 ++;
		}

		if (toupper (*text_1_0) != toupper (*text_2_0))
		{
			return (-1);
		}

	}
	while (   *text_1_0++ != '\0'
					 && *text_2_0++ != '\0');

	return (0);
}



/*==========================================================================*/
/*      Nom: PARS_get_time                                                  */
/*      Description: Decrypte un parametre de temps "hh mm". Signale les    */
/*                   erreurs, et avertit en cas de donnees en trop dans     */
/*                   le parametre.                                          */
/*      Parametres en entree:                                               */
/*        - param_0: pointeur sur la chaine du parametre.                   */
/*      Parametres en sortie:                                               */
/*        - hour_ptr: pointeur sur l'heure decryptee.                       */
/*        - minute_ptr: pointeur sur la minute decryptee.                   */
/*      Retour: 0 si tout s'est bien passe.                                 */
/*==========================================================================*/

signed int	PARS_get_time (const char *param_0, int *hour_ptr, int *minute_ptr)
{
	*hour_ptr = (int) strtol (param_0, (char **) &param_0, 10);
	if (*hour_ptr < 0 || *hour_ptr > 23)
	{
		fprintf (stderr, "Error (line %ld): %d is not a valid hour parameter (range: [0..23]).\n",
					PARS_current_line, *hour_ptr);
		PARS_nbr_errors ++;
		return (-1);
	}

	*minute_ptr = (int) strtol (param_0, (char **) &param_0, 10);
	if (*minute_ptr < 0 || *minute_ptr > 59)
	{
		fprintf (stderr, "Error (line %ld): %d is not a valid minute parameter (range: [0..59]).\n",
					PARS_current_line, *minute_ptr);
		PARS_nbr_errors ++;
		return (-1);
	}

	if (PARS_string_compare (param_0, "") != 0)
	{
		fprintf (stderr, "Warning (line %ld): Garbage following minute parameter.\n",
					PARS_current_line);
		PARS_nbr_warnings ++;
	}

	return (0);
}



/*==========================================================================*/
/*      Nom: PARS_get_date                                                  */
/*      Description: Decrypte une date sous forme de chaine "jour mois an", */
/*                   et avertit en cas de mauvaise entree.                  */
/*      Parametres en entree:                                               */
/*        - param_0: pointeur sur la chaine a convertir.                    */
/*      Parametres en sortie:                                               */
/*        - week_ptr: Numero absolu de la semaine qui correspond a la date. */
/*      Retour: 0 si tout s'est bien passe.                                 */
/*==========================================================================*/

signed int	PARS_get_date (const char *param_0, long *week_ptr)
{
	int		year;
	int		month;
	int		day;
	int		bidon;

	day = (int) strtol (param_0, (char **) &param_0, 10);
	month = (int) strtol (param_0, (char **) &param_0, 10);
	year = (int) strtol (param_0, (char **) &param_0, 10);

	if (month < 1 || month > 12)
	{
		fprintf (stderr, "Error (line %ld): %d is not a valid month parameter (range: [1..12]).\n",
					PARS_current_line, month);
		PARS_nbr_errors ++;
		return (-1);
	}
	month --;

	/*** Test tres approximatif ***/
	if (day < 1 || day > 31)
	{
		fprintf (stderr, "Error (line %ld): %d is not a valid day parameter (range: [1..31]).\n",
					PARS_current_line, day);
		PARS_nbr_errors ++;
		return (-1);
	}
	day --;

	MAIN_get_absolute_day (&bidon, week_ptr, day, month, year);

	return (0);
}



/*==========================================================================*/
/*      Nom: PARS_get_day                                                   */
/*      Description: Decode un parametre de jour. Signale les erreurs.      */
/*      Parametres en entree:                                               */
/*        - param_0: pointeur sur la chaine du parametre.                   */
/*      Retour: Le jour (0 - 6) ou un nombre negatif si une erreur s'est    */
/*              produite.                                                   */
/*==========================================================================*/

signed int	PARS_get_day (const char *param_0)
{
	int		day;
	char		day_list_0 [7] [3] = { "mo", "tu", "we", "th", "fr", "sa", "su" };

	/* Jour exprime en valeur numerique */
	day = atoi (param_0);
	if (day >= 1 && day <= 7)
	{
		return (day - 1);
	}

	/* Jour exprime en texte */
	for (day = 0; day < 7; day ++)
	{
		if (PARS_string_compare (param_0, day_list_0 [day]) == 0)
		{
			return (day);
		}
	}

	/* Jour non valide */
	fprintf (stderr, "Error (line %ld): %s is not a valid day parameter.\n",
				PARS_current_line, param_0);
	PARS_nbr_errors ++;

	return (-1);
}



/*==========================================================================*/
/*      Nom: PARS_get_type                                                  */
/*      Description: Decode un type de cours ou de salle. Signale les       */
/*                   erreurs.                                               */
/*      Parametres en entree:                                               */
/*        - param_0: pointeur sur la chaine du parametre.                   */
/*      Retour: Le type (0 - 2) ou un nombre negatif si une erreur s'est    */
/*              produite.                                                   */
/*==========================================================================*/

signed int	PARS_get_type (const char *param_0)
{
	if (PARS_string_compare (param_0, "lecture") == 0)
	{
		return (PARM_CLASS_TYPE_LECT);
	}

	else if (PARS_string_compare (param_0, "tutorial") == 0)
	{
		return (PARM_CLASS_TYPE_TUT);
	}

	else if (PARS_string_compare (param_0, "lab") == 0)
	{
		return (PARM_CLASS_TYPE_LAB);
	}

	fprintf (stderr, "Error (line %ld): %s is not a valid class type parameter.\n",
				PARS_current_line, param_0);
	PARS_nbr_errors ++;

	return (-1);
}




/****************************************************************************/
/*                                                                          */
/*      TRAITEMENT DES TABLEAUX                                             */
/*                                                                          */
/****************************************************************************/



/*==========================================================================*/
/*      Nom: PARS_get_building_nbr                                          */
/*      Description: Renvoie le numero d'index d'un batiment a partir de    */
/*                   son nom. Si ce batiment n'existe pas, il peut etre     */
/*                   cree.                                                  */
/*      Parametres en entree:                                               */
/*        - building_name_0: pointeur sur le nom du batiment.               */
/*        - creation_flag: TRUE si le batiment inexistant doit etre cree.   */
/*      Retour: 0 si tout s'est bien passe.                                 */
/*==========================================================================*/

signed int	PARS_get_building_nbr (const char *building_name_0, int creation_flag)
{
	int		count;

	/* Recherche parmi les batiments deja definies */
	for (count = 0; count < MAIN_nbr_buildings; count ++)
	{
		if (PARS_string_compare (building_name_0, MAIN_building_ptr [count].name_0) == 0)
		{
			return (count);
		}
	}

	/* On n'a pas trouve et on ne veut pas crer */
	if (! creation_flag)
	{
		return (-1);
	}

	/* Alloue de la memoire pour un nouvel objet dans le tableau */
	if (PARS_allocate_mem_for_new_object ((void **) &MAIN_building_ptr,
													  &PARS_defined_building_flag_ptr,
													  MAIN_nbr_buildings,
													  sizeof (*MAIN_building_ptr)))
	{
		return (-1);
	}

	/* Creation d'un batiment a definir plus tard */
	PARS_defined_building_flag_ptr [MAIN_nbr_buildings] = FALSE;
	MAIN_building_ptr [MAIN_nbr_buildings].name_0 = strdup (building_name_0);
	if (MAIN_building_ptr [MAIN_nbr_buildings].name_0 == NULL)
	{
		return (-1);
	}
	MAIN_nbr_buildings ++;

	return (MAIN_nbr_buildings - 1);
}



/*==========================================================================*/
/*      Nom: PARS_get_period_nbr                                            */
/*      Description: Renvoie le numero d'index d'une periode a partir de    */
/*                   son nom. Si la periode n'existe pas, elle peut etre    */
/*                   cree.                                                  */
/*      Parametres en entree:                                               */
/*        - period_name_0: pointeur sur le nom de la periode                */
/*        - section: numero de la section attachee a la periode.            */
/*        - creation_flag: TRUE si la section inexistant doit etre cree.    */
/*      Retour: 0 si tout s'est bien passe.                                 */
/*==========================================================================*/

signed int	PARS_get_period_nbr (const char *period_name_0, int section, int creation_flag)
{
	int		count;

	/* Recherche parmi les periodes deja definies */
	for (count = 0; count < MAIN_nbr_periods; count ++)
	{
		if (   PARS_string_compare (period_name_0, MAIN_period_ptr [count].name_0) == 0
			 && section == MAIN_period_ptr [count].section)
		{
			return (count);
		}
	}

	/* On n'a pas trouve et on ne veut pas crer */
	if (! creation_flag)
	{
		return (-1);
	}

	/* Alloue de la memoire pour un nouvel objet dans le tableau */
	if (PARS_allocate_mem_for_new_object ((void **) &MAIN_period_ptr,
													  &PARS_defined_period_flag_ptr,
													  MAIN_nbr_periods,
													  sizeof (*MAIN_period_ptr)))
	{
		return (-1);
	}

	/* Creation d'une periode a definir plus tard */
	PARS_defined_period_flag_ptr [MAIN_nbr_periods] = FALSE;
	MAIN_period_ptr [MAIN_nbr_periods].name_0 = strdup (period_name_0);
	if (MAIN_period_ptr [MAIN_nbr_periods].name_0 == NULL)
	{
		return (-1);
	}
	MAIN_period_ptr [MAIN_nbr_periods].section = section;
	MAIN_nbr_periods ++;

	return (MAIN_nbr_periods - 1);
}



/*==========================================================================*/
/*      Nom: PARS_get_subject_nbr                                           */
/*      Description: Renvoie le numero d'index d'une matiere a partir de    */
/*                   son nom. Si la matiere n'existe pas, elle peut etre    */
/*                   cree.                                                  */
/*      Parametres en entree:                                               */
/*        - subject_name_0: pointeur sur le nom de la matiere               */
/*        - creation_flag: TRUE si la matiere inexistante doit etre cree.   */
/*      Retour: 0 si tout s'est bien passe.                                 */
/*==========================================================================*/

signed int	PARS_get_subject_nbr (const char *subject_name_0, int creation_flag)
{
	int		count;

	/* Recherche parmi les matieres deja definies */
	for (count = 0; count < MAIN_nbr_subjects; count ++)
	{
		if (PARS_string_compare (subject_name_0, MAIN_subject_ptr [count].name_0) == 0)
		{
			return (count);
		}
	}

	/* On n'a pas trouve et on ne veut pas crer */
	if (! creation_flag)
	{
		return (-1);
	}

	/* Alloue de la memoire pour un nouvel objet dans le tableau */
	if (PARS_allocate_mem_for_new_object ((void **) &MAIN_subject_ptr,
													  NULL,
													  MAIN_nbr_subjects,
													  sizeof (*MAIN_subject_ptr)))
	{
		return (-1);
	}

	/* Creation d'une matiere a definir plus tard */
	MAIN_subject_ptr [MAIN_nbr_subjects].name_0 = strdup (subject_name_0);
	if (MAIN_subject_ptr [MAIN_nbr_subjects].name_0 == NULL)
	{
		return (-1);
	}
	MAIN_nbr_subjects ++;

	return (MAIN_nbr_subjects - 1);
}



/*==========================================================================*/
/*      Nom: PARS_get_section_nbr                                           */
/*      Description: Renvoie le numero d'index d'une section a partir de    */
/*                   son nom. Si la section n'existe pas, elle peut etre    */
/*                   cree.                                                  */
/*      Parametres en entree:                                               */
/*        - section_name_0: pointeur sur le nom de la section               */
/*        - creation_flag: TRUE si la section inexistante doit etre cree.   */
/*      Retour: 0 si tout s'est bien passe.                                 */
/*==========================================================================*/

signed int	PARS_get_section_nbr (const char *section_name_0, int creation_flag)
{
	int		count;

	/* Recherche parmi les sections deja definies */
	for (count = 0; count < MAIN_nbr_sections; count ++)
	{
		if (PARS_string_compare (section_name_0, MAIN_section_ptr [count].name_0) == 0)
		{
			return (count);
		}
	}

	/* On n'a pas trouve et on ne veut pas crer */
	if (! creation_flag)
	{
		return (-1);
	}

	/* Alloue de la memoire pour un nouvel objet dans le tableau */
	if (PARS_allocate_mem_for_new_object ((void **) &MAIN_section_ptr,
													  &PARS_defined_section_flag_ptr,
													  MAIN_nbr_sections,
													  sizeof (*MAIN_section_ptr)))
	{
		return (-1);
	}

	/* Creation d'une section a definir plus tard */
	PARS_defined_section_flag_ptr [MAIN_nbr_sections] = FALSE;
	MAIN_section_ptr [MAIN_nbr_sections].name_0 = strdup (section_name_0);
	if (MAIN_section_ptr [MAIN_nbr_sections].name_0 == NULL)
	{
		return (-1);
	}
	MAIN_nbr_sections ++;

	return (MAIN_nbr_sections - 1);
}



/*==========================================================================*/
/*      Nom: PARS_allocate_mem_for_new_object                               */
/*      Description: Agrandit un tableau dynamique en reallouant de la      */
/*                   memoire si necessaire. S'il y a aussi un tableau de    */
/*                   flags associes, ce dernier est aussi redimensionne.    */
/*      Parametres en entree:                                               */
/*        - size: taille actuelle du tableau. Si celle-ci est nulle, on le  */
/*                cree.                                                     */
/*        - object_size: taille en octets des objets du tableau.            */
/*      Parametres en entree/sortie:                                        */
/*        - block_ptr_ptr: pointeur sur l'adresse du tableau.               */
/*        - flag_ptr_ptr: pointeur sur l'adresse du tableau de flags. NULL  */
/*                        s'il n'y a pas de flags.                          */
/*      Retour: 0 si tout s'est bien passe.                                 */
/*==========================================================================*/

signed int	PARS_allocate_mem_for_new_object (void **block_ptr_ptr, int **flag_ptr_ptr, int size, size_t object_size)
{
	void		*temp_ptr;

	/* Il n'y a aucun objet deja defini dans le tableau, on le cree. */
	if (size == 0)
	{
		*block_ptr_ptr = malloc (MAIN_OBJECT_ARRAY_GRANUL * object_size);
		if (*block_ptr_ptr == NULL)
		{
			return (-1);
		}

		if (flag_ptr_ptr != NULL)
		{
			*flag_ptr_ptr = (int *) malloc (MAIN_OBJECT_ARRAY_GRANUL * sizeof (**flag_ptr_ptr));
			if (*flag_ptr_ptr == NULL)
			{
				return (-1);
			}
		}
	}

	/* Sinon il faut sans doute reserver de la place
		pour une nouvelle serie d'objets */
	else if ((size % MAIN_OBJECT_ARRAY_GRANUL) == 0)
	{
		temp_ptr = realloc (*block_ptr_ptr, (size + MAIN_OBJECT_ARRAY_GRANUL) * object_size);
		if (temp_ptr == NULL)
		{
			return (-1);
		}
		*block_ptr_ptr = temp_ptr;

		if (flag_ptr_ptr != NULL)
		{
			temp_ptr = realloc (*flag_ptr_ptr, (size + MAIN_OBJECT_ARRAY_GRANUL) * sizeof (**flag_ptr_ptr));
			if (temp_ptr == NULL)
			{
				return (-1);
			}
			*flag_ptr_ptr = (int *) temp_ptr;
		}
	}

	return (0);
}





/****************************************************************************/
/*                                                                          */
/*      TRAITEMENT DES CLES                                                 */
/*                                                                          */
/****************************************************************************/



/*==========================================================================*/
/*      Nom: PARS_read_key_general                                          */
/*      Description: Lit la cle ":general".                                 */
/*==========================================================================*/

void	PARS_read_key_general (void)
{
	signed int	result;
	int		hour;
	int		minute;

	/* Heure de debut de la matinee */
	if (PARS_read_lexeme_in_file (PARS_file_ptr, PARS_lexeme_0, PARS_LEXEME_MAX_LEN,
											&PARS_current_line, TRUE))
	{
		return;
	}
	if (PARS_get_time (PARS_lexeme_0, &hour, &minute) == 0)
	{
		MAIN_am_start_h = hour;
		MAIN_am_start_m = minute;
	}

	/* Longueur de la matinee */
	if (PARS_read_lexeme_in_file (PARS_file_ptr, PARS_lexeme_0, PARS_LEXEME_MAX_LEN,
											&PARS_current_line, TRUE))
	{
		return;
	}
	if (PARS_get_time (PARS_lexeme_0, &hour, &minute) == 0)
	{
		MAIN_am_length = hour * 60 + minute;
	}

	/* Heure de debut de l'apres-midi */
	if (PARS_read_lexeme_in_file (PARS_file_ptr, PARS_lexeme_0, PARS_LEXEME_MAX_LEN,
											&PARS_current_line, TRUE))
	{
		return;
	}
	if (PARS_get_time (PARS_lexeme_0, &hour, &minute) == 0)
	{
		MAIN_pm_start_h = hour;
		MAIN_pm_start_m = minute;
	}

	/* Longueur de l'apres-midi */
	if (PARS_read_lexeme_in_file (PARS_file_ptr, PARS_lexeme_0, PARS_LEXEME_MAX_LEN,
											&PARS_current_line, TRUE))
	{
		return;
	}
	if (PARS_get_time (PARS_lexeme_0, &hour, &minute) == 0)
	{
		MAIN_pm_length = hour * 60 + minute;
	}

	/* Premier jour de la semaine */
	if (PARS_read_lexeme_in_file (PARS_file_ptr, PARS_lexeme_0, PARS_LEXEME_MAX_LEN,
											&PARS_current_line, TRUE))
	{
		return;
	}
	result = PARS_get_day (PARS_lexeme_0);
	if (result >= 0)
	{
		MAIN_first_day = result;
	}

	/* Dernier jour de la semaine */
	if (PARS_read_lexeme_in_file (PARS_file_ptr, PARS_lexeme_0, PARS_LEXEME_MAX_LEN,
											&PARS_current_line, TRUE))
	{
		return;
	}
	result = PARS_get_day (PARS_lexeme_0);
	if (result >= 0)
	{
		MAIN_last_day = result;
	}

	/* Granularite du placement des cours */
	if (PARS_read_lexeme_in_file (PARS_file_ptr, PARS_lexeme_0, PARS_LEXEME_MAX_LEN,
											&PARS_current_line, TRUE))
	{
		return;
	}
	if (PARS_get_time (PARS_lexeme_0, &hour, &minute) == 0)
	{
		MAIN_time_granularity = hour * 60 + minute;
	}
}



/*==========================================================================*/
/*      Nom: PARS_read_key_holiday                                          */
/*      Description: Lit la cle ":holiday".                                 */
/*==========================================================================*/

void	PARS_read_key_holiday (void)
{
	void		*temp_ptr;
	int		day;
	int		month;
	char		holiday_name_0 [PARS_LEXEME_MAX_LEN+1];

	for ( ; ; )
	{
		/* Nom du jour ferie */
		if (PARS_read_lexeme_in_file (PARS_file_ptr, holiday_name_0, PARS_LEXEME_MAX_LEN,
												&PARS_current_line, TRUE))
		{
			return;
		}

		if (PARS_string_compare (holiday_name_0, ":") == 0)
		{
			break;
		}

		/* Creation du jour ferie */
		if (MAIN_nbr_holidays <= 0)
		{
			MAIN_holiday_ptr = (PARM_HOLIDAY *) malloc (MAIN_OBJECT_ARRAY_GRANUL * sizeof (*MAIN_holiday_ptr));
			if (MAIN_holiday_ptr == NULL)
			{
				fprintf (stderr, "Error (line %ld): Cannot allocate memory to store this building.\n",
							PARS_current_line);
				PARS_nbr_errors ++;
				return;
			}
		}

		else if (MAIN_nbr_holidays % MAIN_OBJECT_ARRAY_GRANUL == 0)
		{
			temp_ptr = realloc (MAIN_holiday_ptr,
										 (MAIN_nbr_holidays + MAIN_OBJECT_ARRAY_GRANUL)
									  * sizeof (*MAIN_holiday_ptr));
			if (temp_ptr == NULL)
			{
				fprintf (stderr, "Error (line %ld): Cannot allocate memory to store this holiday.\n",
							PARS_current_line);
				PARS_nbr_errors ++;
				return;
			}
			MAIN_holiday_ptr = (PARM_HOLIDAY *) temp_ptr;
		}

		MAIN_holiday_ptr [MAIN_nbr_holidays].name_0 = strdup (holiday_name_0);
		if (MAIN_holiday_ptr [MAIN_nbr_holidays].name_0 == NULL)
		{
				fprintf (stderr, "Error (line %ld): Cannot allocate memory to store holiday name.\n",
							PARS_current_line);
				PARS_nbr_errors ++;
				return;
		}

		/* Jour */
		if (PARS_read_lexeme_in_file (PARS_file_ptr, PARS_lexeme_0, PARS_LEXEME_MAX_LEN,
												&PARS_current_line, TRUE))
		{
			return;
		}
		day = atoi (PARS_lexeme_0);
		if (day < 1 || day > 31)
		{
			fprintf (stderr, "Error (line %ld): %d is not a valid day.\n",
						PARS_current_line, day);
			PARS_nbr_errors ++;
		}
		MAIN_holiday_ptr [MAIN_nbr_holidays].day = day - 1;


		/* Mois */
		if (PARS_read_lexeme_in_file (PARS_file_ptr, PARS_lexeme_0, PARS_LEXEME_MAX_LEN,
												&PARS_current_line, TRUE))
		{
			return;
		}
		month = atoi (PARS_lexeme_0);
		if (month < 1 || month > 12)
		{
			fprintf (stderr, "Error (line %ld): %d is not a valid month.\n",
						PARS_current_line, month);
			PARS_nbr_errors ++;
		}
		MAIN_holiday_ptr [MAIN_nbr_holidays].month = month - 1;

		MAIN_nbr_holidays ++;
	}
}



/*==========================================================================*/
/*      Nom: PARS_read_key_building                                         */
/*      Description: Lit la cle ":building".                                */
/*==========================================================================*/

void	PARS_read_key_building (void)
{
	signed int	building;
	char		building_name_0 [PARS_LEXEME_MAX_LEN+1];

	for ( ; ; )
	{
		/* Nom du building */
		if (PARS_read_lexeme_in_file (PARS_file_ptr, building_name_0, PARS_LEXEME_MAX_LEN,
											&PARS_current_line, TRUE))
		{
			return;
		}

		if (PARS_string_compare (building_name_0, ":") == 0)
		{
			break;
		}

		/* Creation du building */
		building = PARS_get_building_nbr (building_name_0, TRUE);
		if (building < 0)
		{
			fprintf (stderr, "Error (line %ld): Cannot allocate memory to store this building.\n",
						PARS_current_line);
			PARS_nbr_errors ++;
			return;
		}

		PARS_defined_building_flag_ptr [building] = TRUE;
	}
}



/*==========================================================================*/
/*      Nom: PARS_read_key_class                                            */
/*      Description: Lit la cle ":class".                                   */
/*==========================================================================*/

void	PARS_read_key_class (void)
{
	int		subdiv;
	int		type;
	int		number;
	int		duration;
	int		hour;
	int		minute;
	signed int	section;
	signed int	subject;
	signed int	period;
	char		section_name_0 [PARS_LEXEME_MAX_LEN+1];
	char		subject_name_0 [PARS_LEXEME_MAX_LEN+1];
	char		period_name_0 [PARS_LEXEME_MAX_LEN+1];

/*______________________________________________
 *
 * Lecture des parametres
 *______________________________________________
 */

	/* Nom de la section */
	if (PARS_read_lexeme_in_file (PARS_file_ptr, section_name_0, PARS_LEXEME_MAX_LEN,
											&PARS_current_line, TRUE))
	{
		return;
	}

	/* Nombre de subdivisions de la classe */
	if (PARS_read_lexeme_in_file (PARS_file_ptr, PARS_lexeme_0, PARS_LEXEME_MAX_LEN,
											&PARS_current_line, TRUE))
	{
		return;
	}
	if (PARS_string_compare (PARS_lexeme_0, "auto") == 0)
	{
		subdiv = 0;
	}
	else
	{
		subdiv = atoi (PARS_lexeme_0);
		if (subdiv <= 0)
		{
			fprintf (stderr, "Error (line %ld): %d is not a valid subdivision of a class.\n",
						PARS_current_line, subdiv);
			PARS_nbr_errors ++;
		}
	}

	/* Matiere */
	if (PARS_read_lexeme_in_file (PARS_file_ptr, subject_name_0, PARS_LEXEME_MAX_LEN,
											&PARS_current_line, TRUE))
	{
		return;
	}

	/* Type */
	if (PARS_read_lexeme_in_file (PARS_file_ptr, PARS_lexeme_0, PARS_LEXEME_MAX_LEN,
											&PARS_current_line, TRUE))
	{
		return;
	}
	type = PARS_get_type (PARS_lexeme_0);

	if (   type == PARM_CLASS_TYPE_LECT
		 && subdiv > 1)
	{
		fprintf (stderr, "Warning (line %ld): Lecture classes can't be divided.\n",
					PARS_current_line);
		PARS_nbr_warnings ++;
		subdiv = 1;
	}

	/* Nom de la periode */
	if (PARS_read_lexeme_in_file (PARS_file_ptr, period_name_0, PARS_LEXEME_MAX_LEN,
											&PARS_current_line, TRUE))
	{
		return;
	}

	/* Nombre de cours */
	if (PARS_read_lexeme_in_file (PARS_file_ptr, PARS_lexeme_0, PARS_LEXEME_MAX_LEN,
											&PARS_current_line, TRUE))
	{
		return;
	}
	number = atoi (PARS_lexeme_0);
	if (number <= 0)
	{
		fprintf (stderr, "Error (line %ld): %d is not a valid number of classes.\n",
					PARS_current_line, number);
		PARS_nbr_errors ++;
	}

	/* Duree de chaque cours */
	if (PARS_read_lexeme_in_file (PARS_file_ptr, PARS_lexeme_0, PARS_LEXEME_MAX_LEN,
											&PARS_current_line, TRUE))
	{
		return;
	}
	if (PARS_get_time (PARS_lexeme_0, &hour, &minute) == 0)
	{
		duration = hour * 60 + minute;
	}
	if (duration <= 0)
	{
		fprintf (stderr, "Error (line %ld): %d:%d is not a valid duration for a class.\n",
					PARS_current_line, hour, minute);
		PARS_nbr_errors ++;
	}

	/* Derniere verification */
	subject = PARS_get_subject_nbr (subject_name_0, TRUE);
	section = PARS_get_section_nbr (section_name_0, TRUE);
	if (   subdiv < 0
		 || type < 0
		 || number < 0
		 || duration < 0
		 || subject < 0
		 || section < 0)
	{
		fprintf (stderr, "Error (line %ld): Cannot define this class.\n",
					PARS_current_line);
		PARS_nbr_errors ++;
		return;
	}

	period = PARS_get_period_nbr (period_name_0, section, TRUE);
	if (period < 0)
	{
		fprintf (stderr, "Error (line %ld): Cannot define this class.\n",
					PARS_current_line);
		PARS_nbr_errors ++;
		return;
	}

/*______________________________________________
 *
 * Construction du cours
 *______________________________________________
 */

	/* Alloue de la memoire pour un nouvel objet dans le tableau */
	if (PARS_allocate_mem_for_new_object ((void **) &MAIN_class_ptr,
													  NULL,
													  MAIN_nbr_classes,
													  sizeof (*MAIN_class_ptr)))
	{
		fprintf (stderr, "Error (line %ld): Cannot allocate memory to store this class.\n",
					PARS_current_line);
		PARS_nbr_errors ++;
		return;
	}

	/* Initialise la structure */
	MAIN_class_ptr [MAIN_nbr_classes].section = section;
	MAIN_class_ptr [MAIN_nbr_classes].nbr_subdiv = subdiv;
	MAIN_class_ptr [MAIN_nbr_classes].subject = subject;
	MAIN_class_ptr [MAIN_nbr_classes].type = type;
	MAIN_class_ptr [MAIN_nbr_classes].period = period;
	MAIN_class_ptr [MAIN_nbr_classes].nbr_classes = number;
	MAIN_class_ptr [MAIN_nbr_classes].duration = duration;
	MAIN_nbr_classes ++;
}



/*==========================================================================*/
/*      Nom: PARS_read_key_period                                           */
/*      Description: Lit la cle ":period".                                  */
/*==========================================================================*/

void	PARS_read_key_period (void)
{
	long		start_week;
	int		duration;
	int		nbr_subjects;
	signed int	section;
	signed int	period;
	signed int	subject;
	char		section_name_0 [PARS_LEXEME_MAX_LEN+1];
	char		period_name_0 [PARS_LEXEME_MAX_LEN+1];
	char		half_day_name_0 [PARS_LEXEME_MAX_LEN+1];
	char		subject_name_0 [PARS_LEXEME_MAX_LEN+1];
	void		*temp_ptr;
	PARM_HALF_DAY	*half_day_ptr;

	/* Nom de la periode */
	if (PARS_read_lexeme_in_file (PARS_file_ptr, period_name_0, PARS_LEXEME_MAX_LEN,
											&PARS_current_line, TRUE))
	{
		return;
	}

	/* Nom de la section */
	if (PARS_read_lexeme_in_file (PARS_file_ptr, section_name_0, PARS_LEXEME_MAX_LEN,
											&PARS_current_line, TRUE))
	{
		return;
	}
	section =  PARS_get_section_nbr (section_name_0, TRUE);
	if (section < 0)
	{
		fprintf (stderr, "Error (line %ld): Cannot define this period.\n",
					PARS_current_line);
		PARS_nbr_errors ++;
		return;
	}

	period = PARS_get_period_nbr (period_name_0, section, TRUE);
	if (period < 0)
	{
		fprintf (stderr, "Error (line %ld): Cannot define this period.\n",
					PARS_current_line);
		PARS_nbr_errors ++;
		return;
	}

	/* Date de debut */
	if (PARS_read_lexeme_in_file (PARS_file_ptr, PARS_lexeme_0, PARS_LEXEME_MAX_LEN,
											&PARS_current_line, TRUE))
	{
		return;
	}
	if (PARS_get_date (PARS_lexeme_0, &start_week))
	{
		return;
	}
	MAIN_period_ptr [period].start_week = start_week;

	/* Duree */
	if (PARS_read_lexeme_in_file (PARS_file_ptr, PARS_lexeme_0, PARS_LEXEME_MAX_LEN,
											&PARS_current_line, TRUE))
	{
		return;
	}
	duration = atoi (PARS_lexeme_0);
	if (duration < 1)
	{
		fprintf (stderr, "Error (line %ld): %d is not a valid period duration.\n",
					PARS_current_line, duration);
		PARS_nbr_errors ++;
	}
	MAIN_period_ptr [period].duration = duration;

	/* Demi-journees reservees */
	for ( ; ; )
	{
		if (PARS_read_lexeme_in_file (PARS_file_ptr, half_day_name_0, PARS_LEXEME_MAX_LEN,
												&PARS_current_line, TRUE))
		{
			return;
		}

		if (PARS_string_compare (half_day_name_0, ":") == 0)
		{
			break;
		}

		/* Reserve de la memoire pour la demi-journee */
		if (MAIN_nbr_half_days == 0)
		{
			MAIN_half_day_ptr = (PARM_HALF_DAY *) malloc (  MAIN_OBJECT_ARRAY_GRANUL
																		 * sizeof (*MAIN_half_day_ptr));
			if (MAIN_half_day_ptr == NULL)
			{
				fprintf (stderr, "Error (line %ld): Cannot allocate memory to store this half-day.\n",
							PARS_current_line);
				PARS_nbr_errors ++;
				return;
			}
		}
		else if (MAIN_nbr_half_days % MAIN_OBJECT_ARRAY_GRANUL == 0)
		{
			temp_ptr = realloc (MAIN_half_day_ptr,
										 (MAIN_nbr_half_days + MAIN_OBJECT_ARRAY_GRANUL)
									  * sizeof (*MAIN_half_day_ptr));
			if (temp_ptr == NULL)
			{
				fprintf (stderr, "Error (line %ld): Cannot allocate memory to store this half-day.\n",
							PARS_current_line);
				PARS_nbr_errors ++;
				return;
			}
			MAIN_half_day_ptr = (PARM_HALF_DAY *) temp_ptr;
		}
		half_day_ptr = MAIN_half_day_ptr + MAIN_nbr_half_days;
		MAIN_nbr_half_days ++;
		half_day_ptr->period = period;

		/* Nom */
		half_day_ptr->name_0 = strdup (half_day_name_0);
		if (half_day_ptr->name_0 == NULL)
		{
			fprintf (stderr, "Error (line %ld): Cannot allocate memory to store half-day name.\n",
						PARS_current_line);
			PARS_nbr_errors ++;
			return;
		}

		/* Jour de la semaine */
		if (PARS_read_lexeme_in_file (PARS_file_ptr, PARS_lexeme_0, PARS_LEXEME_MAX_LEN,
												&PARS_current_line, TRUE))
		{
			return;
		}
		half_day_ptr->day = PARS_get_day (PARS_lexeme_0);
		if (half_day_ptr->day < 0)
		{
			return;
		}

		/* Demi-journee */
		if (PARS_read_lexeme_in_file (PARS_file_ptr, PARS_lexeme_0, PARS_LEXEME_MAX_LEN,
												&PARS_current_line, TRUE))
		{
			return;
		}
		if (   PARS_string_compare (PARS_lexeme_0, "am") == 0
			 || PARS_string_compare (PARS_lexeme_0, "1") == 0)
		{
			half_day_ptr->half = 0;
		}
		else if (   PARS_string_compare (PARS_lexeme_0, "pm") == 0
					|| PARS_string_compare (PARS_lexeme_0, "2") == 0)
		{
			half_day_ptr->half = 1;
		}
		else
		{
			fprintf (stderr, "Error (line %ld): %s is not a valid half-day parameter.\n",
						PARS_current_line, PARS_lexeme_0);
			PARS_nbr_errors ++;
			return;
		}

		/* Nombre de pieces reservees */
		if (PARS_read_lexeme_in_file (PARS_file_ptr, PARS_lexeme_0, PARS_LEXEME_MAX_LEN,
												&PARS_current_line, TRUE))
		{
			return;
		}
		half_day_ptr->nbr_reserved_rooms = atoi (PARS_lexeme_0);
		if (half_day_ptr->nbr_reserved_rooms < 0)
		{
			fprintf (stderr, "Error (line %ld): %d is not a number of reserved rooms.\n",
						PARS_current_line, half_day_ptr->nbr_reserved_rooms);
			PARS_nbr_errors ++;
			return;
		}

		/* Type des salles reservees */
		if (half_day_ptr->nbr_reserved_rooms > 0)
		{
			if (PARS_read_lexeme_in_file (PARS_file_ptr, PARS_lexeme_0, PARS_LEXEME_MAX_LEN,
													&PARS_current_line, TRUE))
			{
				return;
			}
			half_day_ptr->reserved_room_type = PARS_get_type (PARS_lexeme_0);
			if (half_day_ptr->reserved_room_type < 0)
			{
				return;
			}

			/* Matieres des labos */
			if (half_day_ptr->reserved_room_type == PARM_CLASS_TYPE_LAB)
			{
				for (nbr_subjects = 0; ; nbr_subjects ++)
				{
					if (PARS_read_lexeme_in_file (PARS_file_ptr, subject_name_0, PARS_LEXEME_MAX_LEN,
															&PARS_current_line, TRUE))
					{
						return;
					}

					if (PARS_string_compare (subject_name_0, "::") == 0)
					{
						break;
					}

					if (nbr_subjects < PARM_MAX_SUBJECTS_PER_LAB)
					{
						subject = PARS_get_subject_nbr (subject_name_0, TRUE);
						if (subject < 0)
						{
							fprintf (stderr, "Error (line %ld): Cannot define this period.\n",
										PARS_current_line);
							PARS_nbr_errors ++;
							return;
						}
						half_day_ptr->lab [nbr_subjects] = subject;
					}
					else
					{
						fprintf (stderr, "Error (line %ld): Too many subjects for this lab.\n",
									PARS_current_line);
						PARS_nbr_errors ++;
					}
				}

				nbr_subjects = MIN (nbr_subjects, PARM_MAX_SUBJECTS_PER_LAB);
				half_day_ptr->nbr_subjects = nbr_subjects;
			}
		}
	}

	PARS_defined_period_flag_ptr [period] = TRUE;
}



/*==========================================================================*/
/*      Nom: PARS_read_key_room                                             */
/*      Description: Lit la cle ":room".                                    */
/*==========================================================================*/

void	PARS_read_key_room (void)
{
	int		capacity;
	int		type;
	int		nbr_subjects;
	signed int	building;
	signed int	subject;
	int		tab [PARM_MAX_SUBJECTS_PER_LAB];
	char		subject_name_0 [PARS_LEXEME_MAX_LEN+1];
	char		building_name_0 [PARS_LEXEME_MAX_LEN+1];
	char		room_name_0 [PARS_LEXEME_MAX_LEN+1];
	char		*room_name_2_0;

/*______________________________________________
 *
 * Lecture des parametres
 *______________________________________________
 */

	/* Nom de la salle */
	if (PARS_read_lexeme_in_file (PARS_file_ptr, room_name_0, PARS_LEXEME_MAX_LEN,
											&PARS_current_line, TRUE))
	{
		return;
	}

	/* Capacite de la salle */
	if (PARS_read_lexeme_in_file (PARS_file_ptr, PARS_lexeme_0, PARS_LEXEME_MAX_LEN,
											&PARS_current_line, TRUE))
	{
		return;
	}
	capacity = atoi (PARS_lexeme_0);
	if (capacity <= 0)
	{
		fprintf (stderr, "Error (line %ld): %d is not a valid room capacity.\n",
					PARS_current_line, capacity);
		PARS_nbr_errors ++;
	}

	/* Batiment */
	if (PARS_read_lexeme_in_file (PARS_file_ptr, building_name_0, PARS_LEXEME_MAX_LEN,
											&PARS_current_line, TRUE))
	{
		return;
	}
	building = PARS_get_building_nbr (building_name_0, TRUE);

	/* Type de cours */
	if (PARS_read_lexeme_in_file (PARS_file_ptr, PARS_lexeme_0, PARS_LEXEME_MAX_LEN,
											&PARS_current_line, TRUE))
	{
		return;
	}
	type = PARS_get_type (PARS_lexeme_0);
	nbr_subjects = 0;

	/* Matieres du labo */
	if (type == PARM_CLASS_TYPE_LAB)
	{
		for ( ; ; nbr_subjects ++)
		{
			if (PARS_read_lexeme_in_file (PARS_file_ptr, subject_name_0, PARS_LEXEME_MAX_LEN,
													&PARS_current_line, TRUE))
			{
				return;
			}

			if (PARS_string_compare (subject_name_0, ":") == 0)
			{
				break;
			}

			if (nbr_subjects < PARM_MAX_SUBJECTS_PER_LAB)
			{
				subject = PARS_get_subject_nbr (subject_name_0, TRUE);
				if (subject < 0)
				{
					fprintf (stderr, "Error (line %ld): Cannot define this room.\n",
								PARS_current_line);
					PARS_nbr_errors ++;
					return;
				}
				tab [nbr_subjects] = subject;
			}
			else
			{
				fprintf (stderr, "Error (line %ld): Too many subjects for this lab.\n",
							PARS_current_line);
				PARS_nbr_errors ++;
			}
		}
		nbr_subjects = MIN (nbr_subjects, PARM_MAX_SUBJECTS_PER_LAB);
	}

/*______________________________________________
 *
 * Construction de la salle
 *______________________________________________
 */

	/* Alloue de la memoire pour un nouvel objet dans le tableau */
	if (PARS_allocate_mem_for_new_object ((void **) &MAIN_room_ptr,
													  NULL,
													  MAIN_nbr_rooms,
													  sizeof (*MAIN_room_ptr)))
	{
		fprintf (stderr, "Error (line %ld): Cannot allocate memory to store this room.\n",
					PARS_current_line);
		PARS_nbr_errors ++;
		return;
	}

	room_name_2_0 = strdup (room_name_0);
	if (room_name_2_0 == NULL)
	{
		fprintf (stderr, "Error (line %ld): Cannot allocate memory to store room name.\n",
					PARS_current_line);
		PARS_nbr_errors ++;
		return;
	}

	/* Initialise la structure */
	MAIN_room_ptr [MAIN_nbr_rooms].name_0 = room_name_2_0;
	MAIN_room_ptr [MAIN_nbr_rooms].capacity = capacity;
	MAIN_room_ptr [MAIN_nbr_rooms].building = building;
	MAIN_room_ptr [MAIN_nbr_rooms].type = type;
	MAIN_room_ptr [MAIN_nbr_rooms].nbr_subjects = nbr_subjects;
	while (nbr_subjects > 0)
	{
		nbr_subjects --;
		MAIN_room_ptr [MAIN_nbr_rooms].lab [nbr_subjects] = tab [nbr_subjects];
	}
	MAIN_nbr_rooms ++;
}



/*==========================================================================*/
/*      Nom: PARS_read_key_section                                          */
/*      Description: Lit la cle ":section".                                 */
/*==========================================================================*/

void	PARS_read_key_section (void)
{
	int		nbr_students;
	signed int	section;
	char		section_name_0 [PARS_LEXEME_MAX_LEN+1];

/*______________________________________________
 *
 * Lecture des parametres
 *______________________________________________
 */

	/* Nom de la section */
	if (PARS_read_lexeme_in_file (PARS_file_ptr, section_name_0, PARS_LEXEME_MAX_LEN,
											&PARS_current_line, TRUE))
	{
		return;
	}

	/* Nombre d'eleves */
	if (PARS_read_lexeme_in_file (PARS_file_ptr, PARS_lexeme_0, PARS_LEXEME_MAX_LEN,
											&PARS_current_line, TRUE))
	{
		return;
	}
	nbr_students = atoi (PARS_lexeme_0);
	if (nbr_students <= 0)
	{
		fprintf (stderr, "Error (line %ld): %d is not a valid number of students.\n",
					PARS_current_line, nbr_students);
		PARS_nbr_errors ++;
	}

/*______________________________________________
 *
 * Construction de la section
 *______________________________________________
 */

	/* Alloue de la memoire pour un nouvel objet dans le tableau */
	section = PARS_get_section_nbr (section_name_0, TRUE);
	if (section < 0)
	{
		fprintf (stderr, "Error (line %ld): Cannot define this section.\n",
					PARS_current_line);
		PARS_nbr_errors ++;
		return;
	}

	/* Initialise la structure */
	MAIN_section_ptr [section].nbr_students = nbr_students;
	PARS_defined_section_flag_ptr [section] = TRUE;
}



/*==========================================================================*/
/*      Nom: PARS_read_key_teacher                                          */
/*      Description: Lit la cle ":teacher".                                 */
/*==========================================================================*/

void	PARS_read_key_teacher (void)
{
	int		nbr_sections;
	signed int	subject;
	signed int	section;
	int		tab_subj [PARM_MAX_SECTIONS_BY_TEACHER];
	int		tab_sect [PARM_MAX_SECTIONS_BY_TEACHER];
	char		teacher_name_0 [PARS_LEXEME_MAX_LEN+1];
	char		subject_name_0 [PARS_LEXEME_MAX_LEN+1];
	char		section_name_0 [PARS_LEXEME_MAX_LEN+1];
	char		*teacher_name_2_0;

/*______________________________________________
 *
 * Lecture des parametres
 *______________________________________________
 */

	/* Nom du professeur */
	if (PARS_read_lexeme_in_file (PARS_file_ptr, teacher_name_0, PARS_LEXEME_MAX_LEN,
											&PARS_current_line, TRUE))
	{
		return;
	}

	/* Sections dans lesquelles il enseigne */
	for (nbr_sections = 0; ; nbr_sections ++)
	{
		if (PARS_read_lexeme_in_file (PARS_file_ptr, subject_name_0, PARS_LEXEME_MAX_LEN,
												&PARS_current_line, TRUE))
		{
			return;
		}

		if (PARS_string_compare (subject_name_0, ":") == 0)
		{
			break;
		}

		/* Nom de la matiere enseignee */
		if (PARS_read_lexeme_in_file (PARS_file_ptr, section_name_0, PARS_LEXEME_MAX_LEN,
												&PARS_current_line, TRUE))
		{
			return;
		}

		if (nbr_sections < PARM_MAX_SECTIONS_BY_TEACHER)
		{
			subject = PARS_get_subject_nbr (subject_name_0, TRUE);
			section = PARS_get_section_nbr (section_name_0, TRUE);
			if (section < 0 || subject < 0)
			{
				fprintf (stderr, "Error (line %ld): Cannot define this teacher.\n",
							PARS_current_line);
				PARS_nbr_errors ++;
				return;
			}
			tab_subj [nbr_sections] = subject;
			tab_sect [nbr_sections] = section;
		}
		else
		{
			fprintf (stderr, "Error (line %ld): Too many sections for this teacher.\n",
						PARS_current_line);
			PARS_nbr_errors ++;
		}
	}
	nbr_sections = MIN (nbr_sections, PARM_MAX_SECTIONS_BY_TEACHER);

/*______________________________________________
 *
 * Construction du prof
 *______________________________________________
 */

	/* Alloue de la memoire pour un nouvel objet dans le tableau */
	if (PARS_allocate_mem_for_new_object ((void **) &MAIN_teacher_ptr,
													  NULL,
													  MAIN_nbr_teachers,
													  sizeof (*MAIN_teacher_ptr)))
	{
		fprintf (stderr, "Error (line %ld): Cannot allocate memory to store this teacher.\n",
					PARS_current_line);
		PARS_nbr_errors ++;
		return;
	}

	teacher_name_2_0 = strdup (teacher_name_0);
	if (teacher_name_2_0 == NULL)
	{
		fprintf (stderr, "Error (line %ld): Cannot allocate memory to store teacher name.\n",
					PARS_current_line);
		PARS_nbr_errors ++;
		return;
	}

	/* Initialise la structure */
	MAIN_teacher_ptr [MAIN_nbr_teachers].name_0 = teacher_name_2_0;
	MAIN_teacher_ptr [MAIN_nbr_teachers].nbr_sections = nbr_sections;
	while (nbr_sections > 0)
	{
		nbr_sections --;
		MAIN_teacher_ptr [MAIN_nbr_teachers].section [nbr_sections].subject = tab_subj [nbr_sections];
		MAIN_teacher_ptr [MAIN_nbr_teachers].section [nbr_sections].section = tab_sect [nbr_sections];
	}
	MAIN_nbr_teachers ++;
}






/*==========================================================================*/
/*      Nom:                                                                */
/*      Description:                                                        */
/*      Parametres en entree:                                               */
/*      Parametres en sortie:                                               */
/*      Parametres en entree/sortie:                                        */
/*      Retour:                                                             */
/*==========================================================================*/
