/*****************************************************************************
*                                                                            *
*       SOLVER.C                                                             *
*       Creation d'un emploi du temps a partir des parametres                *
*                                                                            *
*       Laurent de Soras                                                     *
*       Eric Lalitte                                                         *
*       A1B7E                                                                *
*                                                                            *
* Tab = 3                                                                    *
*****************************************************************************/



/*\\\ FICHIERS INCLUDE \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*/

#include <stdio.h>
#include <stdlib.h>
#include <stdarg.h>
#include	<math.h>
#include <string.h>
#include	<ctype.h>
#include	<limits.h>

#include "param.h"
#include	"solver.h"
#include	"main.h"



/*\\\ CONSTANTES ET MACROS PRIVEES \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*/



/*\\\ TYPES & STRUCTURES PRIVEES \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*/

typedef struct SOLV_Point
{
	int		point;	/* Instant de debut dans la demi-journee */
	struct SOLV_Point	*next_ptr;	/* Pointeur sur l'intervalle suivant (NULL si aucun) */
} SOLV_POINT;

typedef	SOLV_POINT	*SOLV_POINT_WEEK [7] [2];



typedef struct
{
	int		class_nbr;		/* Numero du cours */
	int		subdiv;			/* Numero de subdivision  (0 a n-1) */
	signed int	tutorial;	/* Numero du TD associe (-1 si aucun) */
	long		priority;		/* Priorite du cours */
} SOLV_CLASS_ORDER;



typedef struct
{
	int		week;
	int		day;
	int		half;
	int		date;
} SOLV_SESSION;



/*\\\ PROTOTYPES DES FONCTIONS PRIVEES \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*/

/*--------------------------------------------------------------------------*/
/*      Initialisations et pre-machages                                     */
/*--------------------------------------------------------------------------*/

void	SOLV_init_mem_for_plannings (void);
void	SOLV_restore_planning (void);
void	SOLV_reset_half_day (PARM_PLANNING **planning_ptr_ptr);
void	SOLV_init_sections (void);
void	SOLV_init_classes (void);
void	SOLV_find_class_subdiv (void);
void	SOLV_init_teacher_activity (void);
void	SOLV_init_section_activity (void);
void	SOLV_init_room_activity (void);
void	SOLV_compute_class_priority (void);
void	SOLV_init_plannings (void);
void	SOLV_eliminate_holidays (void);
void	SOLV_eliminate_reserved_half_days (void);
void	SOLV_sort_classes (void);

/*--------------------------------------------------------------------------*/
/*      Resolution de l'emploi du temps                                     */
/*--------------------------------------------------------------------------*/

signed int	SOLV_organize_classes (void);
signed int	SOLV_make_list_for_class (SOLV_POINT_WEEK **tab_point_ptr_ptr_ptr, int class_nbr, int nbr_groups, signed int subdiv);
signed int	SOLV_make_list_one_period (SOLV_POINT_WEEK **tab_point_ptr_ptr_ptr, int class_nbr, signed int subdiv);
signed int	SOLV_make_list_one_week (SOLV_POINT_WEEK tab_point_ptr, int class_nbr, long week, signed int subdiv);
signed int	SOLV_choose_schedule (SOLV_SESSION session_tab_ptr [], SOLV_POINT_WEEK *tab_point_ptr_ptr, int period_len, int nbr_sessions, int class_nbr);
signed int	SOLV_reserve_rooms_and_teachers (SOLV_SESSION session_tab_ptr [], int class_nbr, int nbr_groups, int nbr_sessions, signed int subdiv);
PARM_PLANNING	*SOLV_reserve_room (PARM_PLANNING **planning_ptr_ptr, int room, int start_time, int duration);
signed int	SOLV_test_interval (PARM_PLANNING *planning_ptr, int start, int length);

void	SOLV_random_order (void *tableau_ptr, size_t object_size, long nbr_objects, long key);

/*--------------------------------------------------------------------------*/
/*      Gestion des listes d'horaires                                       */
/*--------------------------------------------------------------------------*/

signed int	SOLV_union_availability_elt (SOLV_POINT **point_ptr_ptr, int point);
signed int	SOLV_union_availability_list (SOLV_POINT_WEEK tab_point_1_ptr, SOLV_POINT_WEEK tab_point_2_ptr);
signed int	SOLV_intersect_availability_list (SOLV_POINT_WEEK tab_point_1_ptr, SOLV_POINT_WEEK tab_point_2_ptr);
signed int	SOLV_create_availability_list (SOLV_POINT_WEEK tab_point_ptr, PARM_PLANNING *(planning [7] [2]), int length, signed int class_nbr, signed int subdiv);
signed int	SOLV_substract_availability_list (SOLV_POINT **point_ptr_ptr, int start, int length);
signed int	SOLV_translate_availability_list (SOLV_POINT_WEEK tab_point_ptr, signed int trans);
signed int	SOLV_delete_availability_list (SOLV_POINT_WEEK tab_point_ptr);



/*\\\ DEFINITION DES VARIABLES EXTERNES \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*/

/* Cle de generation du planning. Elle n'est modifiee que par
	la fonction SOLV_make_planning. */
long	SOLV_planning_key;



/*\\\ VARIABLES PRIVEES \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*/

/* Dans les plannings, les jours feries ont leurs cours qui pointent sur
	ce tableau (a remplir avec les jours feries). Le membre subdiv indique
	le numero du jour ferie. */
PARM_PLANNING	*SOLV_holiday_planning_ptr;

/* Tableau contenant l'ordre de chaque cours */
SOLV_CLASS_ORDER	*SOLV_class_order_ptr;
int	SOLV_nbr_class_orders;	/* La taille du tableau */

/* Methode de balayage pour le choix des horaires (0, 1 ou 2) */
int	SOLV_schedule_choice_method = 1;



/*\\\ FONCTIONS \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*/

/*==========================================================================*/
/*      Nom: SOLV_make_planning                                             */
/*      Description: Fabrique le planning. Les donnees auront deja dues     */
/*                   etres lues et initialisees.                            */
/*      Parametres en entree:                                               */
/*        - key: numero de la cle de generation (>= 0)                      */
/*        - center: type de methode de centrage des cours dans la semaine.  */
/*      Retour: 0 si tout s'est bien passe.                                 */
/*==========================================================================*/

signed int	SOLV_make_planning (long key, int center)
{
	fprintf (stderr, "Brainstorming (key %04X%04X)...\n",
	         (int) (key >> 16), (int) (key & 0xFFFFL));

	SOLV_planning_key = key;
	SOLV_schedule_choice_method = center;

	/* Classe les cours en fontion de leur facteur de priorite */
	SOLV_sort_classes ();

	/* Organise l'emploi du temps */
	if (SOLV_organize_classes ())
	{
		return (-1);
	}

	return (0);
}



/****************************************************************************/
/*                                                                          */
/*      INITIALISATIONS ET PREMACHAGES                                      */
/*                                                                          */
/****************************************************************************/



/*==========================================================================*/
/*      Nom: SOLV_init_and_check_planning                                   */
/*      Description: Initialise toutes le donnees de l'emploi du temps et   */
/*                   premache le travail.                                   */
/*==========================================================================*/

void	SOLV_init_and_check_planning (void)
{
	fprintf (stderr, "Initializing...\n");

	/* Initialise la memoire pour les plannings */
	SOLV_init_mem_for_plannings ();

	/* Construit les tableaux de recoupement de donnees */
	SOLV_init_sections ();
	SOLV_init_classes ();

	#ifdef DEBUG
	/* Petit check des preattributions cours->profs/salles et sections->cours */
	{
		int	cours;
		int	salle;
		int	prof;
		int	section;

		for (section = 0; section < MAIN_nbr_sections; section ++)
		{
			printf ("Section %s:\n", MAIN_section_ptr [section].name_0);
			for (cours = 0; cours < MAIN_section_ptr [section].nbr_classes; cours ++)
			{
				printf ("\t%s, type %d\n",
						  MAIN_subject_ptr [MAIN_class_ptr [MAIN_section_ptr [section].class_ptr [cours]].subject].name_0,
						  MAIN_class_ptr [MAIN_section_ptr [section].class_ptr [cours]].type);
			}
			printf ("\n");
		}
		for (cours = 0; cours < MAIN_nbr_classes; cours ++)
		{
			printf ("Cours de %s, type %d:\n",
					  MAIN_subject_ptr [MAIN_class_ptr [cours].subject].name_0,
					  MAIN_class_ptr [cours].type);
			for (salle = 0; salle < MAIN_class_ptr [cours].nbr_rooms; salle ++)
			{
				printf ("\t%s\n", MAIN_room_ptr [MAIN_class_ptr [cours].room_ptr [salle]].name_0);
			}
			for (prof = 0; prof < MAIN_class_ptr [cours].nbr_teachers; prof ++)
			{
				printf ("\t%s\n", MAIN_teacher_ptr [MAIN_class_ptr [cours].teacher_ptr [prof]].name_0);
			}
			printf ("\n");
		}
	}
	#endif

	/* Trouve le nombre de divisions pour les cours a division automatique */
	SOLV_find_class_subdiv ();

	/* Calcule les parametres d'activite de chaque entite materielle */
	SOLV_init_teacher_activity ();
	SOLV_init_section_activity ();
	SOLV_init_room_activity ();

	/* Calcule le facteur de priorite de chaque cours */
	SOLV_compute_class_priority ();

	/* Mets les plannings a 0 */
	SOLV_init_plannings ();

	/* Vire les jours feries du planning */
	SOLV_eliminate_holidays ();

	/* Vire les demi-journees reservees */
	SOLV_eliminate_reserved_half_days ();
}



/*==========================================================================*/
/*      Nom: SOLV_restore_planning                                          */
/*      Description: Remet le planning dans l'etat dans lequel il etait     */
/*                   avant l'initialisation et la resolution en liberant    */
/*                   la memoire reservee. Ceci permet de retrouver un       */
/*                   nouvel emploi du temps en reinitialisant et en         */
/*                   relancant la fonction.                                 */
/*==========================================================================*/

void	SOLV_restore_planning (void)
{
	int		count;
	int		week;
	int		day;
	int		half;
	int		nbr_weeks;

	nbr_weeks = (int) (MAIN_last_week - MAIN_first_week + 1);

	/* Libere tous les cours, mais pas le 1/2 journees
		reservees et les jours feries. */
	for (week = 0; week < nbr_weeks; week ++)
	{
		for (day = 0; day < 7; day ++)
		{
			for (half = 0; half < 2; half ++)
			{
				for (count = 0; count < MAIN_nbr_sections; count ++)
				{
					SOLV_reset_half_day (& MAIN_section_ptr [count].planning [week] [day] [half]);
				}

				for (count = 0; count < MAIN_nbr_teachers; count ++)
				{
					SOLV_reset_half_day (& MAIN_teacher_ptr [count].planning [week] [day] [half]);
				}

				for (count = 0; count < MAIN_nbr_rooms; count ++)
				{
					SOLV_reset_half_day (& MAIN_room_ptr [count].planning [week] [day] [half]);
				}
			}
		}
	}

	/* Liberation de l'ordre des cours */
	free (SOLV_class_order_ptr);

}



/*==========================================================================*/
/*      Nom: SOLV_reset_half_day                                            */
/*      Description: Efface tout le contenu d'une demi-journee.             */
/*      Parametres en entree/sortie:                                        */
/*        - planning_ptr_ptr: pointeur sur le pointeur de debut de la liste */
/*                            de la demi-journee.                           */
/*==========================================================================*/

void	SOLV_reset_half_day (PARM_PLANNING **planning_ptr_ptr)
{
	PARM_PLANNING	*planning_ptr;

	planning_ptr = *planning_ptr_ptr;
	while (planning_ptr != NULL)
	{
		/* Cours normal: on le libere */
		if (planning_ptr->type >= 0)
		{
			*planning_ptr_ptr = planning_ptr->next_ptr;
			free (planning_ptr);
		}

		/* 1/2 journee reservee ou jour ferie: on le saute */
		else
		{
			planning_ptr_ptr = &planning_ptr->next_ptr;
		}

		planning_ptr = *planning_ptr_ptr;
	}
}



/*==========================================================================*/
/*      Nom: SOLV_init_mem_for_plannings                                    */
/*      Description: Reserve la memoire pour les tableau de pointeurs de    */
/*                   toutes les demi-journees de toutes les entites         */
/*                   materielles.                                           */
/*==========================================================================*/

void	SOLV_init_mem_for_plannings (void)
{
	long		nbr_weeks;
	int		count;
	void		*mem_ptr;

	nbr_weeks = MAIN_last_week - MAIN_first_week + 1;

	/* Sections */
	for (count = 0; count < MAIN_nbr_sections; count ++)
	{
		mem_ptr = malloc ((size_t) (nbr_weeks * 7 * 2 * sizeof (PARM_PLANNING *)));
		if (mem_ptr == NULL)
		{
			fprintf (stderr, "Error: malloc failed.\n");
			exit (-1);
		}
		MAIN_section_ptr [count].planning = (PARM_PLANNING *((*)[7][2])) mem_ptr;
	}

	/* Salles */
	for (count = 0; count < MAIN_nbr_rooms; count ++)
	{
		mem_ptr = malloc ((size_t) (nbr_weeks * 7 * 2 * sizeof (PARM_PLANNING *)));
		if (mem_ptr == NULL)
		{
			fprintf (stderr, "Error: malloc failed.\n");
			exit (-1);
		}
		MAIN_room_ptr [count].planning = (PARM_PLANNING *((*)[7][2])) mem_ptr;
	}

	/* Profs */
	for (count = 0; count < MAIN_nbr_teachers; count ++)
	{
		mem_ptr = malloc ((size_t) (nbr_weeks * 7 * 2 * sizeof (PARM_PLANNING *)));
		if (mem_ptr == NULL)
		{
			fprintf (stderr, "Error: malloc failed.\n");
			exit (-1);
		}
		MAIN_teacher_ptr [count].planning = (PARM_PLANNING *((*)[7][2])) mem_ptr;
	}
}



/*==========================================================================*/
/*      Nom: SOLV_init_sections                                             */
/*      Description: Etablit le tableau listant tous les cours d'une        */
/*                   section.                                               */
/*==========================================================================*/

void	SOLV_init_sections (void)
{
	int		count;
	int		section;
	int		length;
	int		*temp_ptr;

	/* Reserve de la memoire pour les tableaux de recoupement de donnees. */
	for (count = 0; count < MAIN_nbr_sections; count ++)
	{
		MAIN_section_ptr [count].class_ptr = (int *) malloc (MAIN_OBJECT_ARRAY_GRANUL * sizeof (int));
		if (MAIN_section_ptr [count].class_ptr == NULL)
		{
			fprintf (stderr, "Error: malloc failed.\n");
			exit (-1);
		}
		MAIN_section_ptr [count].nbr_classes = 0;
	}

	/* Liste tous les cours associes a chaque section */
	for (count = 0; count < MAIN_nbr_classes; count ++)
	{
		section = MAIN_class_ptr [count].section;
		length = MAIN_section_ptr [section].nbr_classes;
		if (length % MAIN_OBJECT_ARRAY_GRANUL == 0)
		{
			temp_ptr = (int *) realloc (MAIN_section_ptr [section].class_ptr,
												 (length + MAIN_OBJECT_ARRAY_GRANUL) * sizeof (int));
			if (temp_ptr == NULL)
			{
				fprintf (stderr, "Error: realloc failed.\n");
				exit (-1);
			}
			MAIN_section_ptr [section].class_ptr = temp_ptr;
		}
		MAIN_section_ptr [section].class_ptr [length] = count;
		MAIN_section_ptr [section].nbr_classes ++;
	}
}



/*==========================================================================*/
/*      Nom: SOLV_init_classes                                              */
/*      Description: Etablit le tableau listant pour chaque cours toutes    */
/*                   les salles et tous les professeurs susceptibles de     */
/*                   convenir.                                              */
/*==========================================================================*/

void	SOLV_init_classes (void)
{
	int		count;
	int		section_cnt;
	int		class_cnt;
	int		room_cnt;
	int		subject_cnt;
	int		class_nbr;
	int		section;
	int		subject;
	int		length;
	int		type;
	int		subdiv;
	int		capacity;
	int		next_room_flag;
	int		*temp_ptr;

	/* Reserve de la memoire pour les tableaux de recoupement de donnees. */
	for (count = 0; count < MAIN_nbr_classes; count ++)
	{
		MAIN_class_ptr [count].teacher_ptr = (int *) malloc (MAIN_OBJECT_ARRAY_GRANUL * sizeof (int));
		if (MAIN_class_ptr [count].teacher_ptr == NULL)
		{
			fprintf (stderr, "Error: malloc failed.\n");
			exit (-1);
		}
		MAIN_class_ptr [count].nbr_teachers = 0;

		MAIN_class_ptr [count].room_ptr = (int *) malloc (MAIN_OBJECT_ARRAY_GRANUL * sizeof (int));
		if (MAIN_class_ptr [count].room_ptr == NULL)
		{
			fprintf (stderr, "Error: malloc failed.\n");
			exit (-1);
		}
		MAIN_class_ptr [count].nbr_rooms = 0;
	}

	/* Rassemble les donnees sur les profs */
	for (count = 0; count < MAIN_nbr_teachers; count ++)
	{
		for (section_cnt = 0; section_cnt < MAIN_teacher_ptr [count].nbr_sections; section_cnt ++)
		{
			subject = MAIN_teacher_ptr [count].section [section_cnt].subject;
			section = MAIN_teacher_ptr [count].section [section_cnt].section;

			for (class_cnt = 0; class_cnt < MAIN_section_ptr [section].nbr_classes; class_cnt ++)
			{
				class_nbr = MAIN_section_ptr [section].class_ptr [class_cnt];
				if (MAIN_class_ptr [class_nbr].subject == subject)
				{
					length = MAIN_class_ptr [class_nbr].nbr_teachers;
					if (length % MAIN_OBJECT_ARRAY_GRANUL == 0)
					{
						temp_ptr = (int *) realloc (MAIN_class_ptr [class_nbr].teacher_ptr,
															 (length + MAIN_OBJECT_ARRAY_GRANUL) * sizeof (int));
						if (temp_ptr == NULL)
						{
							fprintf (stderr, "Error: realloc failed.\n");
							exit (-1);
						}
						MAIN_class_ptr [class_nbr].teacher_ptr = temp_ptr;
					}
					MAIN_class_ptr [class_nbr].teacher_ptr [length] = count;
					MAIN_class_ptr [class_nbr].nbr_teachers ++;
				}
			}
		}
	}

	/* Verifie que chaque cours ait au moins un prof */
	for (class_cnt = 0; class_cnt < MAIN_nbr_classes; class_cnt ++)
	{
		if (MAIN_class_ptr [class_cnt].nbr_teachers <= 0)
		{
			fprintf (stderr,
						"Error: too few teacher for %s, section %s.\n",
						MAIN_subject_ptr [MAIN_class_ptr [class_cnt].subject].name_0,
						MAIN_section_ptr [MAIN_class_ptr [class_cnt].section].name_0);
			exit (-1);
		}
	}

	/* Rassemble les donnees sur les salles */
	for (class_cnt = 0; class_cnt < MAIN_nbr_classes; class_cnt ++)
	{
		subject = MAIN_class_ptr [class_cnt].subject;
		type = MAIN_class_ptr [class_cnt].type;
		length = MAIN_class_ptr [class_cnt].nbr_rooms;
		section = MAIN_class_ptr [class_cnt].section;
		subdiv = MAIN_class_ptr [class_cnt].nbr_subdiv;
		if (subdiv == 0)
		{
			capacity = 1;
		}
		else
		{
			capacity = (MAIN_section_ptr [section].nbr_students + subdiv - 1) / subdiv;
		}

		for (room_cnt = 0; room_cnt < MAIN_nbr_rooms; room_cnt ++)
		{
			if (   MAIN_room_ptr [room_cnt].type == type
				 && MAIN_room_ptr [room_cnt].capacity >= capacity)
			{
				if (type == PARM_CLASS_TYPE_LAB)
				{
					next_room_flag = TRUE;
					for (subject_cnt = 0; subject_cnt < MAIN_room_ptr [room_cnt].nbr_subjects; subject_cnt ++)
					{
						if (MAIN_room_ptr [room_cnt].lab [subject_cnt] == subject)
						{
							next_room_flag = FALSE;
							break;
						}
					}

					if (next_room_flag)
					{
						continue;
					}
				}

				if (length % MAIN_OBJECT_ARRAY_GRANUL == 0)
				{
					temp_ptr = (int *) realloc (MAIN_class_ptr [class_cnt].room_ptr,
														 (length + MAIN_OBJECT_ARRAY_GRANUL) * sizeof (int));
					if (temp_ptr == NULL)
					{
						fprintf (stderr, "Error: realloc failed.\n");
						exit (-1);
					}
					MAIN_class_ptr [class_cnt].room_ptr = temp_ptr;
				}
				MAIN_class_ptr [class_cnt].room_ptr [length] = room_cnt;
				length ++;
			}
		}

		MAIN_class_ptr [class_cnt].nbr_rooms = length;

		/* Verifie que chaque cours ait suffisemment de salles */
		if (MAIN_class_ptr [class_cnt].nbr_rooms <= 0)
		{
			fprintf (stderr,
						"Error: too few rooms for %s, section %s.\n",
						MAIN_subject_ptr [MAIN_class_ptr [class_cnt].subject].name_0,
						MAIN_section_ptr [MAIN_class_ptr [class_cnt].section].name_0);
			exit (-1);
		}
	}
}



/*==========================================================================*/
/*      Nom: SOLV_find_class_subdiv                                         */
/*      Description: Trouve pour chaque cours a subdivision automatique     */
/*                   celle qui est le plus adaptee.                         */
/*==========================================================================*/

void	SOLV_find_class_subdiv (void)
{
	int		class_cnt;
	int		nbr_teachers;
	int		nbr_rooms;
	int		indice_1;
	int		indice_2;
	int		end_flag;
	int		val_1;
	int		val_2;
	int		temp;
	int		capacity;
	int		section;
	int		nbr_students;
	int		nbr_subdiv;
	int		*room_ptr;

	for (class_cnt = 0; class_cnt < MAIN_nbr_classes; class_cnt ++)
	{
		if (MAIN_class_ptr [class_cnt].nbr_subdiv <= 0)
		{
			/* Cours magistraux: pas de subdivision */
			if (MAIN_class_ptr [class_cnt].type == PARM_CLASS_TYPE_LECT)
			{
				MAIN_class_ptr [class_cnt].nbr_subdiv = 1;
			}

			/* TD ou TP */
			else
			{
				/* Cherche la Neme plus grande salle, avec N = nombre de
					profs competants pour assurer le cours */
				nbr_teachers = MAIN_class_ptr [class_cnt].nbr_teachers;
				nbr_rooms = MAIN_class_ptr [class_cnt].nbr_rooms;
				room_ptr = (int *) malloc (nbr_rooms * sizeof (*room_ptr));
				if (room_ptr == NULL)
				{
					fprintf (stderr, "Error: malloc failed.\n");
					exit (-1);
				}

				/* Copie du tableau de salles */
				for (indice_1 = 0; indice_1 < nbr_rooms; indice_1 ++)
				{
					room_ptr [indice_1] = MAIN_class_ptr [class_cnt].room_ptr [indice_1];
				}

				/* Tri donc par ordre decroissant nos salles */
				for (indice_1 = nbr_rooms - 1; indice_1 > 0; indice_1 --)
				{
					end_flag = TRUE;

					for (indice_2 = 0; indice_2 < indice_1; indice_2 ++)
					{
						val_1 = MAIN_room_ptr [room_ptr [indice_2]].capacity;
						val_2 = MAIN_room_ptr [room_ptr [indice_2 + 1]].capacity;
						if (val_1 < val_2)
						{
							temp = room_ptr [indice_2];
							room_ptr [indice_2] = room_ptr [indice_2 + 1];
							room_ptr [indice_2 + 1] = temp;
							end_flag = FALSE;
						}
					}

					if (end_flag)
					{
						break;
					}
				}

				/* On a la capacite de la salle */
				capacity = MAIN_room_ptr [room_ptr [MIN (nbr_teachers, nbr_rooms) - 1]].capacity;
				free (room_ptr);

				/* Calcule le nombre de subdivisions et arrondit par exces */
				section = MAIN_class_ptr [class_cnt].section;
				nbr_students = MAIN_section_ptr [section].nbr_students;
				nbr_subdiv = (nbr_students + capacity - 1) / capacity;

				MAIN_class_ptr [class_cnt].nbr_subdiv = nbr_subdiv;

			}
		}
	}
}



/*==========================================================================*/
/*      Nom: SOLV_init_teacher_activity                                     */
/*      Description: Calcule le coefficient d'activite de chaque prof.      */
/*==========================================================================*/

void	SOLV_init_teacher_activity (void)
{
	int		section;
	int		subject;
	int		teacher_cnt;
	int		section_cnt;
	int		class_cnt;
	double	time_cnt;

	/* Scanne chaque prof */
	for (teacher_cnt = 0; teacher_cnt < MAIN_nbr_teachers; teacher_cnt ++)
	{
		time_cnt = 0.0;

		/* Pour chaque section dans laquelle il enseigne */
		for (section_cnt = 0; section_cnt < MAIN_teacher_ptr [teacher_cnt].nbr_sections; section_cnt ++)
		{
			subject = MAIN_teacher_ptr [teacher_cnt].section [section_cnt].subject;
			section = MAIN_teacher_ptr [teacher_cnt].section [section_cnt].section;

			/* Recherche les cours qui correspondent a la matiere/section */
			for (class_cnt = 0; class_cnt < MAIN_nbr_classes; class_cnt ++)
			{
				if (   MAIN_class_ptr [class_cnt].subject == subject
					 && MAIN_class_ptr [class_cnt].section == section)
				{
					time_cnt +=   (double) (  MAIN_class_ptr [class_cnt].duration
													* MAIN_class_ptr [class_cnt].nbr_classes
													* MAIN_class_ptr [class_cnt].nbr_subdiv)
									/ MAIN_class_ptr [class_cnt].nbr_teachers;
				}
			}
		}

		MAIN_teacher_ptr [teacher_cnt].activity = (long) ceil (time_cnt);

		#ifdef DEBUG
		/* Affichage de verification */
		printf ("Prof %s: %ld minutes.\n",
				  MAIN_teacher_ptr [teacher_cnt].name_0,
				  MAIN_teacher_ptr [teacher_cnt].activity);
		#endif
	}
}



/*==========================================================================*/
/*      Nom: SOLV_init_section_activity                                     */
/*      Description: Calcule le coefficient d'activite de chaque section.   */
/*==========================================================================*/

void	SOLV_init_section_activity (void)
{
	int		section_cnt;
	int		class_cnt;
	int		class_nbr;
	long		time_cnt;

	/* Scanne chaque section */
	for (section_cnt = 0; section_cnt < MAIN_nbr_sections; section_cnt ++)
	{
		time_cnt = 0;

		/* Fait le total des heures de cours */
		for (class_cnt = 0; class_cnt < MAIN_section_ptr [section_cnt].nbr_classes; class_cnt ++)
		{
			class_nbr = MAIN_section_ptr [section_cnt].class_ptr [class_cnt];
			time_cnt +=   MAIN_class_ptr [class_nbr].duration
							* MAIN_class_ptr [class_nbr].nbr_classes;
		}

		MAIN_section_ptr [section_cnt].activity = time_cnt;

		#ifdef DEBUG
		/* Affichage de verification */
		printf ("Section %s: %ld minutes.\n",
				  MAIN_section_ptr [section_cnt].name_0,
				  MAIN_section_ptr [section_cnt].activity);
		#endif
	}
}



/*==========================================================================*/
/*      Nom: SOLV_init_room_activity                                        */
/*      Description: Calcule le coefficient d'activite de chaque salle.     */
/*==========================================================================*/

void	SOLV_init_room_activity (void)
{
	int		room_cnt;
	int		class_cnt;
	int		room_cnt_2;
	int		used_flag;
	double	time_cnt;

	/* Scanne chaque salle */
	for (room_cnt = 0; room_cnt < MAIN_nbr_rooms; room_cnt ++)
	{
		time_cnt = 0.0;

		/* Parcours tous les cours pour voir si la salle est
			utilisee par ces cours. */
		for (class_cnt = 0; class_cnt < MAIN_nbr_classes; class_cnt ++)
		{
			used_flag = FALSE;

			/* Regarde dans la liste des salles susceptibles d'accueillir
				le cours si on ne trouve pas notre salle */
			for (room_cnt_2 = 0; room_cnt_2 < MAIN_class_ptr [class_cnt].nbr_rooms; room_cnt_2 ++)
			{
				if (MAIN_class_ptr [class_cnt].room_ptr [room_cnt_2] == room_cnt)
				{
					used_flag = TRUE;
					break;
				}
			}

			if (used_flag)
			{
				time_cnt +=   (double) (  MAIN_class_ptr [class_cnt].duration
												* MAIN_class_ptr [class_cnt].nbr_classes
												* MAIN_class_ptr [class_cnt].nbr_subdiv)
								/ MAIN_class_ptr [class_cnt].nbr_rooms;
			}
		}

		MAIN_room_ptr [room_cnt].activity = (long) floor (time_cnt);

		#ifdef DEBUG
		/* Affichage de verification */
		printf ("Salle %s: %ld minutes.\n",
				  MAIN_room_ptr [room_cnt].name_0,
				  MAIN_room_ptr [room_cnt].activity);
		#endif
	}
}



/*==========================================================================*/
/*      Nom: SOLV_compute_class_priority                                    */
/*      Description: Calcule le degre de priorite de chaque cours, a partir */
/*                   de leur duree et de l'activite des entites susceptibles*/
/*                   d'y etre associees.                                    */
/*==========================================================================*/

void	SOLV_compute_class_priority (void)
{
	int		class_cnt;
	int		class_cnt_2;
	int		room_cnt;
	int		teacher_cnt;
	int		nbr_rooms;
	int		nbr_teachers;
	int		section;
	int		subject;
	int		period;
	int		nbr_weeks;
	long		priority;
	long		priority_1;
	long		priority_2;

	nbr_weeks = (int) (MAIN_last_week - MAIN_first_week + 1);

	/* Commence par mettre a 0 toutes les priorites */
	for (class_cnt = 0; class_cnt < MAIN_nbr_classes; class_cnt ++)
	{
		MAIN_class_ptr [class_cnt].priority = 0;
	}

	/* Scanne chaque cours pour evaluer sa propre priorite. */
	for (class_cnt = 0; class_cnt < MAIN_nbr_classes; class_cnt ++)
	{
		period = MAIN_class_ptr [class_cnt].period;
		priority = MAIN_class_ptr [class_cnt].priority;

		/* Premier critere de priorite: duree du cours */
		priority_1 =   (long)MAIN_class_ptr [class_cnt].duration
						 * MAIN_class_ptr [class_cnt].nbr_classes;

		/* Deuxieme critere: temps d'occupation des differentes parties */
		priority_2 =   MAIN_section_ptr [MAIN_class_ptr [class_cnt].section].activity
						 * MAIN_period_ptr [period].duration
						 / nbr_weeks;
		nbr_rooms = MAIN_class_ptr [class_cnt].nbr_rooms;
		for (room_cnt = 0; room_cnt < nbr_rooms; room_cnt ++)
		{
			priority_2 +=   (long) MAIN_room_ptr [MAIN_class_ptr [class_cnt].room_ptr [room_cnt]].activity
							  * MAIN_class_ptr [class_cnt].nbr_subdiv
							  / (nbr_rooms * nbr_weeks);
		}

		nbr_teachers = MAIN_class_ptr [class_cnt].nbr_teachers;
		for (teacher_cnt = 0; teacher_cnt < nbr_teachers; teacher_cnt ++)
		{
			priority_2 +=   (long) MAIN_teacher_ptr [MAIN_class_ptr [class_cnt].teacher_ptr [teacher_cnt]].activity
							  * MAIN_class_ptr [class_cnt].nbr_subdiv
							  / (nbr_teachers * nbr_weeks);
		}

		/* Equilibre entre les deux criteres */
		priority = priority_1 * 4 + priority_2;

		MAIN_class_ptr [class_cnt].priority = priority;

		/* S'il s'agit d'un TD associe a un cours, on reporte la priorite du
			TD sur le cours. */
		if (MAIN_class_ptr [class_cnt].type == PARM_CLASS_TYPE_TUT)
		{
			section = MAIN_class_ptr [class_cnt].section;
			subject = MAIN_class_ptr [class_cnt].subject;
			for (class_cnt_2 = 0; class_cnt_2 < MAIN_nbr_classes; class_cnt_2 ++)
			{
				if (   MAIN_class_ptr [class_cnt_2].section == section
					 && MAIN_class_ptr [class_cnt_2].subject == subject
					 && MAIN_class_ptr [class_cnt_2].period == period
					 && MAIN_class_ptr [class_cnt_2].type == PARM_CLASS_TYPE_LECT)
				{
					MAIN_class_ptr [class_cnt_2].priority += priority;
					break;
				}
			}
		}
	}

	#ifdef DEBUG
	/* Affichage de controle */
	{
		int		class_cnt;

		for (class_cnt = 0; class_cnt < MAIN_nbr_classes; class_cnt ++)
		{
			printf ("Priorite du cours de %s, type %d, section %s: %ld\n",
					  MAIN_subject_ptr [MAIN_class_ptr [class_cnt].subject].name_0,
					  MAIN_class_ptr [class_cnt].type,
					  MAIN_section_ptr [MAIN_class_ptr [class_cnt].section].name_0,
					  MAIN_class_ptr [class_cnt].priority);
		}
	}
	#endif
}



/*==========================================================================*/
/*      Nom: SOLV_init_plannings                                            */
/*      Description: Mets a 0 tous les pointeurs sur les plannings de       */
/*                   chaque entite.                                         */
/*==========================================================================*/

void	SOLV_init_plannings (void)
{
	int		count;
	long		nbr_weeks;

	nbr_weeks = MAIN_last_week - MAIN_first_week + 1;

	for (count = 0; count < MAIN_nbr_sections; count ++)
	{
		memset (MAIN_section_ptr [count].planning, 0,
				  (size_t) (nbr_weeks * 7 * 2 * sizeof (PARM_PLANNING *)));
	}

	for (count = 0; count < MAIN_nbr_teachers; count ++)
	{
		memset (MAIN_teacher_ptr [count].planning, 0,
				  (size_t) (nbr_weeks * 7 * 2 * sizeof (PARM_PLANNING *)));
	}

	for (count = 0; count < MAIN_nbr_rooms; count ++)
	{
		memset (MAIN_room_ptr [count].planning, 0,
				  (size_t) (nbr_weeks * 7 * 2 * sizeof (PARM_PLANNING *)));
	}
}



/*==========================================================================*/
/*      Nom: SOLV_eliminate_holidays                                        */
/*      Description: Elimine des plannings les jours feries                 */
/*==========================================================================*/

void	SOLV_eliminate_holidays (void)
{
	int		holiday_cnt;
	int		half;
	int		day;
	int		month;
	int		year;
	int		year_2;
	int		count;
	long		week;

	if (MAIN_nbr_holidays <= 0)
	{
		return;
	}

	/* Reserve un tableau de valeurs pour les jours feries */
	SOLV_holiday_planning_ptr = (PARM_PLANNING *) malloc (MAIN_nbr_holidays * sizeof (*SOLV_holiday_planning_ptr));
	if (SOLV_holiday_planning_ptr == NULL)
	{
		fprintf (stderr, "Error: malloc failed.\n");
		exit (-1);
	}

	/* Trouve l'annee qui correspond a la premiere semaine de la premiere periode */
	MAIN_get_date (&day, &month, &year, 0, MAIN_first_week);

	/* Scanne tous les jours feries */
	for (holiday_cnt = 0; holiday_cnt < MAIN_nbr_holidays; holiday_cnt ++)
	{
		SOLV_holiday_planning_ptr [holiday_cnt].section = -1;
		SOLV_holiday_planning_ptr [holiday_cnt].subdiv = holiday_cnt;
		SOLV_holiday_planning_ptr [holiday_cnt].subdiv_tot = -1;
		SOLV_holiday_planning_ptr [holiday_cnt].subject = -1;
		SOLV_holiday_planning_ptr [holiday_cnt].type = -1;
		SOLV_holiday_planning_ptr [holiday_cnt].teacher = -1;
		SOLV_holiday_planning_ptr [holiday_cnt].room = -1;
		SOLV_holiday_planning_ptr [holiday_cnt].start = -1;
		SOLV_holiday_planning_ptr [holiday_cnt].duration = -1;
		SOLV_holiday_planning_ptr [holiday_cnt].next_ptr = NULL;

		year_2 = year;

		/* Parcourt toutes les annees jusqu'a la fin de la derniere periode */
		for ( ; ; )
		{
			/* Convertit le jour ferie de l'annee courante en semaine/jour */
			MAIN_get_absolute_day (&day, &week,
										  MAIN_holiday_ptr [holiday_cnt].day,
										  MAIN_holiday_ptr [holiday_cnt].month,
										  year_2);

			if (week > MAIN_last_week)
			{
				break;
			}

			if (week >= MAIN_first_week)
			{
				for (half = 0; half < 2; half ++)
				{
					/* On retire le jour des plannings de tous les eleves */
					for (count = 0; count < MAIN_nbr_sections; count ++)
					{
						MAIN_section_ptr [count].planning [week - MAIN_first_week] [day] [half] = SOLV_holiday_planning_ptr + holiday_cnt;
					}

					/* On retire le jour des plannings de tous les profs */
					for (count = 0; count < MAIN_nbr_teachers; count ++)
					{
						MAIN_teacher_ptr [count].planning [week - MAIN_first_week] [day] [half] = SOLV_holiday_planning_ptr + holiday_cnt;
					}

					/* On retire le jour des plannings de toutes les salles */
					for (count = 0; count < MAIN_nbr_rooms; count ++)
					{
						MAIN_room_ptr [count].planning [week - MAIN_first_week] [day] [half] = SOLV_holiday_planning_ptr + holiday_cnt;
					}
				}

			}

			year_2 ++;
		}
	}
}



/*==========================================================================*/
/*      Nom: SOLV_eliminate_reserved_half_days                              */
/*      Description: Vire les demi-journees reservees des plqnnings         */
/*==========================================================================*/

void	SOLV_eliminate_reserved_half_days (void)
{
	int		period;
	int		type;
	int		nbr_rooms;
	int		room_cnt;
	int		room;
	int		day;
	int		half;
	int		week;
	int		start_week;
	int		end_week;
	int		ok_flag;
	int		section;
	int		capacity;
	int		subject;
	int		subject_cnt;
	int		duration;
	int		half_day_cnt;
	signed int	selected_room;
	int		selected_room_capacity;
	PARM_HALF_DAY	*half_day_ptr;
	PARM_PLANNING	*class_ptr;

	/* Reserve les salles pour les periodes qui ont besoin de salles
		pendant les demi-journees reservees. */
	for (half_day_cnt = 0; half_day_cnt < MAIN_nbr_half_days; half_day_cnt ++)
	{
		period = MAIN_half_day_ptr [half_day_cnt].period;
		section = MAIN_period_ptr [period].section;
		start_week = (int) MAIN_period_ptr [period].start_week;
		end_week = start_week + MAIN_period_ptr [period].duration - 1;
		half_day_ptr = MAIN_half_day_ptr + half_day_cnt;

		day = half_day_ptr->day;
		half = half_day_ptr->half;
		duration = (half == 0) ? MAIN_am_length : MAIN_pm_length;
		nbr_rooms = half_day_ptr->nbr_reserved_rooms;

		if (nbr_rooms > 0)
		{
			type = half_day_ptr->reserved_room_type;
			capacity =   (  MAIN_section_ptr [section].nbr_students
							  + nbr_rooms - 1)
						  / nbr_rooms;

			for (room_cnt = 0; room_cnt < nbr_rooms; room_cnt ++)
			{
				if (type == PARM_CLASS_TYPE_LAB)
				{
					subject = half_day_ptr->lab [room_cnt % half_day_ptr->nbr_subjects];
				}

				/* Cherche une salle qui reponde le
					mieux possible aux criteres */
				selected_room = -1;
				selected_room_capacity = INT_MAX;
				for (room = 0; room < MAIN_nbr_rooms; room ++)
				{
					/* Si la salle n'est pas du type recherche, ou que sa
						capacite est trop faible pour le groupe, on la passe. */
					if (   type != MAIN_room_ptr [room].type
						 || MAIN_room_ptr [room].capacity < capacity
						 || MAIN_room_ptr [room].capacity > selected_room_capacity)
					{
						continue;
					}

					/* Regarde si le labo possede la bonne matiere */
					if (type == PARM_CLASS_TYPE_LAB)
					{
						ok_flag = FALSE;
						for (subject_cnt = 0; subject_cnt < MAIN_room_ptr [room].nbr_subjects; subject_cnt ++)
						{
							if (MAIN_room_ptr [room].lab [subject_cnt] == subject)
							{
								ok_flag = TRUE;
								break;
							}
						}

						if (! ok_flag)
						{
							continue;
						}
					}

					ok_flag = TRUE;

					/* Cherche si la salle est disponible toute la periode */
					for (week = start_week; week <= end_week; week ++)
					{
						class_ptr = MAIN_room_ptr [room].planning [week] [day] [half];
						if (class_ptr != NULL)
						{
							/* Salle occupee mais ce n'est pas un jour ferie */
							if (class_ptr->start >= 0)
							{
								ok_flag = FALSE;
								break;
							}
						}
					}

					/* Selectionne la salle */
					if (ok_flag)
					{
						selected_room = room;
						selected_room_capacity = MAIN_room_ptr [room].capacity;
					}
				}

				/* Aucune salle libre de trouvee */
				if (selected_room < 0)
				{
					fprintf (stderr, "Warning: can't find any available room "
										  "for period \"%s\", reserved half-day "
										  "\"%s\", room %d of %d.\n",
										  MAIN_period_ptr [period].name_0,
										  half_day_ptr->name_0,
										  room_cnt + 1, nbr_rooms);
				}

				else
				{
					for (week = start_week; week <= end_week; week ++)
					{
						/* Si on tombe sur un jour ferie, inutile de
							reserver la salle. */
						class_ptr = MAIN_section_ptr [section].planning [week] [day] [half];
						if (class_ptr != NULL)
						{
							if (class_ptr->start < 0)
							{
								continue;
							}
						}

						/* Reserve la salle */
						class_ptr = SOLV_reserve_room (& MAIN_room_ptr [selected_room].planning [week] [day] [half],
																 selected_room,
																 0, duration);
						class_ptr->section = section;
						class_ptr->subdiv = half_day_cnt;
						class_ptr->subdiv_tot = 1;
						class_ptr->subject = -1;
						class_ptr->type = -1;
						class_ptr->teacher = -1;

						/* Reserve le groupe */
						class_ptr = SOLV_reserve_room (& MAIN_section_ptr [section].planning [week] [day] [half],
																 selected_room,
																 0, duration);
						class_ptr->section = section;
						class_ptr->subdiv = half_day_cnt;
						class_ptr->subdiv_tot = 1;
						class_ptr->subject = -1;
						class_ptr->type = -1;
						class_ptr->teacher = -1;
					}
				}
			}
		}

		else
		{
			for (week = start_week; week <= end_week; week ++)
			{
				if (MAIN_section_ptr [section].planning [week] [day] [half] == NULL)
				{
					class_ptr = SOLV_reserve_room (& MAIN_section_ptr [section].planning [week] [day] [half],
															 -1,
															 0, duration);
					class_ptr->section = section;
					class_ptr->subdiv = half_day_cnt;
					class_ptr->subdiv_tot = 1;
					class_ptr->subject = -1;
					class_ptr->type = -1;
					class_ptr->teacher = -1;
				}
			}
		}
	}
}



/*==========================================================================*/
/*      Nom: SOLV_sort_classes                                              */
/*      Description: Trie les cours selon leur ordre de priorite, subdivise */
/*                   les groupes, vire les TD qui sont associes a des cours */
/*                   et remelange un peu le tout suivant la cle de          */
/*                   generation de l'emploi du temps.                       */
/*==========================================================================*/

void	SOLV_sort_classes (void)
{
	int		class_nbr;
	int		class_cnt;
	int		class_cnt_2;
	int		length;
	int		nbr_subdiv;
	int		subject;
	int		section;
	int      indice_1;
	int      indice_2;
	int		add_flag;
	int		end_flag;
	int		period;
	signed int	tutorial;
	SOLV_CLASS_ORDER	temp;
	void		*temp_ptr;

	/* Reserve un tableau pour contenir tous les index des cours */
	SOLV_class_order_ptr = (SOLV_CLASS_ORDER *) malloc (MAIN_nbr_classes * sizeof (*SOLV_class_order_ptr));
	if (SOLV_class_order_ptr == NULL)
	{
		fprintf (stderr, "Error: malloc failed.\n");
		exit (-1);
	}

	/* Remplit le tableau avec les index */
	length = 0;
	SOLV_nbr_class_orders = 0;
	for (class_cnt = 0; class_cnt < MAIN_nbr_classes; class_cnt ++)
	{
		section = MAIN_class_ptr [class_cnt].section;
		subject = MAIN_class_ptr [class_cnt].subject;
		period = MAIN_class_ptr [class_cnt].period;
		tutorial = -1;
		add_flag = TRUE;

		/* Si ce n'est pas un TD, on l'ajoute a la liste */
		if (MAIN_class_ptr [class_cnt].type != PARM_CLASS_TYPE_TUT)
		{
			if (MAIN_class_ptr [class_cnt].type == PARM_CLASS_TYPE_LECT)
			{
				/* Recherche le TD (doit etre unique) qui accompagne
					le cours (s'il existe) */
				for (class_cnt_2 = 0; class_cnt_2 < MAIN_nbr_classes; class_cnt_2 ++)
				{
					if (   MAIN_class_ptr [class_cnt_2].section == section
						 && MAIN_class_ptr [class_cnt_2].subject == subject
						 && MAIN_class_ptr [class_cnt_2].period == period
						 && MAIN_class_ptr [class_cnt_2].type == PARM_CLASS_TYPE_TUT)
					{
						tutorial = class_cnt_2;

						/* Un cours avec TD a la suite est toujours en
							un seul groupe. */
						if (MAIN_class_ptr [class_cnt].nbr_subdiv != 1)
						{
							MAIN_class_ptr [class_cnt].nbr_subdiv = 1;
							fprintf (stderr,
										"Warning: a lecture followed by a tutorial "
										"can't be divided in groups (%s, %s, %s).\n",
										MAIN_subject_ptr [subject].name_0,
										MAIN_section_ptr [section].name_0,
										MAIN_period_ptr [period].name_0);
						}

						/* Le nombre de seances de TD doit etre inferieur ou
							egale au nombre de seances de cours */
						if (  MAIN_class_ptr [class_cnt_2].nbr_classes
							 > MAIN_class_ptr [class_cnt].nbr_classes)
						{
							fprintf (stderr,
										"Error: The number of tutorial must be less or "
										"equal than number of lectures. (%s, %s, %s).\n",
										MAIN_subject_ptr [subject].name_0,
										MAIN_section_ptr [section].name_0,
										MAIN_period_ptr [period].name_0);
							exit (-1);
						}

						/* Le nombre de cours doit etre un multiple du nombre
							de TD */
						if (  MAIN_class_ptr [class_cnt].nbr_classes
							 % MAIN_class_ptr [class_cnt_2].nbr_classes != 0)
						{
							fprintf (stderr,
										"Error: The number of lectures must be a multiple "
										"of the number of tutorials. (%s, %s, %s).\n",
										MAIN_subject_ptr [subject].name_0,
										MAIN_section_ptr [section].name_0,
										MAIN_period_ptr [period].name_0);
							exit (-1);
						}

						/* Le nombre de groupes de TD * nombre de TD doit etre
							un multiple du nombre de cours */
						if (  (  MAIN_class_ptr [class_cnt_2].nbr_subdiv
								 * MAIN_class_ptr [class_cnt_2].nbr_classes)
							 % MAIN_class_ptr [class_cnt].nbr_classes != 0)
						{
							fprintf (stderr,
										"Error: The number of tutorial groups multiplied "
										"by the number of tutorials must be a multiple "
										"of the number of lectures. (%s, %s, %s).\n",
										MAIN_subject_ptr [subject].name_0,
										MAIN_section_ptr [section].name_0,
										MAIN_period_ptr [period].name_0);
							exit (-1);
						}

						break;
					}
				}
			}
		}

		/* Si c'est un TD */
		else
		{
			/* On regarde s'il y a un cours associe */
			for (class_cnt_2 = 0; class_cnt_2 < MAIN_nbr_classes; class_cnt_2 ++)
			{
				/* Si oui, on ne l'ajoute pas a la liste */
				if (   MAIN_class_ptr [class_cnt_2].section == section
					 && MAIN_class_ptr [class_cnt_2].subject == subject
					 && MAIN_class_ptr [class_cnt_2].period == period
					 && MAIN_class_ptr [class_cnt_2].type == PARM_CLASS_TYPE_LECT)
				{
					add_flag = FALSE;
					break;
				}
			}
		}

		/* Ajout du cours a la liste */
		if (add_flag)
		{
			SOLV_class_order_ptr [SOLV_nbr_class_orders].class_nbr = class_cnt;
			SOLV_class_order_ptr [SOLV_nbr_class_orders].priority = MAIN_class_ptr [class_cnt].priority;
			SOLV_class_order_ptr [SOLV_nbr_class_orders].subdiv = 0;
			SOLV_class_order_ptr [SOLV_nbr_class_orders].tutorial = tutorial;

			SOLV_nbr_class_orders ++;
			length += MAIN_class_ptr [class_cnt].nbr_subdiv;
		}
	}

	/* Remet le tableau a sa vraie taille */
	temp_ptr = realloc (SOLV_class_order_ptr,
							  length  * sizeof (*SOLV_class_order_ptr));
	if (temp_ptr == NULL)
	{
		fprintf (stderr, "Error: realloc failed.\n");
		exit (-1);
	}
	SOLV_class_order_ptr = (SOLV_CLASS_ORDER *) temp_ptr;

	/* Classe le tableau (bete tri a bulles) */
	for (indice_1 = SOLV_nbr_class_orders - 1; indice_1 > 0; indice_1 --)
	{
		end_flag = TRUE;

		for (indice_2 = 0; indice_2 < indice_1; indice_2 ++)
		{
			if (  SOLV_class_order_ptr [indice_2].priority
				 < SOLV_class_order_ptr [indice_2 + 1].priority)
			{
				temp = SOLV_class_order_ptr [indice_2];
				SOLV_class_order_ptr [indice_2] = SOLV_class_order_ptr [indice_2 + 1];
				SOLV_class_order_ptr [indice_2 + 1] = temp;
				end_flag = FALSE;
			}
		}

		if (end_flag)
		{
			break;
		}
	}

	/* De-classe le tableau pour varier les plannings */
	SOLV_random_order ((void *)SOLV_class_order_ptr,
							 sizeof (*SOLV_class_order_ptr),
							 (long)SOLV_nbr_class_orders,
							 SOLV_planning_key);

	/* Duplique les cours qui sont subdivises */
	for (indice_1 = SOLV_nbr_class_orders - 1; indice_1 >= 0; indice_1 --)
	{
		class_nbr = SOLV_class_order_ptr [indice_1].class_nbr;
		nbr_subdiv = MAIN_class_ptr [class_nbr].nbr_subdiv;
		if (nbr_subdiv > 1)
		{
			memmove (SOLV_class_order_ptr + indice_1 + nbr_subdiv,
						SOLV_class_order_ptr + indice_1 + 1,
						  (SOLV_nbr_class_orders - indice_1 - 1)
						*  sizeof (*SOLV_class_order_ptr));
			for (indice_2 = 1; indice_2 < nbr_subdiv; indice_2 ++)
			{
				SOLV_class_order_ptr [indice_2 + indice_1] = SOLV_class_order_ptr [indice_1];
				SOLV_class_order_ptr [indice_2 + indice_1].subdiv = indice_2;
			}
			SOLV_nbr_class_orders += nbr_subdiv - 1;
		}
	}

	#ifdef DEBUG
	/* Affichage de verification */
	{
		int		subdiv;
		int		indice_1;
		int		class_nbr;
		int		section;
		int		subject;
		int		period;
		int		nbr_subdiv;

		printf ("Tableau d'ordre de placement:\n");
		for (indice_1 = 0; indice_1 < SOLV_nbr_class_orders; indice_1 ++)
		{
			class_nbr = SOLV_class_order_ptr [indice_1].class_nbr;
			subdiv = SOLV_class_order_ptr [indice_1].subdiv;
			section = MAIN_class_ptr [class_nbr].section;
			subject = MAIN_class_ptr [class_nbr].subject;
			period = MAIN_class_ptr [class_nbr].period;
			nbr_subdiv = MAIN_class_ptr [class_nbr].nbr_subdiv;
			printf ("Cours de %s, section %s, periode %s",
					  MAIN_subject_ptr [subject].name_0,
					  MAIN_section_ptr [section].name_0,
					  MAIN_period_ptr [period].name_0);
			if (SOLV_class_order_ptr [indice_1].tutorial < 0)
			{
				if (nbr_subdiv > 1)
				{
					printf (", subdivision %d/%d.\n", subdiv + 1, nbr_subdiv);
				}
				else
				{
					printf (".\n");
				}
			}
			else
			{
				printf (", avec TD.\n");
			}
		}
	}
	#endif
}



/****************************************************************************/
/*                                                                          */
/*      RESOLUTION DE L'EMPLOI DU TEMPS                                     */
/*                                                                          */
/****************************************************************************/



signed int	SOLV_organize_classes (void)
{
	int		order_cnt;
	int		class_nbr;
	int		nbr_groups;
	int		period;
	int		period_len;
	int		week;
	int		nbr_sessions;
	int		session_cnt;
	signed int	tutorial;
	signed int	trans;
	SOLV_POINT_WEEK	*tab_point_ptr_ptr;
	SOLV_POINT_WEEK	*tab_point_2_ptr_ptr;
	SOLV_SESSION		*session_tab_ptr;

	/* Pour chaque cours du tableau */
	for (order_cnt = 0; order_cnt < SOLV_nbr_class_orders; order_cnt ++)
	{
		class_nbr = SOLV_class_order_ptr [order_cnt].class_nbr;
		period = MAIN_class_ptr [class_nbr].period;
		period_len = MAIN_period_ptr [period].duration;
		nbr_sessions = MAIN_class_ptr [class_nbr].nbr_classes;

		/* Fabrique la liste des plages horaires susceptibles
			d'accueillir le cours */
		SOLV_make_list_for_class (&tab_point_ptr_ptr, class_nbr, 1,
										  SOLV_class_order_ptr [order_cnt].subdiv);

		/* On s'occupe des TD associes */
		tutorial = SOLV_class_order_ptr [order_cnt].tutorial;
		if (tutorial >= 0)
		{
			nbr_groups =   (  MAIN_class_ptr [tutorial].nbr_classes
								 * MAIN_class_ptr [tutorial].nbr_subdiv)
							 / nbr_sessions;
			SOLV_make_list_for_class (&tab_point_2_ptr_ptr,
											  tutorial, nbr_groups, -1);

			/* Calcul de la translation: duree du cours arrondie par
				exces sur la granularite du temps. */
			trans =   (MAIN_class_ptr [class_nbr].duration + MAIN_time_granularity - 1)
					  / MAIN_time_granularity;
			trans *= MAIN_time_granularity;

			for (week = 0; week < period_len; week ++)
			{
				/* Translation de la liste de TD de la duree du cours, de
					facon a ce que les horaires successifs entre les deux
					parties se superposent */
				SOLV_translate_availability_list (tab_point_2_ptr_ptr [week], -trans);

				/* Intersection des deux listes */
				SOLV_intersect_availability_list (tab_point_ptr_ptr [week],
															 tab_point_2_ptr_ptr [week]);

				/* Liberation de la liste des TD */
				SOLV_delete_availability_list (tab_point_2_ptr_ptr [week]);
			}
		}

		#ifdef DEBUG
		/* Affichage de controle */
		{
			int		week;
			int		day;
			int		half;
			int		section;
			int		subject;
			SOLV_POINT	*point_ptr;

			section = MAIN_class_ptr [class_nbr].section;
			subject = MAIN_class_ptr [class_nbr].subject;
			printf ("Cours de %s, type %d, section %s, periode %s:\n",
					  MAIN_subject_ptr [subject].name_0,
					  MAIN_class_ptr [class_nbr].type,
					  MAIN_section_ptr [section].name_0,
					  MAIN_period_ptr [period].name_0);
			for (week = 0; week < period_len; week ++)
			{
				printf ("Semaine %d\n", week);
				for (day = MAIN_first_day; day <= MAIN_last_day; day ++)
				{
					printf ("Jour %d\n", day);
					for (half = 0; half < 2; half ++)
					{
						printf ("%cm: ", (half == 0) ? 'a' : 'p');
						point_ptr = tab_point_ptr_ptr [week] [day] [half];
						while (point_ptr != NULL)
						{
							printf ("%d ", (int) point_ptr->point);
							point_ptr = point_ptr->next_ptr;
						}
						printf ("\n");
					}
				}
			}
		}
		#endif

		/* Choix de l'horaire */
		session_tab_ptr = (SOLV_SESSION *) malloc (nbr_sessions * sizeof (*session_tab_ptr));
		if (session_tab_ptr == NULL)
		{
			fprintf (stderr, "Error: malloc failed.\n");
			exit (-1);
		}

		if (SOLV_choose_schedule (session_tab_ptr, tab_point_ptr_ptr,
										  period_len, nbr_sessions, class_nbr))
		{
			/* Si on n'a pas pu trouver d'horaire */
			for (week = 0; week < period_len; week ++)
			{
				SOLV_delete_availability_list (tab_point_ptr_ptr [week]);
			}
			free (session_tab_ptr);
			return (-1);
		}

		/* Choix de la salle et du prof, reservations */
		SOLV_reserve_rooms_and_teachers (session_tab_ptr, class_nbr, 1, nbr_sessions,
													SOLV_class_order_ptr [order_cnt].subdiv);

		if (tutorial >= 0)
		{
			/* Re-transaltion des horaires vers celui des TD */
			for (session_cnt = 0; session_cnt < nbr_sessions; session_cnt ++)
			{
				session_tab_ptr [session_cnt].date += trans;
			}

			/* Choix des salles et des profs, reservations */
			SOLV_reserve_rooms_and_teachers (session_tab_ptr, tutorial,
														nbr_groups, nbr_sessions, -1);
		}

		/* Libere la memoire reservee pour les listes */
		for (week = 0; week < period_len; week ++)
		{
			SOLV_delete_availability_list (tab_point_ptr_ptr [week]);
		}

		/* Libere la memoire reservee pour les horaires des salles */
		free (session_tab_ptr);
	}

	return (0);
}



signed int	SOLV_make_list_for_class (SOLV_POINT_WEEK **tab_point_ptr_ptr_ptr, int class_nbr, int nbr_groups, signed int subdiv)
{
	int		point;
	int		half;
	int		day;
	int		period;
	int		period_len;
	int		week;
	int		duration;
	int		room;
	int		teacher;
	int		teacher_cnt;
	int		room_cnt;
	int		nbr_teachers;
	int		nbr_rooms;
	long		start_week;
	SOLV_POINT	*point_ptr;
	SOLV_POINT	**point_ptr_ptr;
	PARM_PLANNING	*planning_ptr;

	duration = MAIN_class_ptr [class_nbr].duration;
	period = MAIN_class_ptr [class_nbr].period;
	period_len = MAIN_period_ptr [period].duration;
	start_week = MAIN_period_ptr [period].start_week;

	/* Teste tous les emplacements libres pour au moins une salle, un prof */
	SOLV_make_list_one_period (tab_point_ptr_ptr_ptr, class_nbr, subdiv);

	/* Pour chaque horaire trouve, teste si on a simultanement suffisamment
		de salles et de professeurs. */
	for (week = 0; week < period_len; week ++)
	{
		for (day = MAIN_first_day; day <= MAIN_last_day; day ++)
		{
			for (half = 0; half < 2; half ++)
			{
				point_ptr_ptr = & ((*tab_point_ptr_ptr_ptr) [week] [day] [half]);
				point_ptr = *point_ptr_ptr;
				while (point_ptr != NULL)
				{
					nbr_teachers = 0;
					nbr_rooms = 0;
					point = point_ptr->point;

					/* Regarde s'il y a assez de salles */
					for (room_cnt = 0; room_cnt < MAIN_class_ptr [class_nbr].nbr_rooms; room_cnt ++)
					{
						room = MAIN_class_ptr [class_nbr].room_ptr [room_cnt];
						planning_ptr = MAIN_room_ptr [room].planning [week + start_week] [day] [half];
						nbr_rooms += (SOLV_test_interval (planning_ptr, point, duration) != 0);
					}

					/* Regarde s'il y a assez de profs */
					for (teacher_cnt = 0; teacher_cnt < MAIN_class_ptr [class_nbr].nbr_teachers; teacher_cnt ++)
					{
						teacher = MAIN_class_ptr [class_nbr].teacher_ptr [teacher_cnt];
						planning_ptr = MAIN_teacher_ptr [teacher].planning [week + start_week] [day] [half];
						nbr_teachers += (SOLV_test_interval (planning_ptr, point, duration) != 0);
					}

					/* Si on n'a pas respecte les quotas, on vire le point */
					if (   nbr_teachers < nbr_groups
						 || nbr_rooms < nbr_groups)
					{
						*point_ptr_ptr = point_ptr->next_ptr;
						free (point_ptr);
						point_ptr = *point_ptr_ptr;
					}

					else
					{
						/* Point suivant */
						point_ptr_ptr = &point_ptr->next_ptr;
						point_ptr = *point_ptr_ptr;
					}
				}
			}
		}
	}

	return (0);
}



signed int	SOLV_make_list_one_period (SOLV_POINT_WEEK **tab_point_ptr_ptr_ptr, int class_nbr, signed int subdiv)
{
	int		period;
	int		period_len;
	int		week;
	long		start_week;

	period = MAIN_class_ptr [class_nbr].period;
	period_len = MAIN_period_ptr [period].duration;
	start_week = MAIN_period_ptr [period].start_week;

	*tab_point_ptr_ptr_ptr = (SOLV_POINT_WEEK *)
									 malloc (period_len * 7 * 2 * sizeof (SOLV_POINT *));
	if (tab_point_ptr_ptr_ptr == NULL)
	{
		fprintf (stderr, "Error: malloc failed.\n");
		exit (-1);
	}

	for (week = 0; week < period_len; week ++)
	{
		SOLV_make_list_one_week ((*tab_point_ptr_ptr_ptr) [week],
										 class_nbr, start_week + week, subdiv);
	}

	return (0);
}



signed int	SOLV_make_list_one_week (SOLV_POINT_WEEK tab_point_ptr, int class_nbr, long week, signed int subdiv)
{
	int		section;
	int		room;
	int		teacher;
	int		room_cnt;
	int		teacher_cnt;
	int		duration;
	SOLV_POINT	*tab_point_2_ptr [7] [2];
	SOLV_POINT	*tab_point_3_ptr [7] [2];

	duration = MAIN_class_ptr [class_nbr].duration;

	/* Examine les horaires dispo pour la section */
	section = MAIN_class_ptr [class_nbr].section;
	SOLV_create_availability_list (tab_point_ptr,
											 MAIN_section_ptr [section].planning [week],
											 duration, class_nbr, subdiv);

	/* Recupere tous les horaires dispo pour chaque salle susceptible
		d'accueillir le cours */
	memset (tab_point_2_ptr, 0, 7 * 2 * sizeof (SOLV_POINT *));
	for (room_cnt = 0; room_cnt < MAIN_class_ptr [class_nbr].nbr_rooms; room_cnt ++)
	{
		room = MAIN_class_ptr [class_nbr].room_ptr [room_cnt];
		SOLV_create_availability_list (tab_point_3_ptr,
												 MAIN_room_ptr [room].planning [week],
												 duration, -1, -1);
		SOLV_union_availability_list (tab_point_2_ptr, tab_point_3_ptr);
		SOLV_delete_availability_list (tab_point_3_ptr);
	}

	/* Intersection avec la liste principale */
	SOLV_intersect_availability_list (tab_point_ptr, tab_point_2_ptr);
	SOLV_delete_availability_list (tab_point_2_ptr);

	/* Recupere tous les horaires dispo pour chaque prof susceptible
		d'assurer le cours */
	memset (tab_point_2_ptr, 0, 7 * 2 * sizeof (SOLV_POINT *));
	for (teacher_cnt = 0; teacher_cnt < MAIN_class_ptr [class_nbr].nbr_teachers; teacher_cnt ++)
	{
		teacher = MAIN_class_ptr [class_nbr].teacher_ptr [teacher_cnt];
		SOLV_create_availability_list (tab_point_3_ptr,
												 MAIN_teacher_ptr [teacher].planning [week],
												 duration, -1, -1);
		SOLV_union_availability_list (tab_point_2_ptr, tab_point_3_ptr);
		SOLV_delete_availability_list (tab_point_3_ptr);
	}

	/* Intersection avec la liste principale */
	SOLV_intersect_availability_list (tab_point_ptr, tab_point_2_ptr);
	SOLV_delete_availability_list (tab_point_2_ptr);

	return (0);
}



signed int	SOLV_choose_schedule (SOLV_SESSION *session_tab_ptr, SOLV_POINT_WEEK *tab_point_ptr_ptr, int period_len, int nbr_sessions, int class_nbr)
{
	int		week;
	int		day;
	int		half;
	int		index;
	int		mid_week;
	int		*nbr_space_ptr [7] [2];
	int		nbr_points [2];
	int		week_dist;
	int		day_dist_max;
	signed int	point;
	signed int	ret_val;
	int		nbr_sessions_2;
	int		nbr_sessions_max;
	int		nbr_sessions_3;
	int		selected_point;
	int		selected_half;
	int		selected_day;
	int		session_count;
	int		found_flag;
	int		indice_1;
	int		indice_2;
	int		end_flag;
	long		val_1;
	long		val_2;
	long		possibility;
	long		nbr_possibilities;
	int		points_per_day;
	int		duration;
	SOLV_SESSION	temp;
	SOLV_POINT	*point_ptr;

	ret_val = 0;
	nbr_sessions_3 = nbr_sessions;

	nbr_points [0] =   (MAIN_am_length + MAIN_time_granularity - 1)
						  / MAIN_time_granularity;
	nbr_points [1] =   (MAIN_pm_length + MAIN_time_granularity - 1)
						  / MAIN_time_granularity;

	/* Cree un tableau contenant pour chaque jour/demi-journee/unite le nombre
		de seances disponibles dans la periode */
	nbr_sessions_max = 0;
	for (day = MAIN_first_day; day <= MAIN_last_day; day ++)
	{
		for (half = 0; half < 2; half ++)
		{
			nbr_space_ptr [day] [half] = (int *) calloc (nbr_points [half], sizeof (int));
			if (nbr_space_ptr [day] [half] == NULL)
			{
				fprintf (stderr, "Error: malloc failed.\n");
				exit (-1);
			}

			for (week = 0; week < period_len; week ++)
			{
				point_ptr = tab_point_ptr_ptr [week] [day] [half];
				while (point_ptr != NULL)
				{
					index = point_ptr->point / MAIN_time_granularity;
					nbr_space_ptr [day] [half] [index] ++;
					nbr_sessions_max ++;
					point_ptr = point_ptr->next_ptr;
				}
			}
		}
	}

	/* Pas assez d'emplacements disponibles au total */
	if (nbr_sessions_max < nbr_sessions)
	{
		ret_val = -1;
		goto proc_end;
	}

	duration =   (MAIN_class_ptr [class_nbr].duration + MAIN_time_granularity - 1)
				  / MAIN_time_granularity;	/* En nombre de points */
	mid_week = (MAIN_first_day + MAIN_last_day + 1) / 2;	/* Moyenne arrondie par exces */
	week_dist = MAIN_last_day - MAIN_first_day + 1;
	day_dist_max = MAX (nbr_points [0], nbr_points [1]) - duration + 1;
	day_dist_max = MAX (day_dist_max, 0);
	points_per_day = day_dist_max * 2;
	nbr_possibilities =   (long)(MAIN_last_day - MAIN_first_day + 1)
							  * points_per_day;

	while (nbr_sessions > 0)
	{
		/* Choisit la meilleure possibilite */
		nbr_sessions_max = 0;
		for (possibility = 0; possibility < nbr_possibilities; possibility ++)
		{
			/* Suivant le critere choisi, on aura differentes formules
				donnant jour, demi-journee et moment. */

			switch (SOLV_schedule_choice_method)
			{
			/* Tassement sur le debut de la semaine et
				le debut du jour (semaine compacte) */
			case	0:
			default:
				day = (int) (MAIN_first_day + possibility / points_per_day);
				point = (int) (possibility % points_per_day);
				half = point / day_dist_max;
				point %= day_dist_max;

				if (half != 0 && half != 1)
				{
					continue;
				}
				if (   point < 0
					 || point >= nbr_points [half]
					 || day < MAIN_first_day
					 || day > MAIN_last_day)
				{
					continue;
				}
				break;

			/* Centrage dans le jour puis centrage
				dans la semaine (semaine compacte) */
			case	1:
				day = (int) (mid_week +   ((possibility / points_per_day + 1) >> 1)
												* (1 - (((possibility / points_per_day) & 1) * 2)));
				point = (int) (possibility % points_per_day);
				half = point & 1;
				point >>= 1;
				if (half == 0)
				{
					point = nbr_points [0] - duration - point;
				}
				break;

			/* Centrage dans la semaine puis centrage
				dans le jour (journees equilibrees) */
			case	2:
				day = (int) (mid_week +   (((possibility % week_dist) + 1) >> 1)
												* (1 - (((possibility % week_dist) & 1) * 2)));
				point = (int) (possibility / week_dist);
				half = point & 1;
				point >>= 1;
				if (half == 0)
				{
					point = nbr_points [0] - duration - point;
				}
				break;
			}

			nbr_sessions_2 = nbr_space_ptr [day] [half] [point];

			if (nbr_sessions_2 <= 0)
			{
				continue;
			}

			if (nbr_sessions_2 > nbr_sessions_max)
			{
				nbr_sessions_max = nbr_sessions_2;
				selected_point = point;
				selected_day = day;
				selected_half = half;
				if (nbr_sessions_2 >= nbr_sessions)
				{
					nbr_sessions_max = nbr_sessions;
					goto fin_recherche;
				}
			}
		}

		/* Finalement on n'a pas trouve de point qui convienne */
		if (nbr_sessions_max < nbr_sessions)
		{
			ret_val = -1;
			goto proc_end;
		}

	fin_recherche:
		/*** Tout faux: on ne peut pas simplement soustraire le nombre
			  de cours juste pour cet instant. Il faut en fait recalculer
			  tous les espaces disponibles en fonction des horaires
			  precedemment choisis. ---v ***/
		nbr_space_ptr [selected_day] [selected_half] [selected_point] -= nbr_sessions_max;		
		selected_point *= MAIN_time_granularity;

		/* Reservation des emplacements choisi */
		week = 0;
		for (session_count = 0; session_count < nbr_sessions_max; session_count ++)
		{
			/* Cherche la prochaine semaine ou le creneau est disponible */
			found_flag = FALSE;
			for ( ; ; )
			{
				point_ptr = tab_point_ptr_ptr [week] [selected_day] [selected_half];
				while (point_ptr != NULL)
				{
					if (point_ptr->point == selected_point)
					{
						found_flag = TRUE;
						break;
					}
					point_ptr = point_ptr->next_ptr;
				}
				if (found_flag)
				{
					break;
				}
				week ++;
			}

			/* Ajoute cette seance a la liste */
			session_tab_ptr [session_count].week = week;
			session_tab_ptr [session_count].day = selected_day;
			session_tab_ptr [session_count].half = selected_half;
			session_tab_ptr [session_count].date = selected_point;

			nbr_sessions --;
			week ++;
		}
	}

	/* Trie le tableau dans l'ordre chronologique */
	for (indice_1 = nbr_sessions - 1; indice_1 > 0; indice_1 --)
	{
		end_flag = TRUE;

		for (indice_2 = 0; indice_2 < indice_1; indice_2 ++)
		{
			val_1 =   ((long)session_tab_ptr [indice_2].week << 20)
					  + ((long)session_tab_ptr [indice_2].day << 17)
					  + ((long)session_tab_ptr [indice_2].half << 16)
					  + ((long)session_tab_ptr [indice_2].date);
			val_2 =   ((long)session_tab_ptr [indice_2 + 1].week << 20)
					  + ((long)session_tab_ptr [indice_2 + 1].day << 17)
					  + ((long)session_tab_ptr [indice_2 + 1].half << 16)
					  + ((long)session_tab_ptr [indice_2 + 1].date);
			if (  val_1 > val_2)
			{
				temp = session_tab_ptr [indice_2];
				session_tab_ptr [indice_2] = session_tab_ptr [indice_2 + 1];
				session_tab_ptr [indice_2 + 1] = temp;
				end_flag = FALSE;
			}
		}

		if (end_flag)
		{
			break;
		}
	}

proc_end:
	for (day = MAIN_first_day; day <= MAIN_last_day; day ++)
	{
		for (half = 0; half < 2; half ++)
		{
			free (nbr_space_ptr [day] [half]);
		}
	}

	if (ret_val != 0)
	{
		fprintf (stderr,
					"Info: can't find time for class of %s (type %d), "
					"section %s, period %s. Failed for %d sessions.\n",
					MAIN_subject_ptr [MAIN_class_ptr [class_nbr].subject].name_0,
					MAIN_class_ptr [class_nbr].type,
					MAIN_section_ptr [MAIN_class_ptr [class_nbr].section].name_0,
					MAIN_period_ptr [MAIN_class_ptr [class_nbr].period].name_0,
					nbr_sessions);
	}

	return (ret_val);
}



signed int	SOLV_reserve_rooms_and_teachers (SOLV_SESSION session_tab_ptr [], int class_nbr, int nbr_groups, int nbr_sessions, signed int subdiv)
{
	int		nbr_teachers;
	int		nbr_rooms;
	int		tot_nbr_teachers;
	int		tot_nbr_rooms;
	int		teacher_cnt;
	int		room_cnt;
	int		teacher;
	int		room;
	int		group_cnt;
	int		session_cnt;
	int		period;
	int		start_week;
	int		week;
	int		day;
	int		half;
	int		start;
	int		duration;
	int		indice_1;
	int		indice_2;
	long		val_1;
	long		val_2;
	int		temp;
	int		end_flag;
	int		group_subdiv;
	int		section;
	int		*teacher_list_ptr;
	int		*room_list_ptr;
	PARM_PLANNING	*planning_ptr;

	tot_nbr_teachers = MAIN_class_ptr [class_nbr].nbr_teachers;
	teacher_list_ptr = (int *) malloc (tot_nbr_teachers * sizeof (*teacher_list_ptr));
	if (teacher_list_ptr == NULL)
	{
		fprintf (stderr, "Error: malloc failed.\n");
		exit (-1);
	}

	tot_nbr_rooms = MAIN_class_ptr [class_nbr].nbr_rooms;
	room_list_ptr = (int *) malloc (tot_nbr_rooms * sizeof (*room_list_ptr));
	if (room_list_ptr == NULL)
	{
		fprintf (stderr, "Error: malloc failed.\n");
		exit (-1);
	}

	duration = MAIN_class_ptr [class_nbr].duration;
	section = MAIN_class_ptr [class_nbr].section;
	period = MAIN_class_ptr [class_nbr].period;
	start_week = (int) MAIN_period_ptr [period].start_week;

	for (session_cnt = 0; session_cnt < nbr_sessions; session_cnt ++)
	{
		week = session_tab_ptr [session_cnt].week;
		day = session_tab_ptr [session_cnt].day;
		half = session_tab_ptr [session_cnt].half;
		start = session_tab_ptr [session_cnt].date;

		/* Liste toutes les salles disponibles a cette heure, et qui
			conviennent pour le cours */
		nbr_rooms = 0;
		for (room_cnt = 0; room_cnt < tot_nbr_rooms; room_cnt ++)
		{
			room = MAIN_class_ptr [class_nbr].room_ptr [room_cnt];
			planning_ptr = MAIN_room_ptr [room].planning [start_week + week] [day] [half];
			if (SOLV_test_interval (planning_ptr, start, duration))
			{
				room_list_ptr [nbr_rooms] = room;
				nbr_rooms ++;
			}
		}

		/* Trie les salles trouvees par ordre de taille croissante */
		for (indice_1 = nbr_rooms - 1; indice_1 > 0; indice_1 --)
		{
			end_flag = TRUE;

			for (indice_2 = 0; indice_2 < indice_1; indice_2 ++)
			{
				val_1 = MAIN_room_ptr [room_list_ptr [indice_2]].capacity;
				val_2 = MAIN_room_ptr [room_list_ptr [indice_2 + 1]].capacity;
				if (  val_1 > val_2)
				{
					temp = room_list_ptr [indice_2];
					room_list_ptr [indice_2] = room_list_ptr [indice_2 + 1];
					room_list_ptr [indice_2 + 1] = temp;
					end_flag = FALSE;
				}
			}

			if (end_flag)
			{
				break;
			}
		}

		/* Liste tous les profs disponibles a cette heure, et qui
			conviennent pour le cours */
		nbr_teachers = 0;
		for (teacher_cnt = 0; teacher_cnt < tot_nbr_teachers; teacher_cnt ++)
		{
			teacher = MAIN_class_ptr [class_nbr].teacher_ptr [teacher_cnt];
			planning_ptr = MAIN_teacher_ptr [teacher].planning [start_week + week] [day] [half];
			if (SOLV_test_interval (planning_ptr, start, duration))
			{
				teacher_list_ptr [nbr_teachers] = teacher;
				nbr_teachers ++;
			}
		}

		/* Trie les profs trouvees par ordre de priorite decroissante */
		for (indice_1 = nbr_teachers - 1; indice_1 > 0; indice_1 --)
		{
			end_flag = TRUE;

			for (indice_2 = 0; indice_2 < indice_1; indice_2 ++)
			{
				val_1 = MAIN_teacher_ptr [teacher_list_ptr [indice_2]].activity;
				val_2 = MAIN_teacher_ptr [teacher_list_ptr [indice_2 + 1]].activity;
				if (val_1 < val_2)
				{
					temp = teacher_list_ptr [indice_2];
					teacher_list_ptr [indice_2] = teacher_list_ptr [indice_2 + 1];
					teacher_list_ptr [indice_2 + 1] = temp;
					end_flag = FALSE;
				}
			}

			if (end_flag)
			{
				break;
			}
		}

		/* Reserve salles, profs et la section */
		for (group_cnt = 0; group_cnt < nbr_groups; group_cnt ++)
		{
			if (subdiv >= 0)
			{
				group_subdiv = subdiv;
			}
			else
			{
				group_subdiv =   (group_cnt + session_cnt * nbr_groups)
									% MAIN_class_ptr [class_nbr].nbr_subdiv;
			}
			room = room_list_ptr [group_cnt];
			teacher = teacher_list_ptr [group_cnt];

			/* Reserve la salle */
			planning_ptr = SOLV_reserve_room (& (MAIN_room_ptr [room].planning [start_week + week] [day] [half]),
														 room, start, duration);
			planning_ptr->section = section;
			planning_ptr->subdiv = group_subdiv;
			planning_ptr->subdiv_tot = MAIN_class_ptr [class_nbr].nbr_subdiv;
			planning_ptr->subject = MAIN_class_ptr [class_nbr].subject;
			planning_ptr->type = MAIN_class_ptr [class_nbr].type;
			planning_ptr->teacher = teacher;

			/* Reserve le prof */
			planning_ptr = SOLV_reserve_room (& (MAIN_teacher_ptr [teacher].planning [start_week + week] [day] [half]),
														 room, start, duration);
			planning_ptr->section = section;
			planning_ptr->subdiv = group_subdiv;
			planning_ptr->subdiv_tot = MAIN_class_ptr [class_nbr].nbr_subdiv;
			planning_ptr->subject = MAIN_class_ptr [class_nbr].subject;
			planning_ptr->type = MAIN_class_ptr [class_nbr].type;
			planning_ptr->teacher = teacher;

			/* Reserve la section */
			planning_ptr = SOLV_reserve_room (& (MAIN_section_ptr [section].planning [start_week + week] [day] [half]),
														 room, start, duration);
			planning_ptr->section = section;
			planning_ptr->subdiv = group_subdiv;
			planning_ptr->subdiv_tot = MAIN_class_ptr [class_nbr].nbr_subdiv;
			planning_ptr->subject = MAIN_class_ptr [class_nbr].subject;
			planning_ptr->type = MAIN_class_ptr [class_nbr].type;
			planning_ptr->teacher = teacher;
		}
	}

	/* Libere les tableaux */
	free (room_list_ptr);
	free (teacher_list_ptr);

	return (0);
}



PARM_PLANNING	*SOLV_reserve_room (PARM_PLANNING **planning_ptr_ptr, int room, int start_time, int duration)
{
	PARM_PLANNING	*new_ptr;
	PARM_PLANNING	*current_ptr;

	/* Reserve la memoire */
	new_ptr = (PARM_PLANNING *) malloc (sizeof (*new_ptr));
	if (new_ptr == NULL)
	{
		fprintf (stderr, "Error: malloc failed.\n");
		exit (-1);
	}
	new_ptr->room = room;
	new_ptr->start = start_time;
	new_ptr->duration = duration;

	/* Insere l'element dans la liste */
	current_ptr = *planning_ptr_ptr;
	while (current_ptr != NULL)
	{
		if (current_ptr->start >= start_time)
		{
			break;
		}

		planning_ptr_ptr = &current_ptr->next_ptr;
		current_ptr = current_ptr->next_ptr;
	}

	new_ptr->next_ptr = current_ptr;
	*planning_ptr_ptr = new_ptr;

	return (new_ptr);
}




signed int	SOLV_test_interval (PARM_PLANNING *planning_ptr, int start, int length)
{
	/* Si jour ferie ou 1/2 journee reservee, c'est pas libre */
	if (planning_ptr != NULL)
	{
		if (planning_ptr->type < 0)
		{
			return (FALSE);
		}
	}

	while (planning_ptr != NULL)
	{
		/* L'intervalle courant a completement depasse l'intervalle recherche */
		if (start + length <= planning_ptr->start)
		{
			return (TRUE);
		}

		/* La fin de l'intervalle courant depasse le debut de l'intervalle
			recherche: collision */
		if (start < planning_ptr->start + planning_ptr->duration)
		{
			return (FALSE);
		}

		planning_ptr = planning_ptr->next_ptr;
	}

	return (TRUE);
}



void	SOLV_random_order (void *tableau_ptr, size_t object_size, long nbr_objects, long key)
{
	long		count;
	long		position;
	void		*temp_ptr;
	char		*current_pos_ptr;

	if (nbr_objects < 2)
	{
		return;
	}

	temp_ptr = malloc (object_size);
	if (temp_ptr == NULL)
 	{
		fprintf (stderr, "Error: malloc failed.\n");
		exit (-1);
	}

	for (count = 0; count < key; count ++)
	{
		position = (long) rand ();
		if (nbr_objects > RAND_MAX)
		{
			position *= (long) rand ();
		}
		position %= nbr_objects - 1;

		current_pos_ptr = (char *)tableau_ptr + position * object_size;
		memcpy (temp_ptr, (void *) current_pos_ptr, object_size);
		memcpy ((void *) current_pos_ptr, (void *) (current_pos_ptr + object_size), object_size);
		memcpy ((void *) (current_pos_ptr + object_size), temp_ptr, object_size);
	}
}



/****************************************************************************/
/*                                                                          */
/*      GESTION DES LISTES D'HORAIRES                                       */
/*                                                                          */
/****************************************************************************/



signed int	SOLV_union_availability_elt (SOLV_POINT **point_ptr_ptr, int point)
{
	SOLV_POINT	*point_ptr;
	SOLV_POINT	*new_ptr;

	point_ptr = *point_ptr_ptr;

	while (point_ptr != NULL)
	{
		/* Le point existait deja */
		if (point_ptr->point == point)
		{
			return (0);
		}

		/* On a depasse le nouveau point, il faut l'inserer avant le point courant */
		else if (point_ptr->point > point)
		{
			break;
		}

		/* Point suivant */
		point_ptr_ptr = &point_ptr->next_ptr;
		point_ptr = point_ptr->next_ptr;
	}

	/* Insertion du nouveau point */
	new_ptr = (SOLV_POINT *) malloc (sizeof (*new_ptr));
	if (new_ptr == NULL)
	{
		fprintf (stderr, "Error: malloc failed.\n");
		exit (-1);
	}
	new_ptr->next_ptr = point_ptr;
	new_ptr->point = point;
	*point_ptr_ptr = new_ptr;

	return (0);
}



signed int	SOLV_union_availability_list (SOLV_POINT_WEEK tab_point_1_ptr, SOLV_POINT_WEEK tab_point_2_ptr)
{
	int		day;
	int		half;
	int		point;
	int		insert_flag;
	SOLV_POINT	**point_1_ptr_ptr;
	SOLV_POINT	*point_1_ptr;
	SOLV_POINT	*point_2_ptr;
	SOLV_POINT	*new_ptr;

	for (day = MAIN_first_day; day <= MAIN_last_day; day ++)
	{
		for (half = 0; half < 2; half ++)
		{
			point_1_ptr_ptr = &tab_point_1_ptr [day] [half];
			point_1_ptr = *point_1_ptr_ptr;
			point_2_ptr = tab_point_2_ptr [day] [half];

			/* On boucle jusqu'a ce qu'on ait analyse tous
				les points du tableau 2 */
			while (point_2_ptr != NULL)
			{
				point = point_2_ptr->point;

				/* On se place sur le point du tableau 1 qui est superieur
					ou egal au point courant du tableau 2. Si le tableau 1 est
					termine, on insere systematiquement. */
				insert_flag = TRUE;
				while (point_1_ptr != NULL)
				{
					/* Si les deux points sont egaux, on passe
						au point 2 suivant */
					if (point_1_ptr->point == point)
					{
						insert_flag = FALSE;
						break;
					}

					/* Sinon, si le point 1 est superieur, on insere le point 2
						avant. */
					else if (point_1_ptr->point > point)
					{
						break;
					}

					/* Point 1 suivant */
					point_1_ptr_ptr = &point_1_ptr->next_ptr;
					point_1_ptr = point_1_ptr->next_ptr;
				}

				if (insert_flag)
				{
					new_ptr = (SOLV_POINT *) malloc (sizeof (*new_ptr));
					if (new_ptr == NULL)
					{
						fprintf (stderr, "Error: malloc failed.\n");
						exit (-1);
					}
					new_ptr->next_ptr = point_1_ptr;
					new_ptr->point = point;
					*point_1_ptr_ptr = new_ptr;
					point_1_ptr_ptr = &new_ptr->next_ptr;
				}

				/* Point 2 suivant */
				point_2_ptr = point_2_ptr->next_ptr;
			}
		}
	}

	return (0);
}



signed int	SOLV_intersect_availability_list (SOLV_POINT_WEEK tab_point_1_ptr, SOLV_POINT_WEEK tab_point_2_ptr)
{
	int		day;
	int		half;
	int		point;
	int		suppress_flag;
	SOLV_POINT	**point_1_ptr_ptr;
	SOLV_POINT	*point_1_ptr;
	SOLV_POINT	*point_2_ptr;
	SOLV_POINT	*old_ptr;

	for (day = MAIN_first_day; day <= MAIN_last_day; day ++)
	{
		for (half = 0; half < 2; half ++)
		{
			point_1_ptr_ptr = &tab_point_1_ptr [day] [half];
			point_1_ptr = *point_1_ptr_ptr;
			point_2_ptr = tab_point_2_ptr [day] [half];

			/* Analyse tous les points du tableau 1 */
			while (point_1_ptr != NULL)
			{
				point = point_1_ptr->point;

				/* Recherche le prochain point du tableau 2 qui soit superieur
					ou egal au point courant du tableau 1 */
				suppress_flag = TRUE;
				while (point_2_ptr != NULL)
				{
					/* Les deux points sont egaux, on garde le point */
					if (point_2_ptr->point == point)
					{
						suppress_flag = FALSE;
						break;
					}

					/* Les deux points sont egaux, on garde le point */
					if (point_2_ptr->point >= point)
					{
						break;
					}

					/* Point 2 suivant */
					point_2_ptr = point_2_ptr->next_ptr;
				}

				/* Supprime le point 1 courant */
				if (suppress_flag)
				{
					old_ptr = point_1_ptr;
					point_1_ptr = point_1_ptr->next_ptr;
					*point_1_ptr_ptr = point_1_ptr;
					free (old_ptr);
				}

				/* Ou passe au point 1 suivant */
				else
				{
					point_1_ptr_ptr = &point_1_ptr->next_ptr;
					point_1_ptr = point_1_ptr->next_ptr;
				}
			}
		}
	}

	return (0);
}



signed int	SOLV_create_availability_list (SOLV_POINT_WEEK tab_point_ptr, PARM_PLANNING *(planning [7] [2]), int length, signed int class_nbr, signed int subdiv)
{
	int		day;
	int		half;
	int		point;
	int		end;
	int		validated_length;
	int		nbr_subdiv;
	int		subdiv_2;
	int		nbr_subdiv_2;
	SOLV_POINT	*point_ptr;
	SOLV_POINT	**point_ptr_ptr;
	PARM_PLANNING	*class_ptr;
	PARM_PLANNING	*class_2_ptr;

	if (   class_nbr >= 0
		 && subdiv >= 0)
	{
		nbr_subdiv = MAIN_class_ptr [class_nbr].nbr_subdiv;
	}

	memset (tab_point_ptr, 0, 7 * 2 * sizeof (SOLV_POINT *));

	for (day = MAIN_first_day; day <= MAIN_last_day; day ++)
	{
		for (half = 0; half < 2; half ++)
		{
			point_ptr_ptr = &tab_point_ptr [day] [half];
			class_ptr = planning [day] [half];
			end = (half == 0) ? MAIN_am_length : MAIN_pm_length;
			validated_length = 0;

			/* Jour ferie ou demi-journee reservee */
			if (class_ptr != NULL)
			{
				if (class_ptr->type < 0)
				{
					continue;
				}
			}

			for (point = 0; point < end; point += MAIN_time_granularity)
			{
				/* La fin du cours courant doit etre apres le point courant */
				while (class_ptr != NULL)
				{
					if (point < class_ptr->start + class_ptr->duration)
					{
						break;
					}
					class_ptr = class_ptr->next_ptr;
				}

				/* Si on est dans le cours, on n'ajoute pas de point */
				if (class_ptr != NULL)
				{
					if (point >= class_ptr->start)
					{
						if (   class_nbr < 0
							 || subdiv < 0)
						{
							validated_length = 0;
							goto next_point;
						}
						else
						{
							class_2_ptr = class_ptr;
							while (class_2_ptr != NULL)
							{
								/* On s'arrete de chercher que quand le debut du
									cours courant depasse notre point */
								if (point < class_2_ptr->start)
								{
									break;
								}

								/* Si par contre le cours courant se termine avant
									notre point, on saute ce cours. */
								if (point >= class_2_ptr->start + class_2_ptr->duration)
								{
									class_2_ptr = class_2_ptr->next_ptr;
									continue;
								}

								/* Teste si des eleves de notre groupe ne sont pas
									deja en cours quelque part */
								subdiv_2 = MAX (class_2_ptr->subdiv, 0);
								nbr_subdiv_2 = MAX (class_2_ptr->subdiv_tot, 1);
								if (   (   subdiv * nbr_subdiv_2 >= subdiv_2 * nbr_subdiv
										  && subdiv * nbr_subdiv_2 < (subdiv_2 + 1) * nbr_subdiv)
									 || (   subdiv_2 * nbr_subdiv >= subdiv * nbr_subdiv_2
										  && subdiv_2 * nbr_subdiv < (subdiv + 1) * nbr_subdiv_2))
								{
									validated_length = 0;
									goto next_point;
								}

								/* Cours suivant */
								class_2_ptr = class_2_ptr->next_ptr;
							}
						}
					}
				}

				/* Ajoute cet horaire */
				if (validated_length + MAIN_time_granularity >= length)
				{
					point_ptr = (SOLV_POINT *) malloc (sizeof (*point_ptr));
					if (point_ptr == NULL)
					{
						fprintf (stderr, "Error: malloc failed.\n");
						exit (-1);
					}
					point_ptr->next_ptr = NULL;
					point_ptr->point = point - validated_length;
					*point_ptr_ptr = point_ptr;
					point_ptr_ptr = &point_ptr->next_ptr;
				}

				else
				{
					validated_length += MAIN_time_granularity;
				}

			next_point:	;
			}
		}
	}

	return (0);
}



/* Enleve [start, start+length[ */
signed int	SOLV_substract_availability_list (SOLV_POINT **point_ptr_ptr, int start, int length)
{
	int		end;
	SOLV_POINT	*point_ptr;
	SOLV_POINT	*old_ptr;

	point_ptr = *point_ptr_ptr;
	end = start + length;

	while (point_ptr != NULL)
	{
		/* Retire le point */
		if (   point_ptr->point >= start
			 && point_ptr->point < end)
		{
			old_ptr = point_ptr;
			point_ptr = point_ptr->next_ptr;
			*point_ptr_ptr = point_ptr;
			free (old_ptr);
		}

		/* Passe au point suivant */
		else
		{
			point_ptr_ptr = &point_ptr->next_ptr;
			point_ptr = point_ptr->next_ptr;
		}
	}

	return (0);
}



signed int	SOLV_translate_availability_list (SOLV_POINT_WEEK tab_point_ptr, signed int trans)
{
	int		day;
	int		half;
	int		end;
	SOLV_POINT	*point_ptr;
	SOLV_POINT	**point_ptr_ptr;

	for (day = MAIN_first_day; day <= MAIN_last_day; day ++)
	{
		for (half = 0; half < 2; half ++)
		{
			end = (half == 0) ? MAIN_am_length : MAIN_pm_length;

			point_ptr_ptr = &tab_point_ptr [day] [half];
			point_ptr = *point_ptr_ptr;

			while (point_ptr != NULL)
			{
				point_ptr->point += trans;

				/* Si le point est hors de la demi-journee, on le detruit */
				if (   point_ptr->point < 0
					 || point_ptr->point >= end)
				{
					*point_ptr_ptr = point_ptr->next_ptr;
					free (point_ptr);
					point_ptr = *point_ptr_ptr;
				}

				/* Sinon on passe au suivant */
				else
				{
					point_ptr_ptr = &point_ptr->next_ptr;
					point_ptr = *point_ptr_ptr;
				}
			}
		}
	}

	return (0);
}



signed int	SOLV_delete_availability_list (SOLV_POINT_WEEK tab_point_ptr)
{
	int		day;
	int		half;
	SOLV_POINT	*point_ptr;

	for (day = MAIN_first_day; day <= MAIN_last_day; day ++)
	{
		for (half = 0; half < 2; half ++)
		{
			point_ptr = tab_point_ptr [day] [half];

			while (point_ptr != NULL)
			{
				tab_point_ptr [day] [half] = point_ptr->next_ptr;
				free (point_ptr);
				point_ptr = tab_point_ptr [day] [half];
			}
		}
	}

	memset (tab_point_ptr, 0, 7 * 2 * sizeof (SOLV_POINT *));

	return (0);
}



/****************************************************************************/
/*                                                                          */
/*                                                                          */
/*                                                                          */
/****************************************************************************/



/*==========================================================================*/
/*      Nom:                                                                */
/*      Description:                                                        */
/*      Parametres en entree:                                               */
/*      Parametres en sortie:                                               */
/*      Parametres en entree/sortie:                                        */
/*      Retour:                                                             */
/*==========================================================================*/

